@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2019 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2022 by Paolo Lucente
 */
 
 /*
@@ -32,13 +32,13 @@
 #error "--enable-rabbitmq requires --enable-jansson"
 #endif
 #include "net_aggr.h"
-#include "ports_aggr.h"
 
 /* Functions */
 void amqp_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr) 
 {
   struct pkt_data *data;
   struct ports_table pt;
+  struct protos_table prt, tost;
   unsigned char *pipebuf;
   struct pollfd pfd;
   struct insert_data idata;
@@ -62,8 +62,8 @@ void amqp_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
 
 #ifdef WITH_AVRO
   char *avro_acct_data_schema_str = NULL;
-  char *avro_acct_init_schema_str = NULL;
-  char *avro_acct_close_schema_str = NULL;
+  char *p_avro_acct_init_schema_str = NULL;
+  char *p_avro_acct_close_schema_str = NULL;
 #endif
 
 #ifdef WITH_ZMQ
@@ -72,6 +72,10 @@ void amqp_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   void *zmq_host = NULL;
 #endif
 
+#ifdef WITH_REDIS
+  struct p_redis_host redis_host;
+#endif
+
   memcpy(&config, cfgptr, sizeof(struct configuration));
   memcpy(&extras, &((struct channels_list_entry *)ptr)->extras, sizeof(struct extra_primitives));
   recollect_pipe_memory(ptr);
@@ -96,11 +100,11 @@ void amqp_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   else if ((config.message_broker_output & PRINT_OUTPUT_AVRO_BIN) ||
 	   (config.message_broker_output & PRINT_OUTPUT_AVRO_JSON)) {
 #ifdef WITH_AVRO
-    avro_acct_schema = avro_schema_build_acct_data(config.what_to_count, config.what_to_count_2);
-    avro_schema_add_writer_id(avro_acct_schema);
+    p_avro_acct_schema = p_avro_schema_build_acct_data(config.what_to_count, config.what_to_count_2);
+    p_avro_schema_add_writer_id(p_avro_acct_schema);
 
-    avro_acct_init_schema = avro_schema_build_acct_init();
-    avro_acct_close_schema = avro_schema_build_acct_close();
+    p_avro_acct_init_schema = p_avro_schema_build_acct_init();
+    p_avro_acct_close_schema = p_avro_schema_build_acct_close();
 
     if (config.avro_schema_file) {
       char avro_schema_file[SRVBUFLEN];
@@ -110,9 +114,9 @@ void amqp_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
 	exit_gracefully(1);
       }
 
-      write_avro_schema_to_file_with_suffix(config.avro_schema_file, "-acct_data", avro_schema_file, avro_acct_schema);
-      write_avro_schema_to_file_with_suffix(config.avro_schema_file, "-acct_init", avro_schema_file, avro_acct_init_schema);
-      write_avro_schema_to_file_with_suffix(config.avro_schema_file, "-acct_close", avro_schema_file, avro_acct_close_schema);
+      write_avro_schema_to_file_with_suffix(config.avro_schema_file, "-acct_data", avro_schema_file, p_avro_acct_schema);
+      write_avro_schema_to_file_with_suffix(config.avro_schema_file, "-acct_init", avro_schema_file, p_avro_acct_init_schema);
+      write_avro_schema_to_file_with_suffix(config.avro_schema_file, "-acct_close", avro_schema_file, p_avro_acct_close_schema);
     }
 
     if (config.amqp_avro_schema_routing_key) {
@@ -122,17 +126,17 @@ void amqp_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
 
       avro_schema_deadline = time(NULL);
       P_init_refresh_deadline(&avro_schema_deadline, config.amqp_avro_schema_refresh_time, 0, "m");
-      avro_acct_data_schema_str = compose_avro_purge_schema(avro_acct_schema, config.name);
-      avro_acct_init_schema_str = compose_avro_purge_schema(avro_acct_init_schema, config.name);
-      avro_acct_close_schema_str = compose_avro_purge_schema(avro_acct_close_schema, config.name);
+      avro_acct_data_schema_str = compose_avro_purge_schema(p_avro_acct_schema, config.name);
+      p_avro_acct_init_schema_str = compose_avro_purge_schema(p_avro_acct_init_schema, config.name);
+      p_avro_acct_close_schema_str = compose_avro_purge_schema(p_avro_acct_close_schema, config.name);
     }
     else {
       config.amqp_avro_schema_refresh_time = 0;
       avro_schema_deadline = 0;
       avro_schema_timeout = 0;
       avro_acct_data_schema_str = NULL;
-      avro_acct_init_schema_str = NULL;
-      avro_acct_close_schema_str = NULL;
+      p_avro_acct_init_schema_str = NULL;
+      p_avro_acct_close_schema_str = NULL;
     }
 #endif
   }
@@ -165,11 +169,15 @@ void amqp_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   memset(&nt, 0, sizeof(nt));
   memset(&nc, 0, sizeof(nc));
   memset(&pt, 0, sizeof(pt));
+  memset(&prt, 0, sizeof(prt));
+  memset(&tost, 0, sizeof(tost));
 
   load_networks(config.networks_file, &nt, &nc);
   set_net_funcs(&nt);
 
   if (config.ports_file) load_ports(config.ports_file, &pt);
+  if (config.protos_file) load_protos(config.protos_file, &prt);
+  if (config.tos_file) load_tos(config.tos_file, &tost);
   
   memset(&idata, 0, sizeof(idata));
   memset(&prim_ptrs, 0, sizeof(prim_ptrs));
@@ -195,6 +203,15 @@ void amqp_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   /* setting number of entries in _protocols structure */
   while (_protocols[protocols_number].number != -1) protocols_number++;
 
+#ifdef WITH_REDIS
+  if (config.redis_host) {
+    char log_id[SHORTBUFLEN];
+
+    snprintf(log_id, sizeof(log_id), "%s/%s", config.name, config.type);
+    p_redis_init(&redis_host, log_id, p_redis_thread_produce_common_plugin_handler);
+  }
+#endif
+
   /* plugin main loop */
   for(;;) {
     poll_again:
@@ -221,13 +238,13 @@ void amqp_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
     poll_ops:
     P_update_time_reference(&idata);
 
-    if (idata.now > refresh_deadline) P_cache_handle_flush_event(&pt);
+    if (idata.now > refresh_deadline) P_cache_handle_flush_event(&pt, &prt, &tost);
 
 #ifdef WITH_AVRO
     if (idata.now > avro_schema_deadline) {
       amqp_avro_schema_purge(avro_acct_data_schema_str);
-      amqp_avro_schema_purge(avro_acct_init_schema_str);
-      amqp_avro_schema_purge(avro_acct_close_schema_str);
+      amqp_avro_schema_purge(p_avro_acct_init_schema_str);
+      amqp_avro_schema_purge(p_avro_acct_close_schema_str);
 
       avro_schema_deadline += config.amqp_avro_schema_refresh_time;
     }
@@ -315,10 +332,18 @@ void amqp_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
 	  (*net_funcs[num])(&nt, &nc, &data->primitives, prim_ptrs.pbgp, &nfd);
 
 	if (config.ports_file) {
-          if (!pt.table[data->primitives.src_port]) data->primitives.src_port = 0;
-          if (!pt.table[data->primitives.dst_port]) data->primitives.dst_port = 0;
+          if (!pt.table[data->primitives.src_port]) data->primitives.src_port = PM_L4_PORT_OTHERS;
+          if (!pt.table[data->primitives.dst_port]) data->primitives.dst_port = PM_L4_PORT_OTHERS;
         }
 
+	if (config.protos_file) {
+	  if (!prt.table[data->primitives.proto]) data->primitives.proto = PM_IP_PROTO_OTHERS;
+	}
+
+	if (config.tos_file) {
+	  if (!tost.table[data->primitives.tos]) data->primitives.tos = PM_IP_TOS_OTHERS;
+	}
+
         prim_ptrs.data = data;
         (*insert_func)(&prim_ptrs, &idata);
 
@@ -358,6 +383,7 @@ void amqp_cache_purge(struct chained_cache *queue[], int index, int safe_action)
   struct primitives_ptrs prim_ptrs;
   struct pkt_data dummy_data;
   pid_t writer_pid = getpid();
+  struct dynname_tokens writer_id_tokens;
 
   char *json_buf = NULL;
   int json_buf_off = 0;
@@ -372,10 +398,10 @@ void amqp_cache_purge(struct chained_cache *queue[], int index, int safe_action)
   (void)data;
 
 #ifdef WITH_AVRO
-  avro_writer_t avro_writer = {0};
-  char *avro_buf = NULL;
-  int avro_buffer_full = FALSE;
-  size_t avro_len;
+  avro_writer_t p_avro_writer = {0};
+  char *p_avro_buf = NULL;
+  int p_avro_buffer_full = FALSE;
+  size_t p_avro_len;
 #endif
 
   /* setting some defaults */
@@ -413,6 +439,12 @@ void amqp_cache_purge(struct chained_cache *queue[], int index, int safe_action)
     exit_gracefully(1);
   }
 
+  if (!config.writer_id_string) {
+    config.writer_id_string = DYNNAME_DEFAULT_WRITER_ID;
+  }
+
+  dynname_tokens_prepare(config.writer_id_string, &writer_id_tokens, DYN_STR_WRITER_ID);
+
   p_amqp_init_routing_key_rr(&amqpp_amqp_host);
   p_amqp_set_routing_key_rr(&amqpp_amqp_host, config.amqp_routing_key_rr);
 
@@ -453,18 +485,18 @@ void amqp_cache_purge(struct chained_cache *queue[], int index, int safe_action)
 #ifdef WITH_AVRO
     if (!config.avro_buffer_size) config.avro_buffer_size = LARGEBUFLEN;
 
-    avro_buf = malloc(config.avro_buffer_size);
+    p_avro_buf = malloc(config.avro_buffer_size);
 
-    if (!avro_buf) {
-      Log(LOG_ERR, "ERROR ( %s/%s ): malloc() failed (avro_buf). Exiting ..\n", config.name, config.type);
+    if (!p_avro_buf) {
+      Log(LOG_ERR, "ERROR ( %s/%s ): malloc() failed (p_avro_buf). Exiting ..\n", config.name, config.type);
       exit_gracefully(1);
     }
     else {
-      memset(avro_buf, 0, config.avro_buffer_size);
+      memset(p_avro_buf, 0, config.avro_buffer_size);
     }
 
     if (config.message_broker_output & PRINT_OUTPUT_AVRO_BIN) {
-      avro_writer = avro_writer_memory(avro_buf, config.avro_buffer_size);
+      p_avro_writer = avro_writer_memory(p_avro_buf, config.avro_buffer_size);
     }
 #endif
   }
@@ -488,25 +520,25 @@ void amqp_cache_purge(struct chained_cache *queue[], int index, int safe_action)
     else if ((config.message_broker_output & PRINT_OUTPUT_AVRO_BIN) || 
 	     (config.message_broker_output & PRINT_OUTPUT_AVRO_JSON)) {
 #ifdef WITH_AVRO
-      avro_value_iface_t *avro_iface = avro_generic_class_from_schema(avro_acct_init_schema);
-      avro_value_t avro_value = compose_avro_acct_init(config.name, writer_pid, avro_iface);
+      avro_value_iface_t *p_avro_iface = avro_generic_class_from_schema(p_avro_acct_init_schema);
+      avro_value_t p_avro_value = compose_avro_acct_init(config.name, writer_pid, p_avro_iface);
 
       if (config.message_broker_output & PRINT_OUTPUT_AVRO_BIN) {
-        avro_len = avro_writer_tell(avro_writer);
-        ret = p_amqp_publish_binary(&amqpp_amqp_host, avro_buf, avro_len);
-        avro_writer_reset(avro_writer);
+        p_avro_len = avro_writer_tell(p_avro_writer);
+        ret = p_amqp_publish_binary(&amqpp_amqp_host, p_avro_buf, p_avro_len);
+        avro_writer_reset(p_avro_writer);
       }
       else if (config.message_broker_output & PRINT_OUTPUT_AVRO_JSON) {
-	char *avro_locbuf = write_avro_json_record_to_buf(avro_value);
+	char *p_avro_local_buf = write_avro_json_record_to_buf(p_avro_value);
 
-	if (avro_locbuf) {
-          ret = p_amqp_publish_string(&amqpp_amqp_host, avro_locbuf);
-	  free(avro_locbuf);
+	if (p_avro_local_buf) {
+          ret = p_amqp_publish_string(&amqpp_amqp_host, p_avro_local_buf);
+	  free(p_avro_local_buf);
 	}
       }
 
-      avro_value_decref(&avro_value);
-      avro_value_iface_decref(avro_iface);
+      avro_value_decref(&p_avro_value);
+      avro_value_iface_decref(p_avro_iface);
 #endif
     }
   }
@@ -543,7 +575,7 @@ void amqp_cache_purge(struct chained_cache *queue[], int index, int safe_action)
       int idx;
 
       for (idx = 0; idx < N_PRIMITIVES && cjhandler[idx]; idx++) cjhandler[idx](json_obj, queue[j]);
-      add_writer_name_and_pid_json(json_obj, config.name, writer_pid);
+      add_writer_name_and_pid_json(json_obj, &writer_id_tokens);
 
       json_str = compose_json_str(json_obj);
 #endif
@@ -551,28 +583,28 @@ void amqp_cache_purge(struct chained_cache *queue[], int index, int safe_action)
     else if ((config.message_broker_output & PRINT_OUTPUT_AVRO_BIN) ||
 	     (config.message_broker_output & PRINT_OUTPUT_AVRO_JSON)) {
 #ifdef WITH_AVRO
-      avro_value_iface_t *avro_iface = avro_generic_class_from_schema(avro_acct_schema);
-      avro_value_t avro_value = compose_avro_acct_data(config.what_to_count, config.what_to_count_2,
+      avro_value_iface_t *p_avro_iface = avro_generic_class_from_schema(p_avro_acct_schema);
+      avro_value_t p_avro_value = compose_avro_acct_data(config.what_to_count, config.what_to_count_2,
 			   queue[j]->flow_type, &queue[j]->primitives, pbgp, pnat, pmpls, ptun, pcust,
 			   pvlen, queue[j]->bytes_counter, queue[j]->packet_counter,
-			   queue[j]->flow_counter, queue[j]->tcp_flags, &queue[j]->basetime,
-			   queue[j]->stitch, avro_iface);
-      add_writer_name_and_pid_avro(avro_value, config.name, writer_pid);
+			   queue[j]->flow_counter, queue[j]->tcp_flags, queue[j]->tunnel_tcp_flags,
+			   &queue[j]->basetime, queue[j]->stitch, p_avro_iface);
+      add_writer_name_and_pid_avro_v2(p_avro_value, &writer_id_tokens);
 
       if (config.message_broker_output & PRINT_OUTPUT_AVRO_BIN) {
-	size_t avro_value_size;
+	size_t p_avro_value_size;
 
-	avro_value_sizeof(&avro_value, &avro_value_size);
+	avro_value_sizeof(&p_avro_value, &p_avro_value_size);
 
-	if (avro_value_size > config.avro_buffer_size) {
+	if (p_avro_value_size > config.avro_buffer_size) {
 	  Log(LOG_ERR, "ERROR ( %s/%s ): amqp_cache_purge(): avro_buffer_size too small (%u)\n", config.name, config.type, config.avro_buffer_size);
 	  exit_gracefully(1);
 	}
-	else if (avro_value_size >= (config.avro_buffer_size - avro_writer_tell(avro_writer))) {
-	  avro_buffer_full = TRUE;
+	else if (p_avro_value_size >= (config.avro_buffer_size - avro_writer_tell(p_avro_writer))) {
+	  p_avro_buffer_full = TRUE;
 	  j--;
 	}
-	else if (avro_value_write(avro_writer, &avro_value)) {
+	else if (avro_value_write(p_avro_writer, &p_avro_value)) {
 	  Log(LOG_ERR, "ERROR ( %s/%s ): ARVO: unable to write value: %s\n", config.name, config.type, avro_strerror());
 	  exit_gracefully(1);
 	}
@@ -581,30 +613,30 @@ void amqp_cache_purge(struct chained_cache *queue[], int index, int safe_action)
 	}
       }
       else if (config.message_broker_output & PRINT_OUTPUT_AVRO_JSON) {
-	char *avro_locbuf = write_avro_json_record_to_buf(avro_value);
+	char *p_avro_local_buf = write_avro_json_record_to_buf(p_avro_value);
 
-	if (avro_locbuf) {
-	  size_t avro_locbuf_len = strlen(avro_locbuf);
+	if (p_avro_local_buf) {
+	  size_t p_avro_local_buf_len = strlen(p_avro_local_buf);
 
-	  if (avro_locbuf_len > config.avro_buffer_size) {
+	  if (p_avro_local_buf_len > config.avro_buffer_size) {
 	    Log(LOG_ERR, "ERROR ( %s/%s ): amqp_cache_purge(): avro_buffer_size too small (%u)\n", config.name, config.type, config.avro_buffer_size);
 	    exit_gracefully(1);
 	  }
-	  else if (avro_locbuf_len >= (config.avro_buffer_size - strlen(avro_buf))) {
-	    avro_buffer_full = TRUE;
+	  else if (p_avro_local_buf_len >= (config.avro_buffer_size - strlen(p_avro_buf))) {
+	    p_avro_buffer_full = TRUE;
 	    j--;
 	  }
 	  else {
-	    strcat(avro_buf, avro_locbuf);
+	    strcat(p_avro_buf, p_avro_local_buf);
 	    mv_num++;
 	  }
 
-	  free(avro_locbuf);
+	  free(p_avro_local_buf);
 	}
       }
 
-      avro_value_decref(&avro_value);
-      avro_value_iface_decref(avro_iface);
+      avro_value_decref(&p_avro_value);
+      avro_value_iface_decref(p_avro_iface);
 #else
       if (config.debug) Log(LOG_DEBUG, "DEBUG ( %s/%s ): compose_avro_acct_data(): AVRO object not created due to missing --enable-avro\n", config.name, config.type);
 #endif
@@ -678,7 +710,7 @@ void amqp_cache_purge(struct chained_cache *queue[], int index, int safe_action)
     else if ((config.message_broker_output & PRINT_OUTPUT_AVRO_BIN) || 
 	     (config.message_broker_output & PRINT_OUTPUT_AVRO_JSON)) {
 #ifdef WITH_AVRO
-      if (!config.sql_multi_values || (mv_num >= config.sql_multi_values) || avro_buffer_full) {
+      if (!config.sql_multi_values || (mv_num >= config.sql_multi_values) || p_avro_buffer_full) {
         if (is_routing_key_dyn) {
           prim_ptrs.data = &dummy_data;
           primptrs_set_all_from_chained_cache(&prim_ptrs, queue[j]);
@@ -693,15 +725,15 @@ void amqp_cache_purge(struct chained_cache *queue[], int index, int safe_action)
         }
 
 	if (config.message_broker_output & PRINT_OUTPUT_AVRO_BIN) {
-          ret = p_amqp_publish_binary(&amqpp_amqp_host, avro_buf, avro_writer_tell(avro_writer));
-          avro_writer_reset(avro_writer);
+          ret = p_amqp_publish_binary(&amqpp_amqp_host, p_avro_buf, avro_writer_tell(p_avro_writer));
+          avro_writer_reset(p_avro_writer);
 	}
 	else if (config.message_broker_output & PRINT_OUTPUT_AVRO_JSON) {
-	  ret = p_amqp_publish_string(&amqpp_amqp_host, avro_buf);
-	  memset(avro_buf, 0, config.avro_buffer_size);
+	  ret = p_amqp_publish_string(&amqpp_amqp_host, p_avro_buf);
+	  memset(p_avro_buf, 0, config.avro_buffer_size);
 	}
 
-        avro_buffer_full = FALSE;
+        p_avro_buffer_full = FALSE;
         mv_num_save = mv_num;
         mv_num = 0;
 
@@ -726,15 +758,15 @@ void amqp_cache_purge(struct chained_cache *queue[], int index, int safe_action)
 	     (config.message_broker_output & PRINT_OUTPUT_AVRO_JSON)) {
 #ifdef WITH_AVRO
       if (config.message_broker_output & PRINT_OUTPUT_AVRO_BIN) {
-        if (avro_writer_tell(avro_writer)) {
-	  ret = p_amqp_publish_binary(&amqpp_amqp_host, avro_buf, avro_writer_tell(avro_writer));
-	  avro_writer_free(avro_writer);
+        if (avro_writer_tell(p_avro_writer)) {
+	  ret = p_amqp_publish_binary(&amqpp_amqp_host, p_avro_buf, avro_writer_tell(p_avro_writer));
+	  avro_writer_free(p_avro_writer);
 	  if (!ret) qn += mv_num;
 	}
       }
       else if (config.message_broker_output & PRINT_OUTPUT_AVRO_JSON) {
-	if (strlen(avro_buf)) {
-	  ret = p_amqp_publish_string(&amqpp_amqp_host, avro_buf);
+	if (strlen(p_avro_buf)) {
+	  ret = p_amqp_publish_string(&amqpp_amqp_host, p_avro_buf);
           if (!ret) qn += mv_num;
 	}
       }
@@ -763,25 +795,25 @@ void amqp_cache_purge(struct chained_cache *queue[], int index, int safe_action)
     else if ((config.message_broker_output & PRINT_OUTPUT_AVRO_BIN) || 
 	     (config.message_broker_output & PRINT_OUTPUT_AVRO_JSON)) {
 #ifdef WITH_AVRO
-      avro_value_iface_t *avro_iface = avro_generic_class_from_schema(avro_acct_close_schema);
-      avro_value_t avro_value = compose_avro_acct_close(config.name, writer_pid, qn, saved_index, duration, avro_iface);
+      avro_value_iface_t *p_avro_iface = avro_generic_class_from_schema(p_avro_acct_close_schema);
+      avro_value_t p_avro_value = compose_avro_acct_close(config.name, writer_pid, qn, saved_index, duration, p_avro_iface);
 
       if (config.message_broker_output & PRINT_OUTPUT_AVRO_BIN) {
-        avro_len = avro_writer_tell(avro_writer);
-        ret = p_amqp_publish_binary(&amqpp_amqp_host, avro_buf, avro_len);
-        avro_writer_reset(avro_writer);
+        p_avro_len = avro_writer_tell(p_avro_writer);
+        ret = p_amqp_publish_binary(&amqpp_amqp_host, p_avro_buf, p_avro_len);
+        avro_writer_reset(p_avro_writer);
       }
       else if (config.message_broker_output & PRINT_OUTPUT_AVRO_JSON) {
-	char *avro_locbuf = write_avro_json_record_to_buf(avro_value);
+	char *p_avro_local_buf = write_avro_json_record_to_buf(p_avro_value);
 
-	if (avro_locbuf) {
-	  ret = p_amqp_publish_string(&amqpp_amqp_host, avro_locbuf);
-	  free(avro_locbuf);
+	if (p_avro_local_buf) {
+	  ret = p_amqp_publish_string(&amqpp_amqp_host, p_avro_local_buf);
+	  free(p_avro_local_buf);
 	}
       }
 
-      avro_value_decref(&avro_value);
-      avro_value_iface_decref(avro_iface);
+      avro_value_decref(&p_avro_value);
+      avro_value_iface_decref(p_avro_iface);
 #endif
     }
   }
@@ -798,17 +830,17 @@ void amqp_cache_purge(struct chained_cache *queue[], int index, int safe_action)
   if (json_buf) free(json_buf);
 
 #ifdef WITH_AVRO
-  if (avro_buf) free(avro_buf);
+  if (p_avro_buf) free(p_avro_buf);
 #endif
 }
 
 #ifdef WITH_AVRO
-void amqp_avro_schema_purge(char *avro_schema_str)
+void amqp_avro_schema_purge(char *p_avro_schema_str)
 {
   struct p_amqp_host amqp_avro_schema_host;
   int ret;
 
-  if (!avro_schema_str || !config.amqp_avro_schema_routing_key) return;
+  if (!p_avro_schema_str || !config.amqp_avro_schema_routing_key) return;
 
   /* setting some defaults */
   if (!config.sql_host) config.sql_host = default_amqp_host;
@@ -831,7 +863,7 @@ void amqp_avro_schema_purge(char *avro_schema_str)
   ret = p_amqp_connect_to_publish(&amqp_avro_schema_host);
   if (ret) return;
 
-  ret = p_amqp_publish_string(&amqp_avro_schema_host, avro_schema_str);
+  ret = p_amqp_publish_string(&amqp_avro_schema_host, p_avro_schema_str);
 
   p_amqp_close(&amqp_avro_schema_host, FALSE);
 }
