@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2016 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2019 by Paolo Lucente
 */
 
 /*
@@ -18,39 +18,26 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
+#ifndef AMQP_PLUGIN_COMMON_H
+#define AMQP_PLUGIN_COMMON_H
 
 /* includes */
 #include <amqp.h>
 #include <amqp_tcp_socket.h>
 #include <sys/poll.h>
 
+#include "plugin_common.h"
+
 /* defines */
 
 /* structures */
 
 /* prototypes */
-#if (!defined __AMQP_PLUGIN_C)
-#define EXT extern
-#else
-#define EXT
-#endif
-EXT void amqp_plugin(int, struct configuration *, void *);
-EXT void amqp_cache_purge(struct chained_cache *[], int);
-
-/* global vars */
-EXT void (*insert_func)(struct primitives_ptrs *, struct insert_data *); /* pointer to INSERT function */
-EXT void (*purge_func)(struct chained_cache *[], int); /* pointer to purge function */ 
-EXT struct scratch_area sa;
-EXT struct chained_cache *cache;
-EXT struct chained_cache **queries_queue;
-EXT struct timeval flushtime;
-EXT int qq_ptr, pp_size, pb_size, pn_size, pm_size, dbc_size, quit; 
-EXT time_t refresh_deadline;
-
-EXT struct timeval sbasetime;
-
+extern void p_amqp_get_version();
+extern void amqp_plugin(int, struct configuration *, void *);
+extern void amqp_cache_purge(struct chained_cache *[], int, int);
 #ifdef WITH_AVRO
-EXT char *avro_buf;
-EXT avro_schema_t avro_acct_schema;
+extern void amqp_avro_schema_purge(char *);
 #endif
-#undef EXT
+
+#endif //AMQP_PLUGIN_COMMON_H
