@@ -1,6 +1,6 @@
 /*  
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2021 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2022 by Paolo Lucente
 */
 
 /*
@@ -21,7 +21,6 @@
 
 /* includes */
 #include "pmacct.h"
-#include "addr.h"
 #include "bgp.h"
 #include "bgp_blackhole.h"
 
@@ -233,7 +232,7 @@ int bgp_parse_open_msg(struct bgp_msg_data *bmd, char *bgp_packet_ptr, time_t no
 
 	      if (cap_len > optcap_len) {
                 bgp_peer_print(peer, bgp_peer_str, INET6_ADDRSTRLEN);
-		Log(LOG_INFO, "INFO ( %s/%s ): [%s] Received malformed BGP Open packet (malformed capability: %x).\n",
+		Log(LOG_INFO, "INFO ( %s/%s ): [%s] Received malformed BGP Open packet (malformed capability: %u).\n",
 			config.name, bms->log_str, bgp_peer_str, cap_type);
 		return ERR;
    	      }
@@ -246,7 +245,7 @@ int bgp_parse_open_msg(struct bgp_msg_data *bmd, char *bgp_packet_ptr, time_t no
 					  
 		if (online) {
                   bgp_peer_print(peer, bgp_peer_str, INET6_ADDRSTRLEN);
-	  	  Log(LOG_INFO, "INFO ( %s/%s ): [%s] Capability: MultiProtocol [%x] AFI [%x] SAFI [%x]\n",
+	  	  Log(LOG_INFO, "INFO ( %s/%s ): [%s] Capability: MultiProtocol [%u] AFI [%u] SAFI [%u]\n",
 			config.name, bms->log_str, bgp_peer_str, cap_type, ntohs(cap_data.afi), cap_data.safi);
 		}
 		peer->cap_mp = TRUE;
@@ -267,7 +266,7 @@ int bgp_parse_open_msg(struct bgp_msg_data *bmd, char *bgp_packet_ptr, time_t no
 
 		  if (online) {
                     bgp_peer_print(peer, bgp_peer_str, INET6_ADDRSTRLEN);
-		    Log(LOG_INFO, "INFO ( %s/%s ): [%s] Capability: 4-bytes AS [%x] ASN [%u]\n",
+		    Log(LOG_INFO, "INFO ( %s/%s ): [%s] Capability: 4-bytes AS [%u] ASN [%u]\n",
 	    		config.name, bms->log_str, bgp_peer_str, cap_type, ntohl(cap_data.as4));
 		  }
 		  memcpy(&as4_ptr, cap_ptr, 4);
@@ -288,37 +287,52 @@ int bgp_parse_open_msg(struct bgp_msg_data *bmd, char *bgp_packet_ptr, time_t no
 		}
 	      }
 	      else if (cap_type == BGP_CAPABILITY_ADD_PATHS) {
-		char *cap_ptr = (optcap_ptr + 2);
-		struct capability_add_paths cap_data;
+		if (!bms->cap_add_path_ignore) {
+		  char *cap_ptr = (optcap_ptr + 2);
+		  struct capability_add_paths cap_data;
 
-		int cap_set = FALSE;
-		u_int8_t *cap_newlen_ptr; /* ptr to field of outbound cap_len */
-		int cap_idx;
+		  int cap_set = FALSE;
+		  u_int8_t *cap_newlen_ptr; /* ptr to field of outbound cap_len */
+		  int cap_idx;
 		
-		/* check for each AFI.SAFI included if remote can send - only then reply with receive */
-		for (cap_idx = 0; cap_idx < cap_len; cap_idx += sizeof(struct capability_add_paths)) {
-		  memcpy(&cap_data, cap_ptr+cap_idx, sizeof(cap_data));
-		  if (online) {
-		    bgp_peer_print(peer, bgp_peer_str, INET6_ADDRSTRLEN);
-		    Log(LOG_INFO, "INFO ( %s/%s ): [%s] Capability: ADD-PATHs [%x] AFI [%x] SAFI [%x] SEND_RECEIVE [%x]\n",
-		  	config.name, bms->log_str, bgp_peer_str, cap_type, ntohs(cap_data.afi), cap_data.safi,
-			cap_data.sndrcv);
-		  }
-		  if ((cap_data.sndrcv == 2 /* send */) || (cap_data.sndrcv == 3 /* send and receive */)) {
-		    peer->cap_add_paths[ntohs(cap_data.afi)][cap_data.safi] = TRUE; 
+		  /* check for each AFI.SAFI included if remote can send - only then reply with receive */
+		  for (cap_idx = 0; cap_idx < cap_len; cap_idx += sizeof(struct capability_add_paths)) {
+		    memcpy(&cap_data, cap_ptr+cap_idx, sizeof(cap_data));
 		    if (online) {
-		      if (!cap_set) {
-			/* we need to send BGP_CAPABILITY_ADD_PATHS first */
-			cap_set = TRUE;
-			memcpy(bgp_open_cap_reply_ptr, optcap_ptr, 2);
-			cap_newlen_ptr = (u_char *)(bgp_open_cap_reply_ptr + 1);
-			(*cap_newlen_ptr) = 0;
-			bgp_open_cap_reply_ptr += 2;
+		      bgp_peer_print(peer, bgp_peer_str, INET6_ADDRSTRLEN);
+		      Log(LOG_INFO, "INFO ( %s/%s ): [%s] Capability: ADD-PATHs [%u] AFI [%u] SAFI [%u] SEND_RECEIVE [%u]\n",
+		    	  config.name, bms->log_str, bgp_peer_str, cap_type, ntohs(cap_data.afi), cap_data.safi,
+			  cap_data.sndrcv);
+		    }
+		    if ((cap_data.sndrcv == 2 /* send */) || (cap_data.sndrcv == 3 /* send and receive */)) {
+		      afi_t cap_add_paths_afi = ntohs(cap_data.afi);
+
+		      if (cap_add_paths_afi < AFI_MAX && cap_data.safi < SAFI_MAX) {
+		        peer->cap_add_paths.cap[cap_add_paths_afi][cap_data.safi] = TRUE;
+
+		        if (cap_add_paths_afi > peer->cap_add_paths.afi_max) {
+		          peer->cap_add_paths.afi_max = cap_add_paths_afi;
+		        }
+
+		        if (cap_data.safi > peer->cap_add_paths.safi_max) {
+		          peer->cap_add_paths.safi_max = cap_data.safi;
+		        }
+		      }
+
+		      if (online) {
+		        if (!cap_set) {
+			  /* we need to send BGP_CAPABILITY_ADD_PATHS first */
+			  cap_set = TRUE;
+			  memcpy(bgp_open_cap_reply_ptr, optcap_ptr, 2);
+			  cap_newlen_ptr = (u_char *)(bgp_open_cap_reply_ptr + 1);
+			  (*cap_newlen_ptr) = 0;
+			  bgp_open_cap_reply_ptr += 2;
+		        }
+		        cap_data.sndrcv = 1; /* we can receive */
+		        (*cap_newlen_ptr) += sizeof(cap_data);
+		        memcpy(bgp_open_cap_reply_ptr, &cap_data, sizeof(cap_data));
+		        bgp_open_cap_reply_ptr += sizeof(cap_data);
 		      }
-		      cap_data.sndrcv = 1; /* we can receive */
-		      (*cap_newlen_ptr) += sizeof(cap_data);
-		      memcpy(bgp_open_cap_reply_ptr, &cap_data, sizeof(cap_data));
-		      bgp_open_cap_reply_ptr += sizeof(cap_data);
 		    }
 		  }
 	        }
@@ -326,7 +340,7 @@ int bgp_parse_open_msg(struct bgp_msg_data *bmd, char *bgp_packet_ptr, time_t no
 	      else if (cap_type == BGP_CAPABILITY_ROUTE_REFRESH) {
 		if (config.tmp_bgp_daemon_route_refresh) {
 		  bgp_peer_print(peer, bgp_peer_str, INET6_ADDRSTRLEN);
-		  Log(LOG_INFO, "INFO ( %s/%s ): [%s] Capability: Route Refresh [%x]\n",
+		  Log(LOG_INFO, "INFO ( %s/%s ): [%s] Capability: Route Refresh [%u]\n",
 		      config.name, bms->log_str, bgp_peer_str, cap_type);
 
 		  memcpy(bgp_open_cap_reply_ptr, optcap_ptr, cap_len+2); 
@@ -737,8 +751,39 @@ int bgp_parse_update_msg(struct bgp_msg_data *bmd, char *pkt)
     parsed = TRUE;
   }
 
-  /* Receipt of End-of-RIB can be processed here; being a silent
-	 BGP receiver only, honestly it doesn't matter to us */
+  /* Checking receipt of End-of-RIB */
+  if (!update_len && !withdraw_len) {
+    int eor = FALSE;
+    afi_t afi = FALSE;
+    safi_t safi = FALSE;
+
+    if (!attribute_len) {
+      eor = TRUE;
+
+      afi = AFI_IP;
+      safi = SAFI_UNICAST;
+    }
+    else if (!mp_withdraw.length && mp_withdraw.afi && mp_withdraw.safi && !parsed) {
+      eor = TRUE;
+
+      afi = mp_withdraw.afi;
+      safi = mp_withdraw.safi;
+    }
+
+    if (eor) {
+      char event_type[] = "log";
+      struct bgp_info ri;
+
+      bgp_peer_print(peer, bgp_peer_str, INET6_ADDRSTRLEN);
+      Log(LOG_DEBUG, "DEBUG ( %s/%s ): [%s] bgp_parse_update_msg() Received unsupported NLRI afi=%u safi=%u\n",
+	  config.name, bms->log_str, bgp_peer_str, afi, safi);
+
+      memset(&ri, 0, sizeof(ri));
+      ri.peer = peer;
+      ri.bmed = bmd->extra;
+      bgp_peer_log_msg(NULL, &ri, afi, safi, bms->tag, event_type, bms->msglog_output, NULL, BGP_LOG_TYPE_EOR);
+    }
+  }
 
   if ((update_len || withdraw_len ||  mp_update.length || mp_withdraw.length) && !parsed) {
     bgp_peer_print(peer, bgp_peer_str, INET6_ADDRSTRLEN);
@@ -954,10 +999,15 @@ int bgp_attr_parse_origin(struct bgp_peer *peer, u_int16_t len, struct bgp_attr
 
 int bgp_attr_parse_mp_reach(struct bgp_peer *peer, u_int16_t len, struct bgp_attr *attr, char *ptr, struct bgp_nlri *mp_update)
 {
+  struct bgp_misc_structs *bms;
   u_int16_t afi, tmp16, mpreachlen, mpnhoplen;
   u_int16_t nlri_len;
   u_char safi;
 
+  bms = bgp_select_misc_db(peer->type);
+
+  if (!bms) return ERR;
+
   /* length check */
 #define BGP_MP_REACH_MIN_SIZE 5
   if (len < BGP_MP_REACH_MIN_SIZE) return ERR;
@@ -1006,7 +1056,13 @@ int bgp_attr_parse_mp_reach(struct bgp_peer *peer, u_int16_t len, struct bgp_att
     }
     else return ERR;
   }
-  else return ERR;
+  else {
+    char bgp_peer_str[INET6_ADDRSTRLEN];
+
+    bgp_peer_print(peer, bgp_peer_str, INET6_ADDRSTRLEN);
+    Log(LOG_DEBUG, "DEBUG ( %s/%s ): [%s] bgp_attr_parse_mp_reach(): Received malformed or unsupported afi=%u safi=%u\n", config.name, bms->log_str, bgp_peer_str, afi, safi);
+    return ERR;
+  }
 
   nlri_len = mpreachlen;
 
@@ -1079,7 +1135,7 @@ int bgp_nlri_parse(struct bgp_msg_data *bmd, void *attr, struct bgp_attr_extra *
 
   for (idx = 0; pnt < lim; pnt += psize, idx++) {
     /* handle path identifier */
-    if (peer->cap_add_paths[info->afi][info->safi]) {
+    if (peer->cap_add_paths.cap[info->afi][info->safi]) {
       memcpy(&attr_extra->path_id, pnt, 4);
       attr_extra->path_id = ntohl(attr_extra->path_id);
       pnt += 4;
@@ -1324,7 +1380,7 @@ int bgp_process_update(struct bgp_msg_data *bmd, struct prefix *p, void *attr, s
 	  else continue;
         }
 
-        if (peer->cap_add_paths[afi][safi]) {
+        if (peer->cap_add_paths.cap[afi][safi]) {
 	  if (ri->attr_extra && (attr_extra->path_id == ri->attr_extra->path_id));
 	  else continue;
         }
@@ -1412,7 +1468,7 @@ log_update:
   {
     char event_type[] = "log";
 
-    bgp_peer_log_msg(route, ri, afi, safi, event_type, bms->msglog_output, NULL, BGP_LOG_TYPE_UPDATE);
+    bgp_peer_log_msg(route, ri, afi, safi, bms->tag, event_type, bms->msglog_output, NULL, BGP_LOG_TYPE_UPDATE);
   }
 
   if (bms->skip_rib) {
@@ -1454,7 +1510,7 @@ int bgp_process_withdraw(struct bgp_msg_data *bmd, struct prefix *p, void *attr,
           else continue;
         }
 
-        if (peer->cap_add_paths[afi][safi]) {
+        if (peer->cap_add_paths.cap[afi][safi]) {
           if (ri->attr_extra && (attr_extra->path_id == ri->attr_extra->path_id));
           else continue;
 	}
@@ -1486,7 +1542,7 @@ int bgp_process_withdraw(struct bgp_msg_data *bmd, struct prefix *p, void *attr,
   if (ri && bms->msglog_backend_methods) {
     char event_type[] = "log";
 
-    bgp_peer_log_msg(route, ri, afi, safi, event_type, bms->msglog_output, NULL, BGP_LOG_TYPE_WITHDRAW);
+    bgp_peer_log_msg(route, ri, afi, safi, bms->tag, event_type, bms->msglog_output, NULL, BGP_LOG_TYPE_WITHDRAW);
   }
 
   if (!bms->skip_rib) {
