@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2021 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2022 by Paolo Lucente
 */
 
 /*
@@ -45,7 +45,9 @@ extern int cfg_key_aggregate(char *, char *, char *);
 extern int cfg_key_aggregate_primitives(char *, char *, char *);
 extern int cfg_key_snaplen(char *, char *, char *);
 extern int cfg_key_aggregate_filter(char *, char *, char *);
+extern int cfg_key_aggregate_unknown_etype(char *, char *, char *);
 extern int cfg_key_dtls_path(char *, char *, char *);
+extern int cfg_key_writer_id_string(char *, char *, char *);
 extern int cfg_key_pcap_filter(char *, char *, char *);
 extern int cfg_key_pcap_protocol(char *, char *, char *);
 extern int cfg_key_pcap_savefile(char *, char *, char *);
@@ -138,6 +140,8 @@ extern int cfg_key_networks_file_no_lpm(char *, char *, char *);
 extern int cfg_key_networks_no_mask_if_zero(char *, char *, char *);
 extern int cfg_key_networks_cache_entries(char *, char *, char *);
 extern int cfg_key_ports_file(char *, char *, char *);
+extern int cfg_key_protos_file(char *, char *, char *);
+extern int cfg_key_tos_file(char *, char *, char *);
 extern int cfg_key_print_cache_entries(char *, char *, char *);
 extern int cfg_key_print_markers(char *, char *, char *);
 extern int cfg_key_print_output(char *, char *, char *);
@@ -151,7 +155,9 @@ extern int cfg_key_print_output_custom_cfg_file(char *, char *, char *);
 extern int cfg_key_print_latest_file(char *, char *, char *);
 extern int cfg_key_nfacctd_port(char *, char *, char *);
 extern int cfg_key_nfacctd_ip(char *, char *, char *);
+extern int cfg_key_nfacctd_interface(char *, char *, char *);
 extern int cfg_key_nfacctd_ipv6_only(char *, char *, char *);
+extern int cfg_key_nfacctd_rp_ebpf_prog(char *, char *, char *);
 extern int cfg_key_nfacctd_allow_file(char *, char *, char *);
 extern int cfg_key_nfacctd_time_secs(char *, char *, char *);
 extern int cfg_key_nfacctd_time_new(char *, char *, char *);
@@ -173,6 +179,7 @@ extern int cfg_key_nfacctd_kafka_topic(char *, char *, char *);
 extern int cfg_key_nfacctd_kafka_config_file(char *, char *, char *);
 extern int cfg_key_nfacctd_zmq_address(char *, char *, char *);
 extern int cfg_key_nfacctd_dtls_port(char *, char *, char *);
+extern int cfg_key_mpls_label_stack_encode_as_array(char *, char *, char *);
 extern int cfg_key_pmacctd_force_frag_handling(char *, char *, char *);
 extern int cfg_key_pmacctd_frag_buffer_size(char *, char *, char *);
 extern int cfg_key_pmacctd_flow_buffer_size(char *, char *, char *);
@@ -207,9 +214,13 @@ extern int cfg_key_telemetry_daemon(char *, char *, char *);
 extern int cfg_key_telemetry_port_tcp(char *, char *, char *);
 extern int cfg_key_telemetry_port_udp(char *, char *, char *);
 extern int cfg_key_telemetry_ip(char *, char *, char *);
+extern int cfg_key_telemetry_interface(char *, char *, char *);
 extern int cfg_key_telemetry_udp_notif_port(char *, char *, char *);
 extern int cfg_key_telemetry_udp_notif_ip(char *, char *, char *);
+extern int cfg_key_telemetry_udp_notif_interface(char *, char *, char *);
+extern int cfg_key_telemetry_udp_notif_ipv6_only(char *, char *, char *);
 extern int cfg_key_telemetry_udp_notif_nmsgs(char *, char *, char *);
+extern int cfg_key_telemetry_udp_notif_rp_ebpf_prog(char *, char *, char *);
 extern int cfg_key_telemetry_ipv6_only(char *, char *, char *);
 extern int cfg_key_telemetry_zmq_address(char *, char *, char *);
 extern int cfg_key_telemetry_kafka_broker_host(char *, char *, char *);
@@ -219,6 +230,7 @@ extern int cfg_key_telemetry_kafka_config_file(char *, char *, char *);
 extern int cfg_key_telemetry_decoder(char *, char *, char *);
 extern int cfg_key_telemetry_max_peers(char *, char *, char *);
 extern int cfg_key_telemetry_peer_timeout(char *, char *, char *);
+extern int cfg_key_telemetry_tag_map(char *, char *, char *);
 extern int cfg_key_telemetry_allow_file(char *, char *, char *);
 extern int cfg_key_telemetry_pipe_size(char *, char *, char *);
 extern int cfg_key_telemetry_ip_precedence(char *, char *, char *);
@@ -248,6 +260,7 @@ extern int cfg_key_telemetry_dump_output(char *, char *, char *);
 extern int cfg_key_telemetry_dump_file(char *, char *, char *);
 extern int cfg_key_telemetry_dump_latest_file(char *, char *, char *);
 extern int cfg_key_telemetry_dump_refresh_time(char *, char *, char *);
+extern int cfg_key_telemetry_dump_time_slots(char *, char *, char *);
 extern int cfg_key_telemetry_dump_amqp_host(char *, char *, char *);
 extern int cfg_key_telemetry_dump_amqp_vhost(char *, char *, char *);
 extern int cfg_key_telemetry_dump_amqp_user(char *, char *, char *);
@@ -275,13 +288,14 @@ extern int cfg_key_pre_tag_map(char *, char *, char *);
 extern int cfg_key_pre_tag_filter(char *, char *, char *);
 extern int cfg_key_pre_tag2_filter(char *, char *, char *);
 extern int cfg_key_pre_tag_label_filter(char *, char *, char *);
+extern int cfg_key_pre_tag_label_encode_as_map(char *, char *, char *);
+extern int cfg_key_tcpflags_encode_as_array(char *, char *, char *);
+extern int cfg_key_fwd_status_encode_as_string(char *, char *, char *);
+extern int cfg_key_tos_encode_as_dscp(char *, char *, char *);
 extern int cfg_key_post_tag(char *, char *, char *);
 extern int cfg_key_post_tag2(char *, char *, char *);
 extern int cfg_key_sampling_rate(char *, char *, char *);
 extern int cfg_key_sampling_map(char *, char *, char *);
-extern int cfg_key_classifiers(char *, char *, char *);
-extern int cfg_key_classifier_tentatives(char *, char *, char *);
-extern int cfg_key_classifier_table_num(char *, char *, char *);
 extern int cfg_key_classifier_ndpi_num_roots(char *, char *, char *);
 extern int cfg_key_classifier_ndpi_max_flows(char *, char *, char *);
 extern int cfg_key_classifier_ndpi_proto_guess(char *, char *, char *);
@@ -344,12 +358,16 @@ extern int cfg_key_bgp_daemon_msglog_kafka_config_file(char *, char *, char *);
 extern int cfg_key_bgp_daemon_msglog_kafka_avro_schema_registry(char *, char *, char *);
 extern int cfg_key_bgp_daemon_max_peers(char *, char *, char *);
 extern int cfg_key_bgp_daemon_ip(char *, char *, char *);
+extern int cfg_key_bgp_daemon_interface(char *, char *, char *);
 extern int cfg_key_bgp_daemon_ipv6_only(char *, char *, char *);
 extern int cfg_key_bgp_daemon_id(char *, char *, char *);
 extern int cfg_key_bgp_daemon_as(char *, char *, char *);
 extern int cfg_key_bgp_daemon_port(char *, char *, char *);
+extern int cfg_key_bgp_daemon_rp_ebpf_prog(char *, char *, char *);
 extern int cfg_key_bgp_daemon_ip_precedence(char *, char *, char *);
 extern int cfg_key_bgp_daemon_allow_file(char *, char *, char *);
+extern int cfg_key_bgp_daemon_add_path_ignore(char *, char *, char *);
+extern int cfg_key_bgp_daemon_tag_map(char *, char *, char *);
 extern int cfg_key_bgp_daemon_aspath_radius(char *, char *, char *);
 extern int cfg_key_bgp_daemon_stdcomm_pattern(char *, char *, char *);
 extern int cfg_key_bgp_daemon_extcomm_pattern(char *, char *, char *);
@@ -385,6 +403,7 @@ extern int cfg_key_bgp_daemon_table_dump_file(char *, char *, char *);
 extern int cfg_key_bgp_daemon_table_dump_latest_file(char *, char *, char *);
 extern int cfg_key_bgp_daemon_table_dump_avro_schema_file(char *, char *, char *);
 extern int cfg_key_bgp_daemon_table_dump_refresh_time(char *, char *, char *);
+extern int cfg_key_bgp_daemon_table_dump_time_slots(char *, char *, char *);
 extern int cfg_key_bgp_daemon_table_dump_amqp_host(char *, char *, char *);
 extern int cfg_key_bgp_daemon_table_dump_amqp_vhost(char *, char *, char *);
 extern int cfg_key_bgp_daemon_table_dump_amqp_user(char *, char *, char *);
@@ -417,8 +436,11 @@ extern int cfg_key_bgp_lg_passwd(char *, char *, char *);
 extern int cfg_key_bgp_xconnect_map(char *, char *, char *);
 extern int cfg_key_bmp_daemon(char *, char *, char *);
 extern int cfg_key_bmp_daemon_ip(char *, char *, char *);
+extern int cfg_key_bmp_daemon_interface(char *, char *, char *);
 extern int cfg_key_bmp_daemon_ipv6_only(char *, char *, char *);
 extern int cfg_key_bmp_daemon_port(char *, char *, char *);
+extern int cfg_key_bmp_daemon_rp_ebpf_prog(char *, char *, char *);
+extern int cfg_key_bmp_daemon_tag_map(char *, char *, char *);
 extern int cfg_key_bmp_daemon_pipe_size(char *, char *, char *);
 extern int cfg_key_bmp_daemon_max_peers(char *, char *, char *);
 extern int cfg_key_bmp_daemon_allow_file(char *, char *, char *);
@@ -459,6 +481,7 @@ extern int cfg_key_bmp_daemon_dump_file(char *, char *, char *);
 extern int cfg_key_bmp_daemon_dump_avro_schema_file(char *, char *, char *);
 extern int cfg_key_bmp_daemon_dump_latest_file(char *, char *, char *);
 extern int cfg_key_bmp_daemon_dump_refresh_time(char *, char *, char *);
+extern int cfg_key_bmp_daemon_dump_time_slots(char *, char *, char *);
 extern int cfg_key_bmp_daemon_dump_amqp_host(char *, char *, char *);
 extern int cfg_key_bmp_daemon_dump_amqp_vhost(char *, char *, char *);
 extern int cfg_key_bmp_daemon_dump_amqp_user(char *, char *, char *);
@@ -505,8 +528,12 @@ extern int cfg_key_tmp_asa_bi_flow(char *, char *, char *);
 extern int cfg_key_tmp_bgp_lookup_compare_ports(char *, char *, char *);
 extern int cfg_key_tmp_bgp_daemon_route_refresh(char *, char *, char *);
 extern int cfg_key_tmp_bgp_daemon_origin_type_int(char *, char *, char *);
+extern int cfg_key_tmp_telemetry_daemon_udp_notif_legacy(char *, char *, char *);
+extern int cfg_key_tmp_telemetry_decode_cisco_v1_json_string(char *, char *, char *);
 
 extern void parse_time(char *, char *, int *, int *);
+extern void cfg_get_primitive_index_value(u_int64_t, u_int64_t *, u_int64_t *);
+extern void cfg_set_primitive_index_value(u_int64_t, u_int64_t , u_int64_t *);
 extern void cfg_set_aggregate(char *, u_int64_t [], u_int64_t, char *);
 
 #endif //CFG_HANDLER_H
