@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2019 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2022 by Paolo Lucente
 */
 
 /*
@@ -26,7 +26,6 @@
 #include "imt_plugin.h"
 #include "bgp/bgp.h"
 #include "net_aggr.h"
-#include "ports_aggr.h"
 
 //Global variables
 void (*imt_insert_func)(struct primitives_ptrs *);
@@ -45,6 +44,7 @@ void imt_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   struct sockaddr cAddr;
   struct pkt_data *data;
   struct ports_table pt;
+  struct protos_table prt, tost;
   unsigned char srvbuf[maxqsize];
   unsigned char *srvbufptr;
   struct query_header *qh;
@@ -82,6 +82,10 @@ void imt_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   void *zmq_host = NULL;
 #endif
 
+#ifdef WITH_REDIS
+  struct p_redis_host redis_host;
+#endif
+
   memcpy(&config, cfgptr, sizeof(struct configuration));
   memcpy(&extras, &((struct channels_list_entry *)ptr)->extras, sizeof(struct extra_primitives));
   recollect_pipe_memory(ptr);
@@ -99,7 +103,7 @@ void imt_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
     config.logfile_fd = open_output_file(config.logfile, "a", FALSE);
   }
 
-  if (extras.off_pkt_vlen_hdr_primitives) {
+  if (extras.off_pkt_vlen_hdr_primitives || config.what_to_count_2 & COUNT_MPLS_LABEL_STACK) {
     Log(LOG_ERR, "ERROR ( %s/%s ): variable-length primitives, ie. label, are not supported in IMT plugin. Exiting ..\n", config.name, config.type);
     exit_gracefully(1);
   }
@@ -132,11 +136,15 @@ void imt_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   memset(&nt, 0, sizeof(nt));
   memset(&nc, 0, sizeof(nc));
   memset(&pt, 0, sizeof(pt));
+  memset(&prt, 0, sizeof(prt));
+  memset(&tost, 0, sizeof(tost));
 
   load_networks(config.networks_file, &nt, &nc);
   set_net_funcs(&nt);
 
   if (config.ports_file) load_ports(config.ports_file, &pt);
+  if (config.protos_file) load_protos(config.protos_file, &prt);
+  if (config.tos_file) load_tos(config.tos_file, &tost);
 
   if (!config.num_memory_pools) config.num_memory_pools = NUM_MEMORY_POOLS;
   if (!config.memory_pool_size) config.memory_pool_size = MEMORY_POOL_SIZE;  
@@ -198,6 +206,15 @@ void imt_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
 
   qh = (struct query_header *) srvbuf;
 
+#ifdef WITH_REDIS
+  if (config.redis_host) {
+    char log_id[SHORTBUFLEN];
+
+    snprintf(log_id, sizeof(log_id), "%s/%s", config.name, config.type);
+    p_redis_init(&redis_host, log_id, p_redis_thread_produce_common_plugin_handler);
+  }
+#endif
+
   /* plugin main loop */
   for(;;) {
     poll_again:
@@ -358,11 +375,14 @@ void imt_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
     if (reload_map) {
       load_networks(config.networks_file, &nt, &nc);
       load_ports(config.ports_file, &pt);
+      load_protos(config.protos_file, &prt);
+      load_tos(config.tos_file, &tost);
+
       reload_map = FALSE;
     }
 
     if (reload_log) {
-      reload_logs();
+      reload_logs(NULL);
       reload_log = FALSE;
     }
 
@@ -449,8 +469,16 @@ void imt_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
 	    (*net_funcs[num])(&nt, &nc, &data->primitives, pbgp, &nfd);
 
 	  if (config.ports_file) {
-	    if (!pt.table[data->primitives.src_port]) data->primitives.src_port = 0;
-	    if (!pt.table[data->primitives.dst_port]) data->primitives.dst_port = 0;
+	    if (!pt.table[data->primitives.src_port]) data->primitives.src_port = PM_L4_PORT_OTHERS;
+	    if (!pt.table[data->primitives.dst_port]) data->primitives.dst_port = PM_L4_PORT_OTHERS;
+	  }
+
+	  if (config.protos_file) {
+	    if (!prt.table[data->primitives.proto]) data->primitives.proto = PM_IP_PROTO_OTHERS;
+	  }
+
+	  if (config.tos_file) {
+	    if (!tost.table[data->primitives.tos]) data->primitives.tos = PM_IP_TOS_OTHERS;
 	  }
 
 	  prim_ptrs.data = data; 
