@@ -20,8 +20,6 @@
  * 02111-1307, USA.  
  */
 
-#define __TABLE_C
-
 #include "pmacct.h"
 #include "isis.h"
 
@@ -239,7 +237,6 @@ route_node_match_ipv4 (const struct route_table *table,
   return route_node_match (table, (struct isis_prefix *) &p);
 }
 
-#ifdef ENABLE_IPV6
 struct route_node *
 route_node_match_ipv6 (const struct route_table *table,
 		       const struct in6_addr *addr)
@@ -253,7 +250,6 @@ route_node_match_ipv6 (const struct route_table *table,
 
   return route_node_match (table, (struct isis_prefix *) &p);
 }
-#endif /* ENABLE_IPV6 */
 
 /* Lookup same prefix node.  Return NULL when we can't find route. */
 struct route_node *
