@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2019 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2022 by Paolo Lucente
 */
 
 /*
@@ -37,6 +37,7 @@ void kafka_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
 {
   struct pkt_data *data;
   struct ports_table pt;
+  struct protos_table prt, tost;
   unsigned char *pipebuf;
   struct pollfd pfd;
   struct insert_data idata;
@@ -63,6 +64,10 @@ void kafka_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   void *zmq_host = NULL;
 #endif
 
+#ifdef WITH_REDIS
+  struct p_redis_host redis_host;
+#endif
+
   memcpy(&config, cfgptr, sizeof(struct configuration));
   memcpy(&extras, &((struct channels_list_entry *)ptr)->extras, sizeof(struct extra_primitives));
   recollect_pipe_memory(ptr);
@@ -84,11 +89,11 @@ void kafka_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   else if ((config.message_broker_output & PRINT_OUTPUT_AVRO_BIN) ||
 	   (config.message_broker_output & PRINT_OUTPUT_AVRO_JSON)) {
 #ifdef WITH_AVRO
-    avro_acct_schema = avro_schema_build_acct_data(config.what_to_count, config.what_to_count_2);
-    avro_schema_add_writer_id(avro_acct_schema);
+    p_avro_acct_schema = p_avro_schema_build_acct_data(config.what_to_count, config.what_to_count_2);
+    p_avro_schema_add_writer_id(p_avro_acct_schema);
 
-    avro_acct_init_schema = avro_schema_build_acct_init();
-    avro_acct_close_schema = avro_schema_build_acct_close();
+    p_avro_acct_init_schema = p_avro_schema_build_acct_init();
+    p_avro_acct_close_schema = p_avro_schema_build_acct_close();
 
     if (config.avro_schema_file) {
       char avro_schema_file[SRVBUFLEN];
@@ -98,9 +103,9 @@ void kafka_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
 	exit_gracefully(1);
       }
 
-      write_avro_schema_to_file_with_suffix(config.avro_schema_file, "-acct_data", avro_schema_file, avro_acct_schema);
-      write_avro_schema_to_file_with_suffix(config.avro_schema_file, "-acct_init", avro_schema_file, avro_acct_init_schema);
-      write_avro_schema_to_file_with_suffix(config.avro_schema_file, "-acct_close", avro_schema_file, avro_acct_close_schema);
+      write_avro_schema_to_file_with_suffix(config.avro_schema_file, "-acct_data", avro_schema_file, p_avro_acct_schema);
+      write_avro_schema_to_file_with_suffix(config.avro_schema_file, "-acct_init", avro_schema_file, p_avro_acct_init_schema);
+      write_avro_schema_to_file_with_suffix(config.avro_schema_file, "-acct_close", avro_schema_file, p_avro_acct_close_schema);
     }
 
     if (config.kafka_avro_schema_registry) {
@@ -146,11 +151,15 @@ void kafka_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   memset(&nt, 0, sizeof(nt));
   memset(&nc, 0, sizeof(nc));
   memset(&pt, 0, sizeof(pt));
+  memset(&prt, 0, sizeof(prt));
+  memset(&tost, 0, sizeof(tost));
 
   load_networks(config.networks_file, &nt, &nc);
   set_net_funcs(&nt);
 
   if (config.ports_file) load_ports(config.ports_file, &pt);
+  if (config.protos_file) load_protos(config.protos_file, &prt);
+  if (config.tos_file) load_tos(config.tos_file, &tost);
   
   memset(&idata, 0, sizeof(idata));
   memset(&prim_ptrs, 0, sizeof(prim_ptrs));
@@ -176,6 +185,57 @@ void kafka_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   /* setting number of entries in _protocols structure */
   while (_protocols[protocols_number].number != -1) protocols_number++;
 
+#ifdef WITH_REDIS
+  if (config.redis_host) {
+    char log_id[SHORTBUFLEN];
+
+    snprintf(log_id, sizeof(log_id), "%s/%s", config.name, config.type);
+    p_redis_init(&redis_host, log_id, p_redis_thread_produce_common_plugin_handler);
+  }
+#endif
+
+  /* setting some defaults */
+  if (!config.sql_host)
+    config.sql_host = default_kafka_broker_host;
+  if (!config.kafka_broker_port)
+    config.kafka_broker_port = default_kafka_broker_port;
+
+  if (!config.sql_table)
+    config.sql_table = default_kafka_topic;
+
+  p_kafka_init_host_struct(&kafkap_kafka_host);
+
+#if defined(WITH_AVRO) && defined(WITH_SERDES)
+  if (config.kafka_avro_schema_registry) {
+    if (strchr(config.sql_table, '$')) {
+      Log(LOG_ERR, "ERROR ( %s/%s ): dynamic 'kafka_topic' is not compatible with 'avro_schema_registry'. Exiting.\n",
+          config.name, config.type);
+      exit_gracefully(1);
+    }
+
+    kafkap_kafka_host.sd_schema[AVRO_ACCT_DATA_SID] =
+      compose_avro_schema_registry_name_2(config.sql_table, FALSE,
+                                          p_avro_acct_schema, "acct", "data",
+                                          config.kafka_avro_schema_registry);
+    kafkap_kafka_host.sd_schema[AVRO_ACCT_INIT_SID] =
+      compose_avro_schema_registry_name_2(config.sql_table, FALSE,
+                                          p_avro_acct_init_schema, "acct", "init",
+                                          config.kafka_avro_schema_registry);
+    kafkap_kafka_host.sd_schema[AVRO_ACCT_CLOSE_SID] =
+      compose_avro_schema_registry_name_2(config.sql_table, FALSE,
+                                          p_avro_acct_close_schema, "acct", "close",
+                                          config.kafka_avro_schema_registry);
+
+    if (!kafkap_kafka_host.sd_schema[AVRO_ACCT_DATA_SID] ||
+        !kafkap_kafka_host.sd_schema[AVRO_ACCT_INIT_SID] ||
+        !kafkap_kafka_host.sd_schema[AVRO_ACCT_CLOSE_SID]) {
+      Log(LOG_ERR, "ERROR ( %s/%s ): Failed to register schema information. Exiting.\n",
+          config.name, config.type);
+      exit_gracefully(1);
+    }
+  }
+#endif
+
   /* plugin main loop */
   for(;;) {
     poll_again:
@@ -201,7 +261,7 @@ void kafka_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
     poll_ops:
     P_update_time_reference(&idata);
 
-    if (idata.now > refresh_deadline) P_cache_handle_flush_event(&pt);
+    if (idata.now > refresh_deadline) P_cache_handle_flush_event(&pt, &prt, &tost);
 
     recv_budget = 0;
     if (poll_bypass) {
@@ -285,10 +345,18 @@ void kafka_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
 	  (*net_funcs[num])(&nt, &nc, &data->primitives, prim_ptrs.pbgp, &nfd);
 
 	if (config.ports_file) {
-          if (!pt.table[data->primitives.src_port]) data->primitives.src_port = 0;
-          if (!pt.table[data->primitives.dst_port]) data->primitives.dst_port = 0;
+          if (!pt.table[data->primitives.src_port]) data->primitives.src_port = PM_L4_PORT_OTHERS;
+          if (!pt.table[data->primitives.dst_port]) data->primitives.dst_port = PM_L4_PORT_OTHERS;
         }
 
+	if (config.protos_file) {
+	  if (!prt.table[data->primitives.proto]) data->primitives.proto = PM_IP_PROTO_OTHERS;
+	}
+
+	if (config.tos_file) {
+	  if (!tost.table[data->primitives.tos]) data->primitives.tos = PM_IP_TOS_OTHERS;
+	}
+
         prim_ptrs.data = data;
         (*insert_func)(&prim_ptrs, &idata);
 
@@ -329,6 +397,7 @@ void kafka_cache_purge(struct chained_cache *queue[], int index, int safe_action
   struct primitives_ptrs prim_ptrs;
   struct pkt_data dummy_data;
   pid_t writer_pid = getpid();
+  struct dynname_tokens writer_id_tokens;
 
   //TODO solve these warnings correctly
   (void)pvlen;
@@ -343,24 +412,17 @@ void kafka_cache_purge(struct chained_cache *queue[], int index, int safe_action
   int json_buf_off = 0;
 
 #ifdef WITH_AVRO
-  avro_writer_t avro_writer = {0};
-  char *avro_buf = NULL;
-  int avro_buffer_full = FALSE;
-  size_t avro_len = 0;
+  avro_writer_t p_avro_writer = {0};
+  char *p_avro_buf = NULL;
+  int p_avro_buffer_full = FALSE;
+  size_t p_avro_len = 0;
 #endif
 
-  p_kafka_init_host(&kafkap_kafka_host, config.kafka_config_file);
+  p_kafka_init_host_conf(&kafkap_kafka_host, config.kafka_config_file);
 
-  /* setting some defaults */
-  if (!config.sql_host) config.sql_host = default_kafka_broker_host;
-  if (!config.kafka_broker_port) config.kafka_broker_port = default_kafka_broker_port;
-
-  if (!config.sql_table) config.sql_table = default_kafka_topic;
-  else {
-    if (strchr(config.sql_table, '$')) {
-      is_topic_dyn = TRUE;
-      orig_kafka_topic = config.sql_table;
-    }
+  if (strchr(config.sql_table, '$')) {
+    is_topic_dyn = TRUE;
+    orig_kafka_topic = config.sql_table;
   }
 
   if (config.amqp_routing_key_rr) orig_kafka_topic = config.sql_table;
@@ -400,7 +462,7 @@ void kafka_cache_purge(struct chained_cache *queue[], int index, int safe_action
     exit_gracefully(1);
   }
 
-  if (!config.kafka_partition_dynamic) config.kafka_partition = RD_KAFKA_PARTITION_UA;
+  if (!config.kafka_partition_dynamic && !config.kafka_partition) config.kafka_partition = RD_KAFKA_PARTITION_UA;
 
   p_kafka_set_partition(&kafkap_kafka_host, config.kafka_partition);
 
@@ -415,32 +477,24 @@ void kafka_cache_purge(struct chained_cache *queue[], int index, int safe_action
     exit_gracefully(1);
   }
 
+  if (!config.writer_id_string) {
+    config.writer_id_string = DYNNAME_DEFAULT_WRITER_ID;
+  }
+
+  dynname_tokens_prepare(config.writer_id_string, &writer_id_tokens, DYN_STR_WRITER_ID);
+
   for (j = 0, stop = 0; (!stop) && P_preprocess_funcs[j]; j++)
     stop = P_preprocess_funcs[j](queue, &index, j);
 
   Log(LOG_INFO, "INFO ( %s/%s ): *** Purging cache - START (PID: %u) ***\n", config.name, config.type, writer_pid);
   start = time(NULL);
 
-#ifdef WITH_AVRO
+#if defined(WITH_AVRO) && defined(WITH_SERDES)
   if (config.kafka_avro_schema_registry) {
-#ifdef WITH_SERDES
     if (is_topic_dyn) {
       Log(LOG_ERR, "ERROR ( %s/%s ): dynamic 'kafka_topic' is not compatible with 'avro_schema_registry'. Exiting.\n", config.name, config.type);
       exit_gracefully(1);
     }
-
-    kafkap_kafka_host.sd_schema[AVRO_ACCT_DATA_SID] = compose_avro_schema_registry_name_2(p_kafka_get_topic(&kafkap_kafka_host),
-											  FALSE, avro_acct_schema, "acct", "data",
-											  config.kafka_avro_schema_registry);
-
-    kafkap_kafka_host.sd_schema[AVRO_ACCT_INIT_SID] = compose_avro_schema_registry_name_2(p_kafka_get_topic(&kafkap_kafka_host),
-											  FALSE, avro_acct_init_schema, "acct", "init",
-											  config.kafka_avro_schema_registry);
-
-    kafkap_kafka_host.sd_schema[AVRO_ACCT_CLOSE_SID] = compose_avro_schema_registry_name_2(p_kafka_get_topic(&kafkap_kafka_host),
-											   FALSE, avro_acct_close_schema, "acct", "close",
-											   config.kafka_avro_schema_registry);
-#endif
   }
 #endif
 
@@ -460,18 +514,18 @@ void kafka_cache_purge(struct chained_cache *queue[], int index, int safe_action
 #ifdef WITH_AVRO
     if (!config.avro_buffer_size) config.avro_buffer_size = LARGEBUFLEN;
 
-    avro_buf = malloc(config.avro_buffer_size);
+    p_avro_buf = malloc(config.avro_buffer_size);
 
-    if (!avro_buf) {
-      Log(LOG_ERR, "ERROR ( %s/%s ): malloc() failed (avro_buf). Exiting ..\n", config.name, config.type);
+    if (!p_avro_buf) {
+      Log(LOG_ERR, "ERROR ( %s/%s ): malloc() failed (p_avro_buf). Exiting ..\n", config.name, config.type);
       exit_gracefully(1);
     }
     else {
-      memset(avro_buf, 0, config.avro_buffer_size);
+      memset(p_avro_buf, 0, config.avro_buffer_size);
     }
 
     if (config.message_broker_output & PRINT_OUTPUT_AVRO_BIN) {
-      avro_writer = avro_writer_memory(avro_buf, config.avro_buffer_size);
+      p_avro_writer = avro_writer_memory(p_avro_buf, config.avro_buffer_size);
     }
 #endif
   }
@@ -495,41 +549,41 @@ void kafka_cache_purge(struct chained_cache *queue[], int index, int safe_action
     else if ((config.message_broker_output & PRINT_OUTPUT_AVRO_BIN) ||
 	     (config.message_broker_output & PRINT_OUTPUT_AVRO_JSON)) {
 #ifdef WITH_AVRO
-      avro_value_iface_t *avro_iface = avro_generic_class_from_schema(avro_acct_init_schema);
-      avro_value_t avro_value = compose_avro_acct_init(config.name, writer_pid, avro_iface);
+      avro_value_iface_t *p_avro_iface = avro_generic_class_from_schema(p_avro_acct_init_schema);
+      avro_value_t p_avro_value = compose_avro_acct_init(config.name, writer_pid, p_avro_iface);
 
       if (config.message_broker_output & PRINT_OUTPUT_AVRO_BIN) {
 	if (!config.kafka_avro_schema_registry) {
-	  avro_len = avro_writer_tell(avro_writer);
+	  p_avro_len = avro_writer_tell(p_avro_writer);
 	}
 #ifdef WITH_SERDES
 	else {
-	  void *avro_local_buf = NULL;
+	  void *p_avro_local_buf = NULL;
 
-	  if (serdes_schema_serialize_avro(kafkap_kafka_host.sd_schema[AVRO_ACCT_INIT_SID], &avro_value, &avro_local_buf,
-					   &avro_len, kafkap_kafka_host.errstr, sizeof(kafkap_kafka_host.errstr))) {
-	    Log(LOG_ERR, "ERROR ( %s/%s ): kafka_cache_purge(): serdes_schema_serialize_avro() failed: %s\n", config.name, config.type, kafkap_kafka_host.errstr);
+	  if (serdes_schema_serialize_avro(kafkap_kafka_host.sd_schema[AVRO_ACCT_INIT_SID], &p_avro_value, &p_avro_local_buf,
+					   &p_avro_len, kafkap_kafka_host.errstr, sizeof(kafkap_kafka_host.errstr))) {
+	    Log(LOG_ERR, "ERROR ( %s/%s ): kafka_cache_purge(): serdes_schema_serialize_avro() failed for %s: %s\n", config.name, config.type, "acct_init", kafkap_kafka_host.errstr);
 	    exit_gracefully(1);
 	  }
 	  else {
-	    avro_buf = avro_local_buf;
+	    p_avro_buf = p_avro_local_buf;
 	  }
 	}
 #endif
-        ret = p_kafka_produce_data(&kafkap_kafka_host, avro_buf, avro_len);
-        if (!config.kafka_avro_schema_registry) avro_writer_reset(avro_writer);
+        ret = p_kafka_produce_data(&kafkap_kafka_host, p_avro_buf, p_avro_len);
+        if (!config.kafka_avro_schema_registry) avro_writer_reset(p_avro_writer);
       }
       else if (config.message_broker_output & PRINT_OUTPUT_AVRO_JSON) {
-	char *avro_local_buf = write_avro_json_record_to_buf(avro_value);
+	char *p_avro_local_buf = write_avro_json_record_to_buf(p_avro_value);
 
-	if (avro_local_buf) {
-	  ret = p_kafka_produce_data(&kafkap_kafka_host, avro_local_buf, strlen(avro_local_buf));
-	  free(avro_local_buf);
+	if (p_avro_local_buf) {
+	  ret = p_kafka_produce_data(&kafkap_kafka_host, p_avro_local_buf, strlen(p_avro_local_buf));
+	  free(p_avro_local_buf);
 	}
       }
 
-      avro_value_decref(&avro_value);
-      avro_value_iface_decref(avro_iface);
+      avro_value_decref(&p_avro_value);
+      avro_value_iface_decref(p_avro_iface);
 #endif
     }
   }
@@ -574,7 +628,7 @@ void kafka_cache_purge(struct chained_cache *queue[], int index, int safe_action
       int idx;
 
       for (idx = 0; idx < N_PRIMITIVES && cjhandler[idx]; idx++) cjhandler[idx](json_obj, queue[j]);
-      add_writer_name_and_pid_json(json_obj, config.name, writer_pid);
+      add_writer_name_and_pid_json(json_obj, &writer_id_tokens);
 
       json_str = compose_json_str(json_obj);
 #endif
@@ -582,29 +636,29 @@ void kafka_cache_purge(struct chained_cache *queue[], int index, int safe_action
     else if ((config.message_broker_output & PRINT_OUTPUT_AVRO_BIN) ||
 	     (config.message_broker_output & PRINT_OUTPUT_AVRO_JSON)) {
 #ifdef WITH_AVRO
-      avro_value_iface_t *avro_iface = avro_generic_class_from_schema(avro_acct_schema);
-      avro_value_t avro_value = compose_avro_acct_data(config.what_to_count, config.what_to_count_2,
+      avro_value_iface_t *p_avro_iface = avro_generic_class_from_schema(p_avro_acct_schema);
+      avro_value_t p_avro_value = compose_avro_acct_data(config.what_to_count, config.what_to_count_2,
 			   queue[j]->flow_type, &queue[j]->primitives, pbgp, pnat, pmpls, ptun, pcust,
 			   pvlen, queue[j]->bytes_counter, queue[j]->packet_counter,
-			   queue[j]->flow_counter, queue[j]->tcp_flags, &queue[j]->basetime,
-			   queue[j]->stitch, avro_iface);
-      add_writer_name_and_pid_avro(avro_value, config.name, writer_pid);
+			   queue[j]->flow_counter, queue[j]->tcp_flags, queue[j]->tunnel_tcp_flags,
+			   &queue[j]->basetime, queue[j]->stitch, p_avro_iface);
+      add_writer_name_and_pid_avro_v2(p_avro_value, &writer_id_tokens);
 
       if (config.message_broker_output & PRINT_OUTPUT_AVRO_BIN) {
-	size_t avro_value_size;
+	size_t p_avro_value_size;
 
-        avro_value_sizeof(&avro_value, &avro_value_size);
+        avro_value_sizeof(&p_avro_value, &p_avro_value_size);
 
 	if (!config.kafka_avro_schema_registry) {
-	  if (avro_value_size > config.avro_buffer_size) {
+	  if (p_avro_value_size > config.avro_buffer_size) {
 	    Log(LOG_ERR, "ERROR ( %s/%s ): kafka_cache_purge(): avro_buffer_size too small (%u)\n", config.name, config.type, config.avro_buffer_size);
 	    exit_gracefully(1);
 	  }
-	  else if (avro_value_size >= (config.avro_buffer_size - avro_writer_tell(avro_writer))) {
-	    avro_buffer_full = TRUE;
+	  else if (p_avro_value_size >= (config.avro_buffer_size - avro_writer_tell(p_avro_writer))) {
+	    p_avro_buffer_full = TRUE;
 	    j--;
 	  }
-	  else if (avro_value_write(avro_writer, &avro_value)) {
+	  else if (avro_value_write(p_avro_writer, &p_avro_value)) {
 	    Log(LOG_ERR, "ERROR ( %s/%s ): kafka_cache_purge(): avro_value_write() faiiled: %s\n", config.name, config.type, avro_strerror());
 	    exit_gracefully(1);
 	  }
@@ -612,55 +666,55 @@ void kafka_cache_purge(struct chained_cache *queue[], int index, int safe_action
 	    mv_num++;
 	  }
 
-	  avro_len = avro_writer_tell(avro_writer);
+	  p_avro_len = avro_writer_tell(p_avro_writer);
 	}
 #ifdef WITH_SERDES
 	else {
-	  void *avro_local_buf = NULL;
+	  void *p_avro_local_buf = NULL;
 
-	  avro_len = 0;
-	  if (avro_buf) {
-	    free(avro_buf);
-	    avro_buf = NULL;
+	  p_avro_len = 0;
+	  if (p_avro_buf) {
+	    free(p_avro_buf);
+	    p_avro_buf = NULL;
 	  }
 
-	  if (serdes_schema_serialize_avro(kafkap_kafka_host.sd_schema[AVRO_ACCT_DATA_SID], &avro_value, &avro_local_buf,
-					   &avro_len, kafkap_kafka_host.errstr, sizeof(kafkap_kafka_host.errstr))) {
-	    Log(LOG_ERR, "ERROR ( %s/%s ): kafka_cache_purge(): serdes_schema_serialize_avro() failed: %s\n", config.name, config.type, kafkap_kafka_host.errstr);
+	  if (serdes_schema_serialize_avro(kafkap_kafka_host.sd_schema[AVRO_ACCT_DATA_SID], &p_avro_value, &p_avro_local_buf,
+					   &p_avro_len, kafkap_kafka_host.errstr, sizeof(kafkap_kafka_host.errstr))) {
+	    Log(LOG_ERR, "ERROR ( %s/%s ): kafka_cache_purge(): serdes_schema_serialize_avro() failed for %s: %s\n", config.name, config.type, "acct_data", kafkap_kafka_host.errstr);
 	    exit_gracefully(1);
 	  }
 	  else {
-	    avro_buf = avro_local_buf;
+	    p_avro_buf = p_avro_local_buf;
 	    mv_num++;
 	  }
 	}
 #endif
       }
       else if (config.message_broker_output & PRINT_OUTPUT_AVRO_JSON) {
-	char *avro_local_buf = write_avro_json_record_to_buf(avro_value);
+	char *p_avro_local_buf = write_avro_json_record_to_buf(p_avro_value);
 
-	if (avro_local_buf) {
-	  size_t avro_locbuf_len = strlen(avro_local_buf);
+	if (p_avro_local_buf) {
+	  size_t p_avro_locbuf_len = strlen(p_avro_local_buf);
 
-	  if (avro_locbuf_len > config.avro_buffer_size) {
+	  if (p_avro_locbuf_len > config.avro_buffer_size) {
 	    Log(LOG_ERR, "ERROR ( %s/%s ): kafka_cache_purge(): avro_buffer_size too small (%u)\n", config.name, config.type, config.avro_buffer_size);
 	    exit_gracefully(1);
 	  }
-	  else if (avro_locbuf_len >= (config.avro_buffer_size - strlen(avro_buf))) {
-	    avro_buffer_full = TRUE;
+	  else if (p_avro_locbuf_len >= (config.avro_buffer_size - strlen(p_avro_buf))) {
+	    p_avro_buffer_full = TRUE;
 	    j--;
 	  }
 	  else {
-	    strcat(avro_buf, avro_local_buf);
+	    strcat(p_avro_buf, p_avro_local_buf);
 	    mv_num++;
 	  }
 
-	  free(avro_local_buf);
+	  free(p_avro_local_buf);
 	}
       }
 
-      avro_value_decref(&avro_value);
-      avro_value_iface_decref(avro_iface);
+      avro_value_decref(&p_avro_value);
+      avro_value_iface_decref(p_avro_iface);
 #else
       if (config.debug) Log(LOG_DEBUG, "DEBUG ( %s/%s ): compose_avro_acct_data(): AVRO object not created due to missing --enable-avro\n", config.name, config.type);
 #endif
@@ -734,7 +788,7 @@ void kafka_cache_purge(struct chained_cache *queue[], int index, int safe_action
     else if ((config.message_broker_output & PRINT_OUTPUT_AVRO_BIN) ||
 	     (config.message_broker_output & PRINT_OUTPUT_AVRO_JSON)) {
 #ifdef WITH_AVRO
-      if (!config.sql_multi_values || (mv_num >= config.sql_multi_values) || avro_buffer_full) {
+      if (!config.sql_multi_values || (mv_num >= config.sql_multi_values) || p_avro_buffer_full) {
         if (is_topic_dyn) {
 	  prim_ptrs.data = &dummy_data;
 	  primptrs_set_all_from_chained_cache(&prim_ptrs, queue[j]);
@@ -749,15 +803,15 @@ void kafka_cache_purge(struct chained_cache *queue[], int index, int safe_action
         }
 
 	if (config.message_broker_output & PRINT_OUTPUT_AVRO_BIN) { 
-	  ret = p_kafka_produce_data(&kafkap_kafka_host, avro_buf, avro_len);
-	  if (!config.kafka_avro_schema_registry) avro_writer_reset(avro_writer);
+	  ret = p_kafka_produce_data(&kafkap_kafka_host, p_avro_buf, p_avro_len);
+	  if (!config.kafka_avro_schema_registry) avro_writer_reset(p_avro_writer);
 	}
 	else if (config.message_broker_output & PRINT_OUTPUT_AVRO_JSON) {
-	  ret = p_kafka_produce_data(&kafkap_kafka_host, avro_buf, strlen(avro_buf));
-	  memset(avro_buf, 0, config.avro_buffer_size);
+	  ret = p_kafka_produce_data(&kafkap_kafka_host, p_avro_buf, strlen(p_avro_buf));
+	  memset(p_avro_buf, 0, config.avro_buffer_size);
         }
 
-        avro_buffer_full = FALSE;
+        p_avro_buffer_full = FALSE;
         mv_num_save = mv_num;
         mv_num = 0;
 
@@ -782,15 +836,15 @@ void kafka_cache_purge(struct chained_cache *queue[], int index, int safe_action
 	     (config.message_broker_output & PRINT_OUTPUT_AVRO_JSON)) {
 #ifdef WITH_AVRO
       if (config.message_broker_output & PRINT_OUTPUT_AVRO_BIN) {
-	if (avro_len) {
-	  ret = p_kafka_produce_data(&kafkap_kafka_host, avro_buf, avro_len);
-	  if (!config.kafka_avro_schema_registry) avro_writer_free(avro_writer);
+	if (p_avro_len) {
+	  ret = p_kafka_produce_data(&kafkap_kafka_host, p_avro_buf, p_avro_len);
+	  if (!config.kafka_avro_schema_registry) avro_writer_free(p_avro_writer);
           if (!ret) qn += mv_num;
 	}
       }
       else if (config.message_broker_output & PRINT_OUTPUT_AVRO_JSON) {
-        if (strlen(avro_buf)) {
-	  ret = p_kafka_produce_data(&kafkap_kafka_host, avro_buf, strlen(avro_buf));
+        if (strlen(p_avro_buf)) {
+	  ret = p_kafka_produce_data(&kafkap_kafka_host, p_avro_buf, strlen(p_avro_buf));
 	  if (!ret) qn += mv_num;
 	}
       }
@@ -821,41 +875,41 @@ void kafka_cache_purge(struct chained_cache *queue[], int index, int safe_action
     else if ((config.message_broker_output & PRINT_OUTPUT_AVRO_BIN) ||
 	     (config.message_broker_output & PRINT_OUTPUT_AVRO_JSON)) {
 #ifdef WITH_AVRO
-      avro_value_iface_t *avro_iface = avro_generic_class_from_schema(avro_acct_close_schema);
-      avro_value_t avro_value = compose_avro_acct_close(config.name, writer_pid, qn, saved_index, duration, avro_iface);
+      avro_value_iface_t *p_avro_iface = avro_generic_class_from_schema(p_avro_acct_close_schema);
+      avro_value_t p_avro_value = compose_avro_acct_close(config.name, writer_pid, qn, saved_index, duration, p_avro_iface);
 
       if (config.message_broker_output & PRINT_OUTPUT_AVRO_BIN) {
 	if (!config.kafka_avro_schema_registry) {
-	  avro_len = avro_writer_tell(avro_writer);
+	  p_avro_len = avro_writer_tell(p_avro_writer);
 	}
 #ifdef WITH_SERDES
 	else {
-	  void *avro_local_buf = NULL;
+	  void *p_avro_local_buf = NULL;
 
-	  if (serdes_schema_serialize_avro(kafkap_kafka_host.sd_schema[AVRO_ACCT_CLOSE_SID], &avro_value, &avro_local_buf,
-					   &avro_len, kafkap_kafka_host.errstr, sizeof(kafkap_kafka_host.errstr))) {
-	    Log(LOG_ERR, "ERROR ( %s/%s ): kafka_cache_purge(): serdes_schema_serialize_avro() failed: %s\n", config.name, config.type, kafkap_kafka_host.errstr);
+	  if (serdes_schema_serialize_avro(kafkap_kafka_host.sd_schema[AVRO_ACCT_CLOSE_SID], &p_avro_value, &p_avro_local_buf,
+					   &p_avro_len, kafkap_kafka_host.errstr, sizeof(kafkap_kafka_host.errstr))) {
+	    Log(LOG_ERR, "ERROR ( %s/%s ): kafka_cache_purge(): serdes_schema_serialize_avro() failed for %s: %s\n", config.name, config.type, "acct_close", kafkap_kafka_host.errstr);
 	    exit_gracefully(1);
 	  }
 	  else {
-	    avro_buf = avro_local_buf;
+	    p_avro_buf = p_avro_local_buf;
 	  }
 	}
 #endif
-	ret = p_kafka_produce_data(&kafkap_kafka_host, avro_buf, avro_len);
-	if (!config.kafka_avro_schema_registry) avro_writer_reset(avro_writer);
+	ret = p_kafka_produce_data(&kafkap_kafka_host, p_avro_buf, p_avro_len);
+	if (!config.kafka_avro_schema_registry) avro_writer_reset(p_avro_writer);
       }
       else if (config.message_broker_output & PRINT_OUTPUT_AVRO_JSON) {
-	char *avro_local_buf = write_avro_json_record_to_buf(avro_value);
+	char *p_avro_local_buf = write_avro_json_record_to_buf(p_avro_value);
 
-	if (avro_local_buf) {
-	  ret = p_kafka_produce_data(&kafkap_kafka_host, avro_local_buf, strlen(avro_local_buf));
-	  free(avro_local_buf);
+	if (p_avro_local_buf) {
+	  ret = p_kafka_produce_data(&kafkap_kafka_host, p_avro_local_buf, strlen(p_avro_local_buf));
+	  free(p_avro_local_buf);
 	}
       }
 
-      avro_value_decref(&avro_value);
-      avro_value_iface_decref(avro_iface);
+      avro_value_decref(&p_avro_value);
+      avro_value_iface_decref(p_avro_iface);
 #endif
     }
   }
@@ -872,6 +926,6 @@ void kafka_cache_purge(struct chained_cache *queue[], int index, int safe_action
   if (json_buf) free(json_buf);
 
 #ifdef WITH_AVRO
-  if (avro_buf) free(avro_buf);
+  if (p_avro_buf) free(p_avro_buf);
 #endif
 }
