@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2020 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2022 by Paolo Lucente
 */
 
 /*
@@ -48,6 +48,7 @@ void mysql_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
 {
   struct pkt_data *data;
   struct ports_table pt;
+  struct protos_table prt, tost;
   struct pollfd pfd;
   struct insert_data idata;
   time_t refresh_deadline;
@@ -102,7 +103,7 @@ void mysql_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   refresh_deadline = idata.now;
   idata.cfg = &config;
 
-  sql_init_maps(&extras, &prim_ptrs, &nt, &nc, &pt);
+  sql_init_maps(&extras, &prim_ptrs, &nt, &nc, &pt, &prt, &tost);
   sql_init_global_buffers();
   sql_init_historical_acct(idata.now, &idata);
   sql_init_triggers(idata.now, &idata);
@@ -153,25 +154,12 @@ void mysql_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
     }
 
     poll_ops:
-    idata.now = time(NULL);
-
-    if (config.sql_history) {
-      while (idata.now > (idata.basetime + idata.timeslot)) {
-        time_t saved_basetime = idata.basetime;
-
-        idata.basetime += idata.timeslot;
-        if (config.sql_history == COUNT_MONTHLY)
-          idata.timeslot = calc_monthly_timeslot(idata.basetime, config.sql_history_howmany, ADD);
-        glob_basetime = idata.basetime;
-        idata.new_basetime = saved_basetime;
-        glob_new_basetime = saved_basetime;
-      }
-    }
+    sql_update_time_reference(&idata);
 
     /* lazy sql refresh handling */
     if (idata.now > refresh_deadline) {
       if (sql_qq_ptr) sql_cache_flush(sql_queries_queue, sql_qq_ptr, &idata, FALSE);
-      sql_cache_handle_flush_event(&idata, &refresh_deadline, &pt);
+      sql_cache_handle_flush_event(&idata, &refresh_deadline, &pt, &prt, &tost);
     }
     else {
       if (config.sql_trigger_exec) {
@@ -267,8 +255,16 @@ void mysql_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
 	  (*net_funcs[num])(&nt, &nc, &data->primitives, prim_ptrs.pbgp, &nfd);
 
 	if (config.ports_file) {
-	  if (!pt.table[data->primitives.src_port]) data->primitives.src_port = 0;
-	  if (!pt.table[data->primitives.dst_port]) data->primitives.dst_port = 0;
+	  if (!pt.table[data->primitives.src_port]) data->primitives.src_port = PM_L4_PORT_OTHERS;
+	  if (!pt.table[data->primitives.dst_port]) data->primitives.dst_port = PM_L4_PORT_OTHERS;
+	}
+
+	if (config.protos_file) {
+	  if (!prt.table[data->primitives.proto]) data->primitives.proto = PM_IP_PROTO_OTHERS;
+	}
+
+	if (config.tos_file) {
+	  if (!tost.table[data->primitives.tos]) data->primitives.tos = PM_IP_TOS_OTHERS;
 	}
 
         prim_ptrs.data = data;
