@@ -1,6 +1,6 @@
 /*  
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2021 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2022 by Paolo Lucente
 */
 
 /*
@@ -196,8 +196,8 @@ struct data_hdr_v9 {
 /* ... */
 #define NF9_FLOW_BYTES			85 
 #define NF9_FLOW_PACKETS		86 
-
-#define NF9_FORWARDING_STATUS           89
+/* ... */
+#define NF9_FWD_STATUS			89
 #define NF9_MPLS_VPN_RD			90
 /* ... */
 #define NF9_LAYER2_PKT_SECTION_DATA	104
@@ -237,7 +237,7 @@ struct data_hdr_v9 {
 /* ... */
 #define NF9_INITIATOR_OCTETS		231
 #define NF9_RESPONDER_OCTETS		232
-/* ... */
+#define NF9_FW_EVENT			233
 #define NF9_INGRESS_VRFID		234
 #define NF9_EGRESS_VRFID		235
 /* ... */
@@ -512,6 +512,7 @@ extern struct xflow_status_entry *nfv9_check_status(struct packet_ptrs *, u_int3
 extern void nfv9_datalink_frame_section_handler(struct packet_ptrs *);
 
 extern struct template_cache tpl_cache;
+extern cdada_map_t *tpl_data_map, *tpl_opt_map;
 extern struct host_addr debug_a;
 extern char debug_agent_addr[50];
 extern u_int16_t debug_agent_port;
@@ -521,7 +522,7 @@ extern struct template_cache_entry *handle_template(struct template_hdr_v9 *, st
 extern struct template_cache_entry *find_template(u_int16_t, struct sockaddr *, u_int16_t, u_int32_t);
 extern struct template_cache_entry *insert_template(struct template_hdr_v9 *, struct packet_ptrs *, u_int16_t, u_int32_t, u_int16_t *, u_int8_t, u_int16_t, u_int32_t);
 extern struct template_cache_entry *refresh_template(struct template_hdr_v9 *, struct template_cache_entry *, struct packet_ptrs *, u_int16_t, u_int32_t, u_int16_t *, u_int8_t, u_int16_t, u_int32_t);
-extern void log_template_header(struct template_cache_entry *, struct packet_ptrs *, u_int16_t, u_int32_t, u_int8_t);
+extern void log_template_header(struct template_cache_entry *, struct sockaddr *, u_int16_t, u_int32_t, u_int8_t);
 extern void log_opt_template_field(u_int8_t, u_int32_t *, u_int16_t, u_int16_t, u_int16_t, u_int8_t);
 extern void log_template_field(u_int8_t, u_int32_t *, u_int16_t, u_int16_t, u_int16_t, u_int8_t);
 extern void log_template_footer(struct template_cache_entry *, u_int16_t, u_int8_t);
@@ -537,6 +538,12 @@ extern struct template_cache_entry *nfacctd_offline_read_json_template(char *, c
 extern void load_templates_from_file(char *);
 extern void save_template(struct template_cache_entry *, char *);
 
+extern u_int16_t calc_template_keylen();
+extern u_char *compose_template_key(pm_hash_serial_t *, u_int16_t, struct sockaddr *, u_int32_t);
+extern struct template_cache_entry *handle_template_v2(struct template_hdr_v9 *, struct packet_ptrs *, u_int16_t, u_int32_t, u_int16_t *, u_int16_t, u_int32_t);
+extern struct template_cache_entry *compose_template(struct template_hdr_v9 *, struct sockaddr *, u_int16_t, u_int32_t, u_int16_t *, u_int8_t, u_int16_t, u_int32_t);
+extern struct template_cache_entry *compose_opt_template(void *, struct sockaddr *, u_int16_t, u_int32_t, u_int16_t *, u_int8_t, u_int16_t, u_int32_t);
+
 #ifdef WITH_KAFKA
 extern void NF_init_kafka_host(void *);
 #endif
@@ -545,5 +552,7 @@ extern void NF_init_kafka_host(void *);
 extern void NF_init_zmq_host(void *, int *);
 #endif
 
+extern void NF_mpls_vpn_rd_fromie90(struct packet_ptrs *);
+
 extern struct utpl_field *(*get_ext_db_ie_by_type)(struct template_cache_entry *, u_int32_t, u_int16_t, u_int8_t);
 #endif //NFACCTD_H
