@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2019 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2022 by Paolo Lucente
 */
 
 /*
@@ -42,35 +42,35 @@ struct NF9_HEADER {
 	u_int16_t version, flows;
 	u_int32_t uptime_ms, time_sec;
 	u_int32_t package_sequence, source_id;
-} __packed;
+} __attribute__ ((packed));
 struct IPFIX_HEADER {
         u_int16_t version, len;
         u_int32_t time_sec;
         u_int32_t package_sequence, source_id;
-} __packed;
+} __attribute__ ((packed));
 struct NF9_FLOWSET_HEADER_COMMON {
 	u_int16_t flowset_id, length;
-} __packed;
+} __attribute__ ((packed));
 struct NF9_TEMPLATE_FLOWSET_HEADER {
 	struct NF9_FLOWSET_HEADER_COMMON c;
 	u_int16_t template_id, count;
-} __packed;
+} __attribute__ ((packed));
 struct NF9_OPTIONS_TEMPLATE_FLOWSET_HEADER {
         struct NF9_FLOWSET_HEADER_COMMON c;
         u_int16_t template_id, scope_len;
         u_int16_t option_len;
-} __packed;
+} __attribute__ ((packed));
 struct NF9_TEMPLATE_FLOWSET_RECORD {
 	u_int16_t type, length;
-} __packed;
+} __attribute__ ((packed));
 struct IPFIX_PEN_TEMPLATE_FLOWSET_RECORD {
         u_int16_t type;
 	u_int16_t length;
         u_int32_t pen;
-} __packed;
+} __attribute__ ((packed));
 struct NF9_DATA_FLOWSET_HEADER {
 	struct NF9_FLOWSET_HEADER_COMMON c;
-} __packed;
+} __attribute__ ((packed));
 #define NF9_TEMPLATE_FLOWSET_ID		0
 #define NF9_OPTIONS_FLOWSET_ID		1
 #define IPFIX_TEMPLATE_FLOWSET_ID	2
@@ -145,6 +145,8 @@ struct NF9_DATA_FLOWSET_HEADER {
 /* ... */
 #define NF9_FIRST_SWITCHED_MSEC         152
 #define NF9_LAST_SWITCHED_MSEC          153
+#define NF9_FIRST_SWITCHED_USEC         154
+#define NF9_LAST_SWITCHED_USEC          155
 /* ... */
 
 /* CUSTOM TYPES START HERE: supported in IPFIX only with pmacct PEN */
@@ -162,19 +164,19 @@ struct NF9_SOFTFLOWD_TEMPLATE {
 	struct NF9_TEMPLATE_FLOWSET_HEADER h;
 	struct NF9_TEMPLATE_FLOWSET_RECORD r[NF9_SOFTFLOWD_TEMPLATE_NRECORDS];
 	u_int16_t tot_len;
-} __packed;
+} __attribute__ ((packed));
 
 struct IPFIX_PEN_TEMPLATE_ADDENDUM {
         struct IPFIX_PEN_TEMPLATE_FLOWSET_RECORD r[NF9_SOFTFLOWD_TEMPLATE_NRECORDS];
         u_int16_t tot_len;
-} __packed;
+} __attribute__ ((packed));
 
 #define NF9_OPTIONS_TEMPLATE_NRECORDS 4
 struct NF9_OPTIONS_TEMPLATE {
         struct NF9_OPTIONS_TEMPLATE_FLOWSET_HEADER h;
         struct NF9_TEMPLATE_FLOWSET_RECORD r[NF9_OPTIONS_TEMPLATE_NRECORDS];
         u_int16_t tot_len;
-} __packed;
+} __attribute__ ((packed));
 
 typedef int (*flow_to_flowset_handler) (char *, const struct FLOW *, int, int);
 struct NF9_INTERNAL_TEMPLATE_RECORD {
@@ -232,27 +234,6 @@ static u_int8_t send_sampling_option = FALSE;
 static u_int8_t send_class_option = FALSE;
 static u_int8_t send_exporter_option = FALSE;
 
-/*
- * XXX: pmXXX_htonll(): similar to htonl() for 64 bits integers; no checks are done
- * on the length of the buffer.
- */
-u_int64_t pmXXX_htonll(u_int64_t addr)
-{
-#if defined IM_LITTLE_ENDIAN
-  u_int64_t buf;
-
-  u_int32_t *x = (u_int32_t *)(void *) &addr;
-  u_int32_t *y = (u_int32_t *)(void *) &buf;
-
-  y[0] = htonl(x[1]);
-  y[1] = htonl(x[0]);
-
-  return buf;
-#else
-  return addr;
-#endif
-}
-
 static int
 flow_to_flowset_input_handler(char *flowset, const struct FLOW *flow, int idx, int size)
 {
@@ -453,21 +434,17 @@ flow_to_flowset_mpls_label_top_handler(char *flowset, const struct FLOW *flow, i
   return 0;
 }
 
-static int
-flow_to_flowset_class_handler(char *flowset, const struct FLOW *flow, int idx, int size)
-{
-  memcpy(flowset, &flow->class, size);
-
-  return 0;
-}
-
 #if defined (WITH_NDPI)
 static int
 flow_to_flowset_ndpi_class_handler(char *flowset, const struct FLOW *flow, int idx, int size)
 {
-  u_int32_t tmp32 = flow->ndpi_class.app_protocol;
+  u_int8_t ie95_classId = 0x16; /* nDPI */
+  u_int32_t ie95_selectId = htonl(flow->ndpi_class.app_protocol);
 
-  memcpy(flowset, &tmp32, size);
+  if (size == 5) {
+    memcpy(flowset, &ie95_classId, 1);
+    memcpy((flowset + 1), &ie95_selectId, 4);
+  }
 
   return 0;
 }
@@ -478,7 +455,7 @@ flow_to_flowset_tag_handler(char *flowset, const struct FLOW *flow, int idx, int
 {
   pm_id_t tag;
 
-  tag = pmXXX_htonll(flow->tag[idx]);
+  tag = pm_htonll(flow->tag[idx]);
   memcpy(flowset, &tag, size);
 
   return 0;
@@ -489,7 +466,7 @@ flow_to_flowset_tag2_handler(char *flowset, const struct FLOW *flow, int idx, in
 {
   pm_id_t tag;
 
-  tag = pmXXX_htonll(flow->tag2[idx]);
+  tag = pm_htonll(flow->tag2[idx]);
   memcpy(flowset, &tag, size);
 
   return 0;
@@ -771,22 +748,39 @@ nf9_init_template(void)
 	  rcount++;
 	}
 	else if ((config.nfprobe_version == 9 && !config.timestamps_secs) || config.nfprobe_version == 10) {
-          v4_template.r[rcount].type = htons(NF9_LAST_SWITCHED_MSEC);
-          v4_template.r[rcount].length = htons(8);
-          v4_int_template.r[rcount].length = 8;
-          v4_template_out.r[rcount].type = htons(NF9_LAST_SWITCHED_MSEC);
-          v4_template_out.r[rcount].length = htons(8);
-          v4_int_template_out.r[rcount].length = 8;
-          rcount++;
-          v4_template.r[rcount].type = htons(NF9_FIRST_SWITCHED_MSEC);
-          v4_template.r[rcount].length = htons(8);
-          v4_int_template.r[rcount].length = 8;
-          v4_template_out.r[rcount].type = htons(NF9_FIRST_SWITCHED_MSEC);
-          v4_template_out.r[rcount].length = htons(8);
-          v4_int_template_out.r[rcount].length = 8;
-          rcount++;
+	  if (!config.nfprobe_tstamp_usec) {
+	    v4_template.r[rcount].type = htons(NF9_LAST_SWITCHED_MSEC);
+	    v4_template.r[rcount].length = htons(8);
+	    v4_int_template.r[rcount].length = 8;
+	    v4_template_out.r[rcount].type = htons(NF9_LAST_SWITCHED_MSEC);
+	    v4_template_out.r[rcount].length = htons(8);
+	    v4_int_template_out.r[rcount].length = 8;
+	    rcount++;
+	    v4_template.r[rcount].type = htons(NF9_FIRST_SWITCHED_MSEC);
+	    v4_template.r[rcount].length = htons(8);
+	    v4_int_template.r[rcount].length = 8;
+	    v4_template_out.r[rcount].type = htons(NF9_FIRST_SWITCHED_MSEC);
+	    v4_template_out.r[rcount].length = htons(8);
+	    v4_int_template_out.r[rcount].length = 8;
+	    rcount++;
+	  }
+	  else {
+	    v4_template.r[rcount].type = htons(NF9_LAST_SWITCHED_USEC);
+	    v4_template.r[rcount].length = htons(16);
+	    v4_int_template.r[rcount].length = 16;
+	    v4_template_out.r[rcount].type = htons(NF9_LAST_SWITCHED_USEC);
+	    v4_template_out.r[rcount].length = htons(16);
+	    v4_int_template_out.r[rcount].length = 16;
+	    rcount++;
+	    v4_template.r[rcount].type = htons(NF9_FIRST_SWITCHED_USEC);
+	    v4_template.r[rcount].length = htons(16);
+	    v4_int_template.r[rcount].length = 16;
+	    v4_template_out.r[rcount].type = htons(NF9_FIRST_SWITCHED_USEC);
+	    v4_template_out.r[rcount].length = htons(16);
+	    v4_int_template_out.r[rcount].length = 16;
+	    rcount++;
+	  }
 	}
-#if defined HAVE_64BIT_COUNTERS
         v4_template.r[rcount].type = htons(NF9_IN_BYTES);
         v4_template.r[rcount].length = htons(8);
         v4_int_template.r[rcount].length = 8;
@@ -801,26 +795,6 @@ nf9_init_template(void)
         v4_template_out.r[rcount].length = htons(8);
         v4_int_template_out.r[rcount].length = 8;
         rcount++;
-#else
-	v4_template.r[rcount].type = htons(NF9_IN_BYTES);
-	v4_template.r[rcount].length = htons(4);
-	v4_int_template.r[rcount].length = 4;
-	// Cisco doesn't appear to do that (yet?)
-        // v4_template_out.r[rcount].type = htons(NF9_OUT_BYTES);
-        v4_template_out.r[rcount].type = htons(NF9_IN_BYTES);
-        v4_template_out.r[rcount].length = htons(4);
-        v4_int_template_out.r[rcount].length = 4;
-	rcount++;
-	v4_template.r[rcount].type = htons(NF9_IN_PACKETS);
-	v4_template.r[rcount].length = htons(4);
-	v4_int_template.r[rcount].length = 4;
-	// Cisco doesn't appear to do that (yet?)
-        // v4_template_out.r[rcount].type = htons(NF9_OUT_PACKETS);
-	v4_template_out.r[rcount].type = htons(NF9_IN_PACKETS);
-        v4_template_out.r[rcount].length = htons(4);
-        v4_int_template_out.r[rcount].length = 4;
-	rcount++;
-#endif
 	v4_template.r[rcount].type = htons(NF9_IP_PROTOCOL_VERSION);
 	v4_template.r[rcount].length = htons(1);
 	v4_int_template.r[rcount].length = 1;
@@ -1096,27 +1070,16 @@ nf9_init_template(void)
           v4_int_template_out.r[rcount].length = 1;
           rcount++;
         }
-        if (config.nfprobe_what_to_count & COUNT_CLASS) {
-          v4_template.r[rcount].type = htons(NF9_FLOW_APPLICATION_ID);
-          v4_template.r[rcount].length = htons(4);
-          v4_int_template.r[rcount].handler = flow_to_flowset_class_handler;
-          v4_int_template.r[rcount].length = 4;
-          v4_template_out.r[rcount].type = htons(NF9_FLOW_APPLICATION_ID);
-          v4_template_out.r[rcount].length = htons(4);
-          v4_int_template_out.r[rcount].handler = flow_to_flowset_class_handler;
-          v4_int_template_out.r[rcount].length = 4;
-          rcount++;
-        }
 #if defined (WITH_NDPI)
 	if (config.nfprobe_what_to_count_2 & COUNT_NDPI_CLASS) { 
 	  v4_template.r[rcount].type = htons(NF9_FLOW_APPLICATION_ID);
-	  v4_template.r[rcount].length = htons(4);
+	  v4_template.r[rcount].length = htons(5);
 	  v4_int_template.r[rcount].handler = flow_to_flowset_ndpi_class_handler;
-	  v4_int_template.r[rcount].length = 4;
+	  v4_int_template.r[rcount].length = 5;
 	  v4_template_out.r[rcount].type = htons(NF9_FLOW_APPLICATION_ID);
-	  v4_template_out.r[rcount].length = htons(4);
+	  v4_template_out.r[rcount].length = htons(5);
 	  v4_int_template_out.r[rcount].handler = flow_to_flowset_ndpi_class_handler;
-	  v4_int_template_out.r[rcount].length = 4;
+	  v4_int_template_out.r[rcount].length = 5;
 	  rcount++;
 	}
 #endif
@@ -1187,22 +1150,39 @@ nf9_init_template(void)
 	  rcount++;
         }
         else if ((config.nfprobe_version == 9 && !config.timestamps_secs) || config.nfprobe_version == 10) {
-          v6_template.r[rcount].type = htons(NF9_LAST_SWITCHED_MSEC);
-          v6_template.r[rcount].length = htons(8);
-          v6_int_template.r[rcount].length = 8;
-          v6_template_out.r[rcount].type = htons(NF9_LAST_SWITCHED_MSEC);
-          v6_template_out.r[rcount].length = htons(8);
-          v6_int_template_out.r[rcount].length = 8;
-          rcount++;
-          v6_template.r[rcount].type = htons(NF9_FIRST_SWITCHED_MSEC);
-          v6_template.r[rcount].length = htons(8);
-          v6_int_template.r[rcount].length = 8;
-          v6_template_out.r[rcount].type = htons(NF9_FIRST_SWITCHED_MSEC);
-          v6_template_out.r[rcount].length = htons(8);
-          v6_int_template_out.r[rcount].length = 8;
-          rcount++;
+	  if (!config.nfprobe_tstamp_usec) {
+	    v6_template.r[rcount].type = htons(NF9_LAST_SWITCHED_MSEC);
+	    v6_template.r[rcount].length = htons(8);
+	    v6_int_template.r[rcount].length = 8;
+	    v6_template_out.r[rcount].type = htons(NF9_LAST_SWITCHED_MSEC);
+	    v6_template_out.r[rcount].length = htons(8);
+	    v6_int_template_out.r[rcount].length = 8;
+	    rcount++;
+	    v6_template.r[rcount].type = htons(NF9_FIRST_SWITCHED_MSEC);
+	    v6_template.r[rcount].length = htons(8);
+	    v6_int_template.r[rcount].length = 8;
+	    v6_template_out.r[rcount].type = htons(NF9_FIRST_SWITCHED_MSEC);
+	    v6_template_out.r[rcount].length = htons(8);
+	    v6_int_template_out.r[rcount].length = 8;
+	    rcount++;
+	  }
+	  else {
+	    v6_template.r[rcount].type = htons(NF9_LAST_SWITCHED_USEC);
+	    v6_template.r[rcount].length = htons(16);
+	    v6_int_template.r[rcount].length = 16;
+	    v6_template_out.r[rcount].type = htons(NF9_LAST_SWITCHED_USEC);
+	    v6_template_out.r[rcount].length = htons(16);
+	    v6_int_template_out.r[rcount].length = 16;
+	    rcount++;
+	    v6_template.r[rcount].type = htons(NF9_FIRST_SWITCHED_USEC);
+	    v6_template.r[rcount].length = htons(16);
+	    v6_int_template.r[rcount].length = 16;
+	    v6_template_out.r[rcount].type = htons(NF9_FIRST_SWITCHED_USEC);
+	    v6_template_out.r[rcount].length = htons(16);
+	    v6_int_template_out.r[rcount].length = 16;
+	    rcount++;
+	  }
         }
-#if defined HAVE_64BIT_COUNTERS
         v6_template.r[rcount].type = htons(NF9_IN_BYTES);
         v6_template.r[rcount].length = htons(8);
         v6_int_template.r[rcount].length = 8;
@@ -1217,26 +1197,6 @@ nf9_init_template(void)
         v6_template_out.r[rcount].length = htons(8);
         v6_int_template_out.r[rcount].length = 8;
         rcount++;
-#else
-	v6_template.r[rcount].type = htons(NF9_IN_BYTES);
-	v6_template.r[rcount].length = htons(4);
-	v6_int_template.r[rcount].length = 4;
-	// Cisco doesn't appear to do that (yet?)
-        // v6_template_out.r[rcount].type = htons(NF9_OUT_BYTES);
-        v6_template_out.r[rcount].type = htons(NF9_IN_BYTES);
-        v6_template_out.r[rcount].length = htons(4);
-        v6_int_template_out.r[rcount].length = 4;
-	rcount++;
-	v6_template.r[rcount].type = htons(NF9_IN_PACKETS);
-	v6_template.r[rcount].length = htons(4);
-	v6_int_template.r[rcount].length = 4;
-	// Cisco doesn't appear to do that (yet?)
-        // v6_template_out.r[rcount].type = htons(NF9_OUT_PACKETS);
-        v6_template_out.r[rcount].type = htons(NF9_IN_PACKETS);
-        v6_template_out.r[rcount].length = htons(4);
-        v6_int_template_out.r[rcount].length = 4;
-	rcount++;
-#endif
 	v6_template.r[rcount].type = htons(NF9_IP_PROTOCOL_VERSION);
 	v6_template.r[rcount].length = htons(1);
 	v6_int_template.r[rcount].length = 1;
@@ -1512,27 +1472,16 @@ nf9_init_template(void)
           v6_int_template_out.r[rcount].length = 1;
           rcount++;
         }
-        if (config.nfprobe_what_to_count & COUNT_CLASS) { 
-          v6_template.r[rcount].type = htons(NF9_FLOW_APPLICATION_ID);
-          v6_template.r[rcount].length = htons(4);
-          v6_int_template.r[rcount].handler = flow_to_flowset_class_handler;
-          v6_int_template.r[rcount].length = 4;
-          v6_template_out.r[rcount].type = htons(NF9_FLOW_APPLICATION_ID);
-          v6_template_out.r[rcount].length = htons(4);
-          v6_int_template_out.r[rcount].handler = flow_to_flowset_class_handler;
-          v6_int_template_out.r[rcount].length = 4;
-          rcount++;
-        }
 #if defined (WITH_NDPI)
 	if (config.nfprobe_what_to_count_2 & COUNT_NDPI_CLASS) { 
 	  v6_template.r[rcount].type = htons(NF9_FLOW_APPLICATION_ID);
-	  v6_template.r[rcount].length = htons(4);
+	  v6_template.r[rcount].length = htons(5);
 	  v6_int_template.r[rcount].handler = flow_to_flowset_ndpi_class_handler;
-	  v6_int_template.r[rcount].length = 4;
+	  v6_int_template.r[rcount].length = 5;
 	  v6_template_out.r[rcount].type = htons(NF9_FLOW_APPLICATION_ID);
-	  v6_template_out.r[rcount].length = htons(4);
+	  v6_template_out.r[rcount].length = htons(5);
 	  v6_int_template_out.r[rcount].handler = flow_to_flowset_ndpi_class_handler;
-	  v6_int_template_out.r[rcount].length = 4;
+	  v6_int_template_out.r[rcount].length = 5;
 	  rcount++;
 	}
 #endif
@@ -1648,12 +1597,12 @@ nf9_init_options_template(void)
         class_option_int_template.r[rcount].length = 4;
         rcount++;
         class_option_template.r[rcount].type = htons(NF9_FLOW_APPLICATION_ID);
-        class_option_template.r[rcount].length = htons(4);
-        class_option_int_template.r[rcount].length = 4;
+        class_option_template.r[rcount].length = htons(5);
+        class_option_int_template.r[rcount].length = 5;
         rcount++;
         class_option_template.r[rcount].type = htons(NF9_FLOW_APPLICATION_NAME);
-        class_option_template.r[rcount].length = htons(16);
-        class_option_int_template.r[rcount].length = 16;
+        class_option_template.r[rcount].length = htons(MAX_PROTOCOL_LEN);
+        class_option_int_template.r[rcount].length = MAX_PROTOCOL_LEN;
         rcount++;
         class_option_template.h.c.flowset_id = htons(flowset_id);
         class_option_template.h.c.length = htons( sizeof(struct NF9_OPTIONS_TEMPLATE_FLOWSET_HEADER) + (sizeof(struct NF9_TEMPLATE_FLOWSET_RECORD) * rcount) );
@@ -1754,40 +1703,47 @@ nf_flow_to_flowset(const struct FLOW *flow, u_char *packet, u_int len,
 	    ftoft_ptr_0 += 4;
 	  }
 	  else if ((config.nfprobe_version == 9 && !config.timestamps_secs) || config.nfprobe_version == 10) {
-	    u_int64_t tstamp_msec;
+	    if (!config.nfprobe_tstamp_usec) {
+	      u_int64_t tstamp_msec;
 
-	    tstamp_msec = flow->flow_last.tv_sec;
-	    tstamp_msec = tstamp_msec * 1000;
-	    tstamp_msec += (flow->flow_last.tv_usec / 1000);
-            rec64 = pmXXX_htonll(tstamp_msec);
-            memcpy(ftoft_ptr_0, &rec64, 8);
-            ftoft_ptr_0 += 8;
+	      tstamp_msec = flow->flow_last.tv_sec;
+	      tstamp_msec = tstamp_msec * 1000;
+	      tstamp_msec += (flow->flow_last.tv_usec / 1000);
+              rec64 = pm_htonll(tstamp_msec);
+              memcpy(ftoft_ptr_0, &rec64, 8);
+              ftoft_ptr_0 += 8;
 
-            tstamp_msec = flow->flow_start.tv_sec;
-            tstamp_msec = tstamp_msec * 1000;
-            tstamp_msec += (flow->flow_start.tv_usec / 1000);
-            rec64 = pmXXX_htonll(tstamp_msec);
-            memcpy(ftoft_ptr_0, &rec64, 8);
-            ftoft_ptr_0 += 8;
+              tstamp_msec = flow->flow_start.tv_sec;
+              tstamp_msec = tstamp_msec * 1000;
+              tstamp_msec += (flow->flow_start.tv_usec / 1000);
+              rec64 = pm_htonll(tstamp_msec);
+              memcpy(ftoft_ptr_0, &rec64, 8);
+              ftoft_ptr_0 += 8;
+	    }
+	    else {
+	      rec64 = pm_htonll(flow->flow_last.tv_sec);
+	      memcpy(ftoft_ptr_0, &rec64, 8);
+	      ftoft_ptr_0 += 8;
+	      rec64 = pm_htonll(flow->flow_last.tv_usec);
+	      memcpy(ftoft_ptr_0, &rec64, 8);
+	      ftoft_ptr_0 += 8;
+
+	      rec64 = pm_htonll(flow->flow_start.tv_sec);
+	      memcpy(ftoft_ptr_0, &rec64, 8);
+	      ftoft_ptr_0 += 8;
+	      rec64 = pm_htonll(flow->flow_start.tv_usec);
+	      memcpy(ftoft_ptr_0, &rec64, 8);
+	      ftoft_ptr_0 += 8;
+	    }
 	  }
 
-#if defined HAVE_64BIT_COUNTERS
-          rec64 = pmXXX_htonll(flow->octets[0]);
+          rec64 = pm_htonll(flow->octets[0]);
           memcpy(ftoft_ptr_0, &rec64, 8);
           ftoft_ptr_0 += 8;
 
-          rec64 = pmXXX_htonll(flow->packets[0]);
+          rec64 = pm_htonll(flow->packets[0]);
           memcpy(ftoft_ptr_0, &rec64, 8);
           ftoft_ptr_0 += 8;
-#else
-	  rec32 = htonl(flow->octets[0]);
-	  memcpy(ftoft_ptr_0, &rec32, 4);
-	  ftoft_ptr_0 += 4;
-
-	  rec32 = htonl(flow->packets[0]);
-  	  memcpy(ftoft_ptr_0, &rec32, 4);
-	  ftoft_ptr_0 += 4;
-#endif
 
           switch (flow->af) {
           case AF_INET:
@@ -1868,40 +1824,47 @@ nf_flow_to_flowset(const struct FLOW *flow, u_char *packet, u_int len,
 	    ftoft_ptr_1 += 4;
 	  }
           else if ((config.nfprobe_version == 9 && !config.timestamps_secs) || config.nfprobe_version == 10) {
-            u_int64_t tstamp_msec;
+	    if (!config.nfprobe_tstamp_usec) {
+              u_int64_t tstamp_msec;
 
-            tstamp_msec = flow->flow_last.tv_sec;
-            tstamp_msec = tstamp_msec * 1000;
-            tstamp_msec += (flow->flow_last.tv_usec / 1000);
-            rec64 = pmXXX_htonll(tstamp_msec);
-            memcpy(ftoft_ptr_1, &rec64, 8);
-            ftoft_ptr_1 += 8;
+              tstamp_msec = flow->flow_last.tv_sec;
+              tstamp_msec = tstamp_msec * 1000;
+              tstamp_msec += (flow->flow_last.tv_usec / 1000);
+              rec64 = pm_htonll(tstamp_msec);
+              memcpy(ftoft_ptr_1, &rec64, 8);
+              ftoft_ptr_1 += 8;
 
-            tstamp_msec = flow->flow_start.tv_sec;
-            tstamp_msec = tstamp_msec * 1000;
-            tstamp_msec += (flow->flow_start.tv_usec / 1000);
-            rec64 = pmXXX_htonll(tstamp_msec);
-            memcpy(ftoft_ptr_1, &rec64, 8);
-            ftoft_ptr_1 += 8;
+              tstamp_msec = flow->flow_start.tv_sec;
+              tstamp_msec = tstamp_msec * 1000;
+              tstamp_msec += (flow->flow_start.tv_usec / 1000);
+              rec64 = pm_htonll(tstamp_msec);
+              memcpy(ftoft_ptr_1, &rec64, 8);
+              ftoft_ptr_1 += 8;
+	    }
+	    else {
+	      rec64 = pm_htonll(flow->flow_last.tv_sec);
+	      memcpy(ftoft_ptr_1, &rec64, 8);
+	      ftoft_ptr_1 += 8;
+	      rec64 = pm_htonll(flow->flow_last.tv_usec);
+	      memcpy(ftoft_ptr_1, &rec64, 8);
+	      ftoft_ptr_1 += 8;
+
+	      rec64 = pm_htonll(flow->flow_start.tv_sec);
+	      memcpy(ftoft_ptr_1, &rec64, 8);
+	      ftoft_ptr_1 += 8;
+	      rec64 = pm_htonll(flow->flow_start.tv_usec);
+	      memcpy(ftoft_ptr_1, &rec64, 8);
+	      ftoft_ptr_1 += 8;
+	    }
           }
 
-#if defined HAVE_64BIT_COUNTERS
-          rec64 = pmXXX_htonll(flow->octets[1]);
+          rec64 = pm_htonll(flow->octets[1]);
           memcpy(ftoft_ptr_1, &rec64, 8);
           ftoft_ptr_1 += 8;
 
-          rec64 = pmXXX_htonll(flow->packets[1]);
+          rec64 = pm_htonll(flow->packets[1]);
           memcpy(ftoft_ptr_1, &rec64, 8);
           ftoft_ptr_1 += 8;
-#else
-	  rec32 = htonl(flow->octets[1]);
-	  memcpy(ftoft_ptr_1, &rec32, 4);
-	  ftoft_ptr_1 += 4;
-
-	  rec32 = htonl(flow->packets[1]);
-	  memcpy(ftoft_ptr_1, &rec32, 4);
-	  ftoft_ptr_1 += 4;
-#endif
 
           switch (flow->af) {
           case AF_INET:
@@ -2070,13 +2033,25 @@ nf_class_option_to_flowset(u_int idx, u_char *packet, u_int len, const struct ti
           break;
         }
 
-        /* NF9_FLOW_APPLICATION_ID */
-        memcpy(ftoft_ptr_0, &class[idx].id, 4);
-        ftoft_ptr_0 += 4;
+        /* NF9_FLOW_APPLICATION_ID (ClassID) */
+        {
+	  u_int8_t ie95_classId = 0x16; /* nDPI */
+
+          memcpy(ftoft_ptr_0, &ie95_classId, 1);
+          ftoft_ptr_0 += 1;
+        }
+
+        /* NF9_FLOW_APPLICATION_ID (SelectID) */
+        {
+	  u_int32_t ie95_selectId = htonl(class[idx].id);
+
+	  memcpy(ftoft_ptr_0, &ie95_selectId, 4);
+          ftoft_ptr_0 += 4;
+	}
 
         /* NF9_FLOW_APPLICATION_NAME */
-        strlcpy(ftoft_ptr_0, class[idx].protocol, 16);
-        ftoft_ptr_0 += 16;
+        strlcpy(ftoft_ptr_0, class[idx].protocol, MAX_PROTOCOL_LEN);
+        ftoft_ptr_0 += MAX_PROTOCOL_LEN;
 
         freclen = class_option_int_template.tot_rec_len;
 
@@ -2163,7 +2138,7 @@ nf_exporter_option_to_flowset(u_char *packet, u_int len, const struct timeval *s
  * Returns number of packets sent or -1 on error
  */
 int
-send_netflow_v9(struct FLOW **flows, int num_flows, int nfsock,
+send_netflow_v9(struct FLOW **flows, int num_flows, int nfsock, void *dtls,
     u_int64_t *flows_exported, struct timeval *system_boot_time,
     int verbose_flag, u_int8_t unused, u_int32_t source_id)
 {
@@ -2175,7 +2150,11 @@ send_netflow_v9(struct FLOW **flows, int num_flows, int nfsock,
 	u_int num_class, class_j;
 	int direction, new_direction;
 	socklen_t errsz;
-	int err, r, flow_i, class_i;
+	int err, r, flow_i, class_i, ret;
+
+#ifdef WITH_GNUTLS
+        pm_dtls_peer_t *dtls_peer = dtls;
+#endif
 
 	memset(packet, 0, sizeof(packet));
 	gettimeofday(&now, NULL);
@@ -2260,11 +2239,8 @@ send_netflow_v9(struct FLOW **flows, int num_flows, int nfsock,
 			  send_options = TRUE;
 			  send_sampling_option = TRUE;
 			}
-			if (((config.nfprobe_what_to_count & COUNT_CLASS)
 #if defined (WITH_NDPI) 
-			    || (config.nfprobe_what_to_count_2 & COUNT_NDPI_CLASS)
-#endif
-			    ) && num_class > 0) {
+			if ((config.nfprobe_what_to_count_2 & COUNT_NDPI_CLASS) && num_class > 0) {
                           memcpy(packet + offset, &class_option_template, class_option_template.tot_len);
                           offset += class_option_template.tot_len;
                           flows++;
@@ -2273,6 +2249,7 @@ send_netflow_v9(struct FLOW **flows, int num_flows, int nfsock,
 			  send_options = TRUE;
                           send_class_option = TRUE;
 			}
+#endif
                         if (config.nfprobe_source_ip) {
                           memcpy(packet + offset, &exporter_option_template, exporter_option_template.tot_len);
                           offset += exporter_option_template.tot_len;
@@ -2425,10 +2402,22 @@ send_netflow_v9(struct FLOW **flows, int num_flows, int nfsock,
 		  errsz = sizeof(err);
 		  /* Clear ICMP errors */
 		  getsockopt(nfsock, SOL_SOCKET, SO_ERROR, &err, &errsz); 
-		  if (send(nfsock, packet, (size_t)offset, 0) == -1) {
-		    Log(LOG_WARNING, "WARN ( %s/%s ): send() failed: %s\n", config.name, config.type, strerror(errno));
-		    return (-1);
+
+		  if (!config.nfprobe_dtls) {
+		    ret = send(nfsock, packet, (size_t)offset, 0);
+
+		    if (ret == ERR) {
+		      Log(LOG_WARNING, "WARN ( %s/%s ): send() failed: %s\n", config.name, config.type, strerror(errno));
+		      return ret;
+		    }
 		  }
+#ifdef WITH_GNUTLS
+		  else {
+		    ret = pm_dtls_client_send(dtls_peer, packet, (size_t)offset);
+		    if (ret < 0) return ret;
+		  }
+#endif
+
 		  num_packets++;
 		  nf9_pkts_until_template--;
 		}
