@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2019 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2022 by Paolo Lucente
 */
 
 /*
@@ -57,7 +57,6 @@
 
 #include "pmacct-data.h"
 #include "net_aggr.h"
-#include "ports_aggr.h"
 #include "plugin_hooks.h"
 #include "plugin_common.h"
 
@@ -81,7 +80,7 @@ struct CB_CTXT {
 };
 
 /* Netflow send functions */
-typedef int (netflow_send_func_t)(struct FLOW **, int, int, u_int64_t *, struct timeval *, int, u_int8_t, u_int32_t);
+typedef int (netflow_send_func_t)(struct FLOW **, int, int, void *, u_int64_t *, struct timeval *, int, u_int8_t, u_int32_t);
 
 struct NETFLOW_SENDER {
 	int version;
@@ -100,6 +99,7 @@ static const struct NETFLOW_SENDER nf[] = {
 /* Describes a location where we send NetFlow packets to */
 struct NETFLOW_TARGET {
 	int fd;
+	void *dtls;
 	const struct NETFLOW_SENDER *dialect;
 };
 
@@ -150,6 +150,12 @@ flow_compare(struct FLOW *a, struct FLOW *b)
 	if (a->port[1] != b->port[1])
 		return (ntohs(a->port[1]) > ntohs(b->port[1]) ? 1 : -1);
 
+	if (a->ifindex[0] != b->ifindex[0])
+		return (a->ifindex[0] > b->ifindex[0] ? 1 : -1);
+
+	if (a->ifindex[1] != b->ifindex[1])
+		return (a->ifindex[1] > b->ifindex[1] ? 1 : -1);
+
 	return (0);
 }
 
@@ -255,6 +261,7 @@ static int
 transport_to_flowrec(struct FLOW *flow, struct pkt_data *data, struct pkt_extras *extras, int protocol, int ndx)
 {
  struct pkt_primitives *p = &data->primitives;
+ u_int8_t *icmp_ptr = NULL;
 
  /*
   * XXX to keep flow in proper canonical format, it may be necessary
@@ -275,6 +282,12 @@ transport_to_flowrec(struct FLOW *flow, struct pkt_data *data, struct pkt_extras
     flow->port[ndx] = p->src_port;
     flow->port[ndx ^ 1] = p->dst_port;
     break;
+  case IPPROTO_ICMP:
+  case IPPROTO_ICMPV6:
+    icmp_ptr = (u_int8_t *) &flow->port[ndx ^ 1];
+    icmp_ptr[0] = extras->icmp_type;
+    icmp_ptr[1] = extras->icmp_code;
+    break;
   }
 
   return (0);
@@ -601,7 +614,6 @@ flow_update_expiry(struct FLOWTRACK *ft, struct FLOW *flow)
 {
 	EXPIRY_REMOVE(EXPIRIES, &ft->expiries, flow->expiry);
 
-#if defined HAVE_64BIT_COUNTERS
         if (config.nfprobe_version == 9 || config.nfprobe_version == 10) {
 	  if (flow->octets[0] > (1ULL << 63) || flow->octets[1] > (1ULL << 63)) { 
                 flow->expiry->expires_at = 0;
@@ -616,14 +628,6 @@ flow_update_expiry(struct FLOWTRACK *ft, struct FLOW *flow)
                 goto out;
           }
 	}
-#else
-	/* Flows over 2Gb traffic */
-	if (flow->octets[0] > (1U << 31) || flow->octets[1] > (1U << 31)) {
-		flow->expiry->expires_at = 0;
-		flow->expiry->reason = R_OVERBYTES;
-		goto out;
-	}
-#endif
 	
 	/* Flows over maximum life seconds */
 	if (ft->maximum_lifetime != 0 && 
@@ -846,7 +850,7 @@ update_statistics(struct FLOWTRACK *ft, struct FLOW *flow)
 	static double n = 1.0;
 
 	ft->flows_expired++;
-	ft->flows_pp[flow->protocol % 256]++;
+	ft->flows_pp[flow->protocol % DEFAULT_BUCKETS]++;
 
 	tmp = (double)flow->flow_last.tv_sec +
 	    ((double)flow->flow_last.tv_usec / 1000000.0);
@@ -857,15 +861,15 @@ update_statistics(struct FLOWTRACK *ft, struct FLOW *flow)
 
 	update_statistic(&ft->duration, tmp, n);
 	update_statistic(&ft->duration_pp[flow->protocol], tmp, 
-	    (double)ft->flows_pp[flow->protocol % 256]);
+	    (double)ft->flows_pp[flow->protocol % DEFAULT_BUCKETS]);
 
 	tmp = flow->octets[0] + flow->octets[1];
 	update_statistic(&ft->octets, tmp, n);
-	ft->octets_pp[flow->protocol % 256] += tmp;
+	ft->octets_pp[flow->protocol % DEFAULT_BUCKETS] += tmp;
 
 	tmp = flow->packets[0] + flow->packets[1];
 	update_statistic(&ft->packets, tmp, n);
-	ft->packets_pp[flow->protocol % 256] += tmp;
+	ft->packets_pp[flow->protocol % DEFAULT_BUCKETS] += tmp;
 
 	n++;
 }
@@ -1021,15 +1025,19 @@ check_expired(struct FLOWTRACK *ft, struct NETFLOW_TARGET *target, int ex, u_int
                         }
 			else {
 			  r = target->dialect->func(expired_flows, num_expired, 
-			    target->fd, &ft->flows_exported, // &ft->next_datagram_seq,
+			    target->fd, target->dtls, &ft->flows_exported,
 			    &ft->system_boot_time, verbose_flag, engine_type, engine_id);
-			  if (verbose_flag)
-				Log(LOG_DEBUG, "DEBUG ( %s/%s ): Sent %d netflow packets\n", config.name, config.type, r);
+
+			  if (verbose_flag) {
+			    Log(LOG_DEBUG, "DEBUG ( %s/%s ): Sent %d netflow packets\n", config.name, config.type, r);
+			  }
+
 			  if (r > 0) {
-				ft->packets_sent += r;
-				/* XXX what if r < num_expired * 2 ? */
-			  } else {
-				ft->flows_dropped += num_expired * 2;
+			    ft->packets_sent += r;
+			    /* XXX what if r < num_expired * 2 ? */
+			  }
+			  else {
+			    ft->flows_dropped += num_expired * 2;
 			  }
 			}
 		}
@@ -1311,11 +1319,6 @@ parse_engine(char *s, u_int8_t *engine_type, u_int32_t *engine_id)
 {
   char *delim, *ptr;
 
-  if (config.nfprobe_version == 1) {
-    Log(LOG_ERR, "ERROR ( %s/%s ): parse_engine(): NetFlow v1 export does not support nfprobe_engine.\n", config.name, config.type);
-    exit_gracefully(1);
-  }
-
   trim_spaces(s);
   delim = strchr(s, ':');
   
@@ -1348,6 +1351,7 @@ void nfprobe_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   struct pkt_data *data, dummy;
   struct pkt_bgp_primitives dummy_pbgp;
   struct ports_table pt;
+  struct protos_table prt, tost;
   struct pollfd pfd;
   unsigned char *pipebuf;
   int refresh_timeout, ret, num, recv_budget, poll_bypass;
@@ -1363,17 +1367,19 @@ void nfprobe_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   int pollagain = TRUE;
   u_int32_t seq = 1, rg_err_count = 0;
 
-  char *capfile = NULL, dest_addr[256], dest_serv[256];
+  char *capfile = NULL;
   int linktype = 0, i, r, err, always_v6;
   int max_flows, hoplimit;
-  struct sockaddr_storage dest;
   struct FLOWTRACK flowtrack;
-  socklen_t dest_len;
   struct NETFLOW_TARGET target;
   struct CB_CTXT cb_ctxt;
   u_int8_t engine_type = 0;
   u_int32_t engine_id;
 
+  char dest_addr[SRVBUFLEN], dest_serv[SRVBUFLEN];
+  struct sockaddr_storage dest;
+  socklen_t dest_len;
+
   struct extra_primitives extras;
   struct primitives_ptrs prim_ptrs;
 
@@ -1383,6 +1389,10 @@ void nfprobe_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   void *zmq_host = NULL;
 #endif
 
+#ifdef WITH_REDIS
+  struct p_redis_host redis_host;
+#endif
+
   memcpy(&config, cfgptr, sizeof(struct configuration));
   memcpy(&extras, &((struct channels_list_entry *)ptr)->extras, sizeof(struct extra_primitives));
   recollect_pipe_memory(ptr);
@@ -1414,6 +1424,7 @@ void nfprobe_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   signal(SIGCHLD, SIG_IGN);
 	    
   memset(&cb_ctxt, '\0', sizeof(cb_ctxt));
+  memset(empty_mem_area_256b, 0, sizeof(empty_mem_area_256b));
 
   init_flowtrack(&flowtrack);
 
@@ -1437,6 +1448,17 @@ void nfprobe_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   if (config.debug) verbose_flag = TRUE;
   if (config.pcap_savefile) capfile = config.pcap_savefile;
 
+#ifdef WITH_GNUTLS
+  if (config.nfprobe_dtls && !config.dtls_path) {
+    Log(LOG_ERR, "ERROR ( %s/%s ): 'nfprobe_dtls' specified but missing 'dtls_path'. Exiting.\n", config.name, config.type);
+    exit_gracefully(1);
+  }
+
+  if (config.dtls_path) {
+    pm_dtls_init(&config.dtls_globs, config.dtls_path);
+  }
+#endif
+
   dest_len = sizeof(dest);
   if (!config.nfprobe_receiver) config.nfprobe_receiver = default_receiver;
   parse_hostport(config.nfprobe_receiver, (struct sockaddr *)&dest, &dest_len);
@@ -1447,7 +1469,7 @@ sort_version:
   }
 
   if (nf[i].version == -1) {
-    config.nfprobe_version = 5; 
+    config.nfprobe_version = 10; /* default to IPFIX */
     goto sort_version;
   }
   target.dialect = &nf[i];
@@ -1470,9 +1492,15 @@ sort_version:
     }
     target.fd = connsock(&dest, dest_len, hoplimit);
 	
-    if (target.fd != -1)
-      Log(LOG_INFO, "INFO ( %s/%s ): Exporting flows to [%s]:%s\n",
-		    config.name, config.type, dest_addr, dest_serv);
+    if (target.fd != -1) {
+      Log(LOG_INFO, "INFO ( %s/%s ): Exporting flows to [%s]:%s\n", config.name, config.type, dest_addr, dest_serv);
+
+#ifdef WITH_GNUTLS
+      if (config.nfprobe_dtls) {
+	target.dtls = malloc(sizeof(pm_dtls_peer_t));
+      }
+#endif
+    }
   }
 
   /* Main processing loop */
@@ -1484,6 +1512,8 @@ sort_version:
   memset(&nt, 0, sizeof(nt));
   memset(&nc, 0, sizeof(nc));
   memset(&pt, 0, sizeof(pt));
+  memset(&prt, 0, sizeof(prt));
+  memset(&tost, 0, sizeof(tost));
   memset(&dummy, 0, sizeof(dummy));
   memset(&dummy_pbgp, 0, sizeof(dummy_pbgp));
 
@@ -1491,6 +1521,8 @@ sort_version:
   set_net_funcs(&nt);
 
   if (config.ports_file) load_ports(config.ports_file, &pt);
+  if (config.protos_file) load_protos(config.protos_file, &prt);
+  if (config.tos_file) load_tos(config.tos_file, &tost);
 
   pipebuf = (unsigned char *) pm_malloc(config.buffer_size);
   memset(pipebuf, 0, config.buffer_size);
@@ -1516,6 +1548,15 @@ sort_version:
     }
   }
 
+#ifdef WITH_REDIS
+  if (config.redis_host) {
+    char log_id[SHORTBUFLEN];
+
+    snprintf(log_id, sizeof(log_id), "%s/%s", config.name, config.type);
+    p_redis_init(&redis_host, log_id, p_redis_thread_produce_common_plugin_handler);
+  }
+#endif
+
   for(;;) {
     status->wakeup = TRUE;
     poll_bypass = FALSE;
@@ -1527,8 +1568,31 @@ sort_version:
 
     /* Flags set by signal handlers or control socket */
     if (graceful_shutdown_request) {
-      Log(LOG_WARNING, "WARN ( %s/%s ): Shutting down on user request.\n", config.name, config.type);
+#ifdef WITH_GNUTLS
+      if (config.nfprobe_dtls) {
+	pm_dtls_peer_t *dtls_peer = target.dtls;
+
+	if (dtls_peer->conn.do_reconnect && dtls_peer->conn.stage == PM_DTLS_STAGE_UP) {
+	  pm_dtls_client_bye(dtls_peer);
+	}
+
+	if (target.fd != ERR && dtls_peer->conn.stage != PM_DTLS_STAGE_UP) {
+	  pm_dtls_client_init(target.dtls, target.fd, &dest, dest_len, config.nfprobe_dtls_verify_cert);
+	}
+      }
+#endif
+
+      Log(LOG_INFO, "INFO ( %s/%s ): Shutting down on user request.\n", config.name, config.type);
       check_expired(&flowtrack, &target, CE_EXPIRE_ALL, engine_type, engine_id);
+
+#ifdef WITH_GNUTLS
+      if (config.nfprobe_dtls) {
+	pm_dtls_peer_t *dtls_peer = target.dtls;
+
+	pm_dtls_client_bye(dtls_peer);
+      }
+#endif
+
       goto exit_lane;
     }
 
@@ -1544,11 +1608,14 @@ sort_version:
     if (reload_map) {
       load_networks(config.networks_file, &nt, &nc);
       load_ports(config.ports_file, &pt);
+      load_protos(config.protos_file, &prt);
+      load_tos(config.tos_file, &tost);
+
       reload_map = FALSE;
     }
 
     if (reload_log) {
-      reload_logs();
+      reload_logs(NULL);
       reload_log = FALSE;
     }
 
@@ -1631,8 +1698,16 @@ sort_version:
 	  (*net_funcs[num])(&nt, &nc, &data->primitives, prim_ptrs.pbgp, &nfd);
 
 	if (config.ports_file) {
-	  if (!pt.table[data->primitives.src_port]) data->primitives.src_port = 0;
-	  if (!pt.table[data->primitives.dst_port]) data->primitives.dst_port = 0;
+	  if (!pt.table[data->primitives.src_port]) data->primitives.src_port = PM_L4_PORT_OTHERS;
+	  if (!pt.table[data->primitives.dst_port]) data->primitives.dst_port = PM_L4_PORT_OTHERS;
+	}
+
+	if (config.protos_file) {
+	  if (!prt.table[data->primitives.proto]) data->primitives.proto = PM_IP_PROTO_OTHERS;
+	}
+
+	if (config.tos_file) {
+	  if (!tost.table[data->primitives.tos]) data->primitives.tos = PM_IP_TOS_OTHERS;
 	}
 
 	prim_ptrs.data = data;
@@ -1660,6 +1735,20 @@ handle_flow_expiration:
      */
     if (flowtrack.num_flows > max_flows || next_expire(&flowtrack) == 0) {
 expiry_check:
+#ifdef WITH_GNUTLS
+      if (config.nfprobe_dtls) {
+	pm_dtls_peer_t *dtls_peer = target.dtls;
+
+	if (dtls_peer->conn.do_reconnect && dtls_peer->conn.stage == PM_DTLS_STAGE_UP) {
+	  pm_dtls_client_bye(dtls_peer);
+	}
+
+	if (target.fd != ERR && dtls_peer->conn.stage != PM_DTLS_STAGE_UP) {
+          pm_dtls_client_init(target.dtls, target.fd, &dest, dest_len, config.nfprobe_dtls_verify_cert);
+	}
+      }
+#endif
+
       /*
        * If we are reading from a capture file, we never
        * expire flows based on time - instead we only 
@@ -1674,11 +1763,19 @@ expiry_check:
 	  sleep(5);
 	  target.fd = connsock(&dest, dest_len, hoplimit);
 
-	  if (target.fd != -1)
-	    Log(LOG_INFO, "INFO ( %s/%s ): Exporting flows to [%s]:%s\n",
-			config.name, config.type, dest_addr, dest_serv);
+	  if (target.fd != -1) {
+	    Log(LOG_INFO, "INFO ( %s/%s ): Exporting flows to [%s]:%s\n", config.name, config.type, dest_addr, dest_serv);
+	  }
 	}
       }
+
+#ifdef WITH_GNUTLS
+      if (config.nfprobe_dtls) {
+	pm_dtls_peer_t *dtls_peer = target.dtls;
+
+	pm_dtls_client_bye(dtls_peer);
+      }
+#endif
 	
       /*
        * If we are over max_flows, force-expire the oldest 
