@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2019 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2022 by Paolo Lucente
 */
 
 /*
@@ -21,7 +21,6 @@
 
 /* includes */
 #include "pmacct.h"
-#include "addr.h"
 #include "nfacctd.h"
 #include "pmacct-data.h"
 
@@ -105,7 +104,7 @@ struct template_cache_entry *insert_template(struct template_hdr_v9 *hdr, struct
   ptr->template_type = 0;
   ptr->num = num;
 
-  log_template_header(ptr, pptrs, tpl_type, sid, version);
+  log_template_header(ptr, (struct sockaddr *)pptrs->f_agent, tpl_type, sid, version);
 
   count = off = 0;
   tpl = (u_char *) hdr;
@@ -117,7 +116,7 @@ struct template_cache_entry *insert_template(struct template_hdr_v9 *hdr, struct
     if (count >= TPL_LIST_ENTRIES) {
       notify_malf_packet(LOG_INFO, "INFO", "insert_template(): unable to read Data Template (too long)",
 			 (struct sockaddr *) pptrs->f_agent, seq);
-      xflow_tot_bad_datagrams++;
+      xflow_status_table.tot_bad_datagrams++;
       free(ptr);
       return NULL;
     }
@@ -125,7 +124,7 @@ struct template_cache_entry *insert_template(struct template_hdr_v9 *hdr, struct
     if (off >= len) {
       notify_malf_packet(LOG_INFO, "INFO", "insert_template(): unable to read Data Template (malformed)",
 			 (struct sockaddr *) pptrs->f_agent, seq);
-      xflow_tot_bad_datagrams++;
+      xflow_status_table.tot_bad_datagrams++;
       free(ptr);
       return NULL;
     }
@@ -792,7 +791,7 @@ struct template_cache_entry *refresh_template(struct template_hdr_v9 *hdr, struc
   tpl->num = num;
   tpl->next = next;
 
-  log_template_header(tpl, pptrs, tpl_type, sid, version);
+  log_template_header(tpl, (struct sockaddr *)pptrs->f_agent, tpl_type, sid, version);
 
   count = off = 0;
   ptr = (u_char *) hdr;
@@ -804,14 +803,14 @@ struct template_cache_entry *refresh_template(struct template_hdr_v9 *hdr, struc
     if (count >= TPL_LIST_ENTRIES) {
       notify_malf_packet(LOG_INFO, "INFO", "refresh_template(): unable to read Data Template (too long)",
 			 (struct sockaddr *) pptrs->f_agent, seq);
-      xflow_tot_bad_datagrams++;
+      xflow_status_table.tot_bad_datagrams++;
       return NULL;
     }
 
     if (off >= len) {
       notify_malf_packet(LOG_INFO, "INFO", "refresh_template(): unable to read Data Template (malformed)",
                          (struct sockaddr *) pptrs->f_agent, seq);
-      xflow_tot_bad_datagrams++;
+      xflow_status_table.tot_bad_datagrams++;
       memcpy(tpl, &backup, sizeof(struct template_cache_entry));
       return NULL;
     }
@@ -892,13 +891,13 @@ struct template_cache_entry *refresh_template(struct template_hdr_v9 *hdr, struc
   return tpl;
 }
 
-void log_template_header(struct template_cache_entry *tpl, struct packet_ptrs *pptrs, u_int16_t tpl_type, u_int32_t sid, u_int8_t version)
+void log_template_header(struct template_cache_entry *tpl, struct sockaddr *agent, u_int16_t tpl_type, u_int32_t sid, u_int8_t version)
 {
   struct host_addr a;
   char agent_addr[50];
   u_int16_t agent_port;
 
-  sa_to_addr((struct sockaddr *)pptrs->f_agent, &a, &agent_port);
+  sa_to_addr(agent, &a, &agent_port);
   addr_to_str(agent_addr, &a);
 
   Log(LOG_DEBUG, "DEBUG ( %s/core ): NfV%u agent         : %s:%u\n", config.name, version, agent_addr, sid);
@@ -1006,7 +1005,7 @@ struct template_cache_entry *insert_opt_template(void *hdr, struct packet_ptrs *
   ptr->template_type = 1;
   ptr->num = olen+slen;
 
-  log_template_header(ptr, pptrs, tpl_type, sid, version);
+  log_template_header(ptr, (struct sockaddr *)pptrs->f_agent, tpl_type, sid, version);
 
   off = 0;
   count = ptr->num;
@@ -1019,7 +1018,7 @@ struct template_cache_entry *insert_opt_template(void *hdr, struct packet_ptrs *
     if (off >= len) {
       notify_malf_packet(LOG_INFO, "INFO", "insert_opt_template(): unable to read Options Template Flowset (malformed)",
 			 (struct sockaddr *) pptrs->f_agent, seq);
-      xflow_tot_bad_datagrams++;
+      xflow_status_table.tot_bad_datagrams++;
       free(ptr);
       return NULL;
     }
@@ -1058,7 +1057,7 @@ struct template_cache_entry *insert_opt_template(void *hdr, struct packet_ptrs *
       if (count >= TPL_LIST_ENTRIES) {
 	notify_malf_packet(LOG_INFO, "INFO", "insert_opt_template(): unable to read Options Template (too long)",
 			   (struct sockaddr *) pptrs->f_agent, seq);
-	xflow_tot_bad_datagrams++;
+	xflow_status_table.tot_bad_datagrams++;
 	free(ptr);
 	return NULL;
       }
@@ -1129,7 +1128,7 @@ struct template_cache_entry *refresh_opt_template(void *hdr, struct template_cac
   tpl->num = olen+slen;
   tpl->next = next;
 
-  log_template_header(tpl, pptrs, tpl_type, sid, version);  
+  log_template_header(tpl, (struct sockaddr *)pptrs->f_agent, tpl_type, sid, version);  
 
   off = 0;
   count = tpl->num;
@@ -1142,7 +1141,7 @@ struct template_cache_entry *refresh_opt_template(void *hdr, struct template_cac
     if (off >= len) {
       notify_malf_packet(LOG_INFO, "INFO", "refresh_opt_template(): unable to read Options Template Flowset (malformed)",
 			 (struct sockaddr *) pptrs->f_agent, seq);
-      xflow_tot_bad_datagrams++;
+      xflow_status_table.tot_bad_datagrams++;
       memcpy(tpl, &backup, sizeof(struct template_cache_entry));
       return NULL;
     }
@@ -1180,7 +1179,7 @@ struct template_cache_entry *refresh_opt_template(void *hdr, struct template_cac
       if (count >= TPL_LIST_ENTRIES) {
 	notify_malf_packet(LOG_INFO, "INFO", "refresh_opt_template: unable to read Options Template (too long)",
 			   (struct sockaddr *) pptrs->f_agent, seq);
-	xflow_tot_bad_datagrams++;
+	xflow_status_table.tot_bad_datagrams++;
 	return NULL;
       }
 
@@ -1328,3 +1327,328 @@ struct utpl_field *ext_db_get_next_ie(struct template_cache_entry *ptr, u_int16_
 
   return ext_db_ptr;
 }
+
+u_int16_t calc_template_keylen()
+{
+  return (sizeof(u_int16_t /* template id */) +
+	  sizeof(u_int32_t /* source id */) +
+	  sizeof(struct sockaddr_storage /* sender IP */)); 
+}
+
+struct template_cache_entry *compose_template(struct template_hdr_v9 *hdr,
+					      struct sockaddr *agent, u_int16_t tpl_type,
+					      u_int32_t sid, u_int16_t *pens, u_int8_t version,
+					      u_int16_t len, u_int32_t seq)
+{
+  struct template_cache_entry *tpl;
+  struct template_field_v9 *field;
+  u_int16_t num = ntohs(hdr->num), type, port, off, count;
+  u_int32_t *pen;
+  u_int8_t ipfix_ebit;
+  u_char *tpl_ptr;
+
+  tpl = malloc(sizeof(struct template_cache_entry));
+  if (!tpl) {
+    Log(LOG_ERR, "ERROR ( %s/core ): compose_template(): unable to allocate new Data Template Cache Entry.\n", config.name);
+    return NULL;
+  }
+
+  memset(tpl, 0, sizeof(struct template_cache_entry));
+  sa_to_addr(agent, &tpl->agent, &port);
+  tpl->source_id = sid;
+  tpl->template_id = hdr->template_id;
+  tpl->template_type = 0;
+  tpl->num = num;
+
+  log_template_header(tpl, agent, tpl_type, sid, version);
+
+  count = off = 0;
+  tpl_ptr = (u_char *) hdr;
+  tpl_ptr += NfTplHdrV9Sz;
+  off += NfTplHdrV9Sz;
+  field = (struct template_field_v9 *)tpl_ptr;
+
+  while (count < num) {
+    if (count >= TPL_LIST_ENTRIES) {
+      notify_malf_packet(LOG_INFO, "INFO", "compose_template(): unable to read Data Template (too long)", agent, seq);
+      xflow_status_table.tot_bad_datagrams++;
+      free(tpl);
+      return NULL;
+    }
+
+    if (off >= len) {
+      notify_malf_packet(LOG_INFO, "INFO", "compose_template(): unable to read Data Template (malformed)", agent, seq);
+      xflow_status_table.tot_bad_datagrams++;
+      free(tpl);
+      return NULL;
+    }
+
+    pen = NULL; 
+    ipfix_ebit = FALSE;
+    type = ntohs(field->type);
+
+    if (type & IPFIX_TPL_EBIT && version == 10) {
+      ipfix_ebit = TRUE;
+      type ^= IPFIX_TPL_EBIT;
+      if (pens) (*pens)++;
+      pen = (u_int32_t *) field;
+      pen++;
+    }
+
+    log_template_field(tpl->vlen, pen, type, tpl->len, ntohs(field->len), version);
+
+    /* Let's determine if we use legacy template registry or the
+       new template database (ie. if we have a PEN or high field
+       value, >= 384) */
+    if (type < NF9_MAX_DEFINED_FIELD && !pen) {
+      tpl->tpl[type].off = tpl->len; 
+      tpl->tpl[type].tpl_len = ntohs(field->len);
+
+      if (tpl->vlen) tpl->tpl[type].off = 0;
+
+      if (tpl->tpl[type].tpl_len == IPFIX_VARIABLE_LENGTH) {
+        tpl->tpl[type].len = 0;
+        tpl->vlen = TRUE;
+        tpl->len = 0;
+      }
+      else {
+        tpl->tpl[type].len = tpl->tpl[type].tpl_len;
+        if (!tpl->vlen) tpl->len += tpl->tpl[type].len;
+      }
+      tpl->list[count].ptr = (char *) &tpl->tpl[type];
+      tpl->list[count].type = TPL_TYPE_LEGACY;
+    }
+    else {
+      u_int8_t repeat_id = 0;
+      struct utpl_field *ext_db_tpl = ext_db_get_next_ie(tpl, type, &repeat_id);
+
+      if (ext_db_tpl) {
+	if (pen) ext_db_tpl->pen = ntohl(*pen);
+	ext_db_tpl->type = type;
+	ext_db_tpl->off = tpl->len;
+	ext_db_tpl->tpl_len = ntohs(field->len);
+	ext_db_tpl->repeat_id = repeat_id;
+
+        if (tpl->vlen) ext_db_tpl->off = 0;
+
+	if (ext_db_tpl->tpl_len == IPFIX_VARIABLE_LENGTH) {
+	  ext_db_tpl->len = 0;
+	  tpl->vlen = TRUE;
+	  tpl->len = 0;
+	}
+	else {
+	  ext_db_tpl->len = ext_db_tpl->tpl_len;
+	  if (!tpl->vlen) tpl->len += ext_db_tpl->len;
+	}
+      }
+      tpl->list[count].ptr = (char *) ext_db_tpl;
+      tpl->list[count].type = TPL_TYPE_EXT_DB;
+    }
+
+    count++;
+    off += NfTplFieldV9Sz;
+    if (ipfix_ebit) {
+      field++; /* skip 32-bits ahead */ 
+      off += sizeof(u_int32_t);
+    }
+    field++;
+  }
+
+  log_template_footer(tpl, tpl->len, version);
+
+#ifdef WITH_JANSSON
+  if (config.nfacctd_templates_file)
+    save_template(tpl, config.nfacctd_templates_file);
+#endif
+
+  return tpl;
+}
+ 
+struct template_cache_entry *compose_opt_template(void *hdr, struct sockaddr *agent,
+						  u_int16_t tpl_type, u_int32_t sid, u_int16_t *pens,
+						  u_int8_t version, u_int16_t len, u_int32_t seq)
+{
+  struct options_template_hdr_v9 *hdr_v9 = (struct options_template_hdr_v9 *) hdr;
+  struct options_template_hdr_ipfix *hdr_v10 = (struct options_template_hdr_ipfix *) hdr;
+  struct template_cache_entry *tpl;
+  struct template_field_v9 *field;
+  u_int16_t count, slen, olen, type, port, tid, off;
+  u_int32_t *pen;
+  u_int8_t ipfix_ebit;
+  u_char *tpl_ptr;
+
+  /* NetFlow v9 */
+  if (tpl_type == 1) {
+    tid = hdr_v9->template_id;
+    slen = ntohs(hdr_v9->scope_len)/sizeof(struct template_field_v9);
+    olen = ntohs(hdr_v9->option_len)/sizeof(struct template_field_v9);
+  }
+  /* IPFIX */
+  else if (tpl_type == 3) {
+    tid = hdr_v10->template_id;
+    slen = ntohs(hdr_v10->scope_count);
+    olen = ntohs(hdr_v10->option_count)-slen;
+  }
+  else {
+    Log(LOG_ERR, "ERROR ( %s/core ): Unknown template type (%u).\n", config.name, tpl_type);
+    return NULL;
+  }
+
+  tpl = malloc(sizeof(struct template_cache_entry));
+  if (!tpl) {
+    Log(LOG_ERR, "ERROR ( %s/core ): insert_opt_template(): unable to allocate new Options Template Cache Entry.\n", config.name);
+    return NULL;
+  }
+
+  memset(tpl, 0, sizeof(struct template_cache_entry));
+  sa_to_addr(agent, &tpl->agent, &port);
+  tpl->source_id = sid; 
+  tpl->template_id = tid;
+  tpl->template_type = 1;
+  tpl->num = olen+slen;
+
+  log_template_header(tpl, agent, tpl_type, sid, version);
+
+  off = 0;
+  count = tpl->num;
+  tpl_ptr = (u_char *) hdr;
+  tpl_ptr += NfOptTplHdrV9Sz;
+  off += NfOptTplHdrV9Sz;
+  field = (struct template_field_v9 *)tpl_ptr;
+
+  while (count) {
+    if (off >= len) {
+      notify_malf_packet(LOG_INFO, "INFO", "insert_opt_template(): unable to read Options Template Flowset (malformed)", agent, seq);
+      xflow_status_table.tot_bad_datagrams++;
+      free(tpl);
+      return NULL;
+    }
+
+    pen = NULL;
+    ipfix_ebit = FALSE;
+    type = ntohs(field->type);
+
+    if (type & IPFIX_TPL_EBIT && version == 10) {
+      ipfix_ebit = TRUE;
+      type ^= IPFIX_TPL_EBIT;
+      if (pens) (*pens)++;
+      pen = (u_int32_t *) field;
+      pen++;
+    }
+
+    log_opt_template_field(FALSE, pen, type, tpl->len, ntohs(field->len), version);
+    if (type < NF9_MAX_DEFINED_FIELD && !pen) { 
+      tpl->tpl[type].off = tpl->len;
+      tpl->tpl[type].len = ntohs(field->len);
+      tpl->len += tpl->tpl[type].len;
+    }
+    else {
+      u_int8_t repeat_id = 0;
+      struct utpl_field *ext_db_tpl = ext_db_get_next_ie(tpl, type, &repeat_id);
+
+      if (ext_db_tpl) {
+        if (pen) ext_db_tpl->pen = ntohl(*pen);
+        ext_db_tpl->type = type;
+        ext_db_tpl->off = tpl->len;
+        ext_db_tpl->tpl_len = ntohs(field->len);
+        ext_db_tpl->repeat_id = repeat_id;
+        ext_db_tpl->len = ext_db_tpl->tpl_len;
+      }
+
+      if (count >= TPL_LIST_ENTRIES) {
+	notify_malf_packet(LOG_INFO, "INFO", "insert_opt_template(): unable to read Options Template (too long)", agent, seq);
+	xflow_status_table.tot_bad_datagrams++;
+	free(tpl);
+	return NULL;
+      }
+
+      tpl->list[count].ptr = (char *) ext_db_tpl;
+      tpl->list[count].type = TPL_TYPE_EXT_DB;
+      tpl->len += ext_db_tpl->len;
+    }
+
+    count--;
+    off += NfTplFieldV9Sz;
+    if (ipfix_ebit) {
+      field++; /* skip 32-bits ahead */
+      off += sizeof(u_int32_t);
+    }
+    field++;
+  }
+
+  log_template_footer(tpl, tpl->len, version);
+
+#ifdef WITH_JANSSON
+  if (config.nfacctd_templates_file)
+    save_template(tpl, config.nfacctd_templates_file);
+#endif
+
+  return tpl;
+}
+
+u_char *compose_template_key(pm_hash_serial_t *ser, u_int16_t template_id, struct sockaddr *agent, u_int32_t source_id)
+{
+  pm_hash_key_t *hash_key;
+  u_int16_t hash_keylen;
+
+  hash_keylen = calc_template_keylen();
+  hash_init_serial(ser, hash_keylen);
+  hash_serial_append(ser, (char *)&template_id, sizeof(template_id), FALSE);
+  hash_serial_append(ser, (char *)&source_id, sizeof(source_id), TRUE);
+  hash_serial_append(ser, (char *)agent, sizeof(struct sockaddr_storage), TRUE);
+  hash_key = hash_serial_get_key(ser);
+
+  return hash_key_get_val(hash_key);
+}
+
+struct template_cache_entry *handle_template_v2(struct template_hdr_v9 *hdr, struct packet_ptrs *pptrs, u_int16_t tpl_type,
+						u_int32_t sid, u_int16_t *pens, u_int16_t len, u_int32_t seq)
+{
+  struct template_cache_entry *tpl = NULL;
+  u_int8_t version = 0;
+  int ret;
+
+  pm_hash_serial_t hash_serializer;
+  u_char *hash_keyval;
+
+  if (pens) {
+    *pens = FALSE;
+  }
+
+  if (tpl_type == 0 || tpl_type == 1) {
+    version = 9;
+  }
+  else if (tpl_type == 2 || tpl_type == 3) {
+    version = 10;
+  }
+
+  hash_keyval = compose_template_key(&hash_serializer, hdr->template_id, (struct sockaddr *)pptrs->f_agent, sid);
+
+  /* 0 NetFlow v9, 2 IPFIX */
+  if (tpl_type == 0 || tpl_type == 2) {
+    tpl = compose_template(hdr, (struct sockaddr *)pptrs->f_agent, tpl_type, sid, pens, version, len, seq);
+
+    ret = cdada_map_insert_replace(tpl_data_map, hash_keyval, tpl);
+    if (ret != CDADA_SUCCESS) {
+      Log(LOG_WARNING, "WARN ( %s/core ): Unable to insert / refresh template in tpl_data_map\n", config.name);
+      goto exit_lane;
+    }
+  }
+  /* 1 NetFlow v9, 3 IPFIX */
+  else if (tpl_type == 1 || tpl_type == 3) {
+    tpl = compose_opt_template(hdr, (struct sockaddr *)pptrs->f_agent, tpl_type, sid, pens, version, len, seq);
+
+    ret = cdada_map_insert_replace(tpl_data_map, hash_keyval, tpl);
+    if (ret != CDADA_SUCCESS) {
+      Log(LOG_WARNING, "WARN ( %s/core ): Unable to insert / refresh template in tpl_opt_map\n", config.name);
+      goto exit_lane;
+    }
+  }
+
+  exit_lane:
+
+  /* freeing hash key */
+  hash_destroy_serial(&hash_serializer);
+
+  return tpl;
+}
