@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2020 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2022 by Paolo Lucente
 */
 
 /*
@@ -25,6 +25,7 @@
 extern struct channels_list_entry channels_list[MAX_N_PLUGINS]; /* communication channels: core <-> plugins */
 extern pkt_handler phandler[N_PRIMITIVES];
 
+extern void warn_unsupported_packet_handler(u_int64_t, u_int64_t);
 extern void evaluate_packet_handlers(); 
 extern void src_mac_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void dst_mac_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
@@ -33,7 +34,7 @@ extern void cos_handler(struct channels_list_entry *, struct packet_ptrs *, char
 extern void etype_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void mpls_label_top_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void mpls_label_bottom_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
-extern void mpls_stack_depth_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+extern void mpls_label_stack_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void src_host_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void dst_host_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void src_port_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
@@ -49,6 +50,7 @@ extern void tunnel_ip_proto_handler(struct channels_list_entry *, struct packet_
 extern void tunnel_ip_tos_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void tunnel_src_port_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void tunnel_dst_port_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+extern void tunnel_tcp_flags_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void vxlan_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void counters_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void counters_renormalize_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
@@ -57,7 +59,6 @@ extern void time_pcap_handler(struct channels_list_entry *, struct packet_ptrs *
 extern void post_tag_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void post_tag2_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void flows_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
-extern void class_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void in_iface_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void out_iface_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void sampling_rate_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
@@ -72,6 +73,8 @@ extern void mpls_vpn_rd_frommap_handler(struct channels_list_entry *, struct pac
 extern void NF_src_mac_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void NF_dst_mac_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void NF_vlan_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+extern void NF_in_vlan_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+extern void NF_out_vlan_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void NF_cos_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void NF_etype_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void NF_src_host_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
@@ -89,6 +92,7 @@ extern void NF_peer_dst_ip_handler(struct channels_list_entry *, struct packet_p
 extern void NF_ip_tos_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void NF_ip_proto_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void NF_tcp_flags_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+extern void NF_fwd_status_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void NF_counters_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void NF_time_msecs_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void NF_time_secs_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
@@ -104,9 +108,10 @@ extern void NF_post_nat_dst_host_handler(struct channels_list_entry *, struct pa
 extern void NF_post_nat_src_port_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void NF_post_nat_dst_port_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void NF_nat_event_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+extern void NF_fw_event_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+extern void NF_mpls_label_stack_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void NF_mpls_label_top_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void NF_mpls_label_bottom_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
-extern void NF_mpls_stack_depth_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void NF_mpls_vpn_id_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void NF_mpls_vpn_rd_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void NF_mpls_pw_id_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
@@ -143,6 +148,8 @@ extern void igp_peer_dst_ip_handler(struct channels_list_entry *, struct packet_
 extern void SF_src_mac_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void SF_dst_mac_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void SF_vlan_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+extern void SF_in_vlan_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+extern void SF_out_vlan_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void SF_cos_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void SF_etype_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void SF_src_host_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
@@ -164,7 +171,6 @@ extern void SF_counters_renormalize_handler(struct channels_list_entry *, struct
 extern void SF_counters_map_renormalize_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void SF_tag_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void SF_tag2_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
-extern void SF_class_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void SF_in_iface_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void SF_out_iface_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void SF_sampling_rate_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
@@ -189,11 +195,12 @@ extern void SF_tunnel_ip_proto_handler(struct channels_list_entry *, struct pack
 extern void SF_tunnel_ip_tos_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void SF_tunnel_src_port_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void SF_tunnel_dst_port_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+extern void SF_tunnel_tcp_flags_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void SF_vxlan_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void SF_tee_payload_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void SF_mpls_label_top_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void SF_mpls_label_bottom_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
-extern void SF_mpls_stack_depth_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+extern void SF_mpls_label_stack_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void SF_mpls_pw_id_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void SF_custom_primitives_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 
