@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2020 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2022 by Paolo Lucente
 */
 
 /*
@@ -21,7 +21,6 @@
 
 /* includes */
 #include "pmacct.h"
-#include "addr.h"
 #include "pmacct-data.h"
 #include "plugin_common.h"
 #include "plugin_cmn_json.h"
@@ -61,7 +60,12 @@ void compose_json(u_int64_t wtc, u_int64_t wtc_2)
   }
 
   if (wtc_2 & COUNT_LABEL) {
-    cjhandler[idx] = compose_json_label;
+    if (config.pretag_label_encode_as_map) {
+      cjhandler[idx] = compose_json_map_label;
+    }
+    else {
+      cjhandler[idx] = compose_json_label;
+    }
     idx++;
   }
 
@@ -93,6 +97,11 @@ void compose_json(u_int64_t wtc, u_int64_t wtc_2)
     idx++;
   }
 
+  if (wtc_2 & COUNT_OUT_VLAN) {
+    cjhandler[idx] = compose_json_out_vlan;
+    idx++;
+  }
+
   if (wtc & COUNT_COS) {
     cjhandler[idx] = compose_json_cos;
     idx++;
@@ -309,7 +318,32 @@ void compose_json(u_int64_t wtc, u_int64_t wtc_2)
 #endif
 
   if (wtc & COUNT_TCPFLAGS) {
-    cjhandler[idx] = compose_json_tcp_flags;
+    if (config.tcpflags_encode_as_array) {
+      cjhandler[idx] = compose_json_array_tcpflags;
+    }
+    else {
+      cjhandler[idx] = compose_json_tcp_flags;
+    }
+    idx++;
+  }
+  
+  if (wtc_2 & COUNT_FWD_STATUS) {
+    if (config.fwd_status_encode_as_string) {
+      cjhandler[idx] = compose_json_string_fwd_status;
+    }
+    else {
+      cjhandler[idx] = compose_json_fwd_status;
+    }
+    idx++;
+  }
+
+  if (wtc_2 & COUNT_MPLS_LABEL_STACK) {
+    if (config.mpls_label_stack_encode_as_array) {
+      cjhandler[idx] = compose_json_array_mpls_label_stack;
+    }
+    else {
+      cjhandler[idx] = compose_json_mpls_label_stack;
+    }
     idx++;
   }
 
@@ -358,6 +392,11 @@ void compose_json(u_int64_t wtc, u_int64_t wtc_2)
     idx++;
   }
 
+  if (wtc_2 & COUNT_FW_EVENT) {
+    cjhandler[idx] = compose_json_fw_event;
+    idx++;
+  }
+
   if (wtc_2 & COUNT_MPLS_LABEL_TOP) {
     cjhandler[idx] = compose_json_mpls_label_top;
     idx++;
@@ -368,11 +407,6 @@ void compose_json(u_int64_t wtc, u_int64_t wtc_2)
     idx++;
   }
 
-  if (wtc_2 & COUNT_MPLS_STACK_DEPTH) {
-    cjhandler[idx] = compose_json_mpls_stack_depth;
-    idx++;
-  }
-
   if (wtc_2 & COUNT_TUNNEL_SRC_MAC) {
     cjhandler[idx] = compose_json_tunnel_src_mac;
     idx++;
@@ -413,6 +447,16 @@ void compose_json(u_int64_t wtc, u_int64_t wtc_2)
     idx++;
   }
 
+  if (wtc_2 & COUNT_TUNNEL_TCPFLAGS) {
+    if (config.tcpflags_encode_as_array) {
+      cjhandler[idx] = compose_json_array_tunnel_tcp_flags;
+    }
+    else {
+      cjhandler[idx] = compose_json_tunnel_tcp_flags;
+    }
+    idx++;
+  }
+
   if (wtc_2 & COUNT_VXLAN) {
     cjhandler[idx] = compose_json_vxlan;
     idx++;
@@ -543,7 +587,17 @@ void compose_json_dst_mac(json_t *obj, struct chained_cache *cc)
 
 void compose_json_vlan(json_t *obj, struct chained_cache *cc)
 {
-  json_object_set_new_nocheck(obj, "vlan", json_integer((json_int_t)cc->primitives.vlan_id));
+  if (config.tmp_vlan_legacy) {
+    json_object_set_new_nocheck(obj, "vlan", json_integer((json_int_t)cc->primitives.vlan_id));
+  }
+  else {
+    json_object_set_new_nocheck(obj, "vlan_in", json_integer((json_int_t)cc->primitives.vlan_id));
+  }
+}
+
+void compose_json_out_vlan(json_t *obj, struct chained_cache *cc)
+{
+  json_object_set_new_nocheck(obj, "vlan_out", json_integer((json_int_t)cc->primitives.out_vlan_id));
 }
 
 void compose_json_cos(json_t *obj, struct chained_cache *cc)
@@ -920,6 +974,30 @@ void compose_json_tcp_flags(json_t *obj, struct chained_cache *cc)
   json_object_set_new_nocheck(obj, "tcp_flags", json_string(misc_str));
 }
 
+void compose_json_fwd_status(json_t *obj, struct chained_cache *cc)
+{
+  char misc_str[VERYSHORTBUFLEN];
+
+  sprintf(misc_str, "%u", cc->pnat->fwd_status);
+  json_object_set_new_nocheck(obj, "fwd_status", json_string(misc_str));
+}
+
+void compose_json_mpls_label_stack(json_t *obj, struct chained_cache *cc)
+{
+  char label_stack[MAX_MPLS_LABEL_STACK];
+  char *label_stack_ptr = NULL;
+  int label_stack_len = 0;
+
+  memset(label_stack, 0, MAX_MPLS_LABEL_STACK);
+
+  label_stack_len = vlen_prims_get(cc->pvlen, COUNT_INT_MPLS_LABEL_STACK, &label_stack_ptr);
+  if (label_stack_ptr) {
+    mpls_label_stack_to_str(label_stack, sizeof(label_stack), (u_int32_t *)label_stack_ptr, label_stack_len);
+  }
+
+  json_object_set_new_nocheck(obj, "mpls_label_stack", json_string(label_stack));
+}
+
 void compose_json_proto(json_t *obj, struct chained_cache *cc)
 {
   char proto[PROTO_NUM_STRLEN];
@@ -939,7 +1017,7 @@ void compose_json_sampling_rate(json_t *obj, struct chained_cache *cc)
 
 void compose_json_sampling_direction(json_t *obj, struct chained_cache *cc)
 {
-  json_object_set_new_nocheck(obj, "sampling_direction", json_string(cc->primitives.sampling_direction));
+  json_object_set_new_nocheck(obj, "sampling_direction", json_string(sampling_direction_print(cc->primitives.sampling_direction)));
 }
 
 void compose_json_post_nat_src_host(json_t *obj, struct chained_cache *cc)
@@ -973,6 +1051,11 @@ void compose_json_nat_event(json_t *obj, struct chained_cache *cc)
   json_object_set_new_nocheck(obj, "nat_event", json_integer((json_int_t)cc->pnat->nat_event));
 }
 
+void compose_json_fw_event(json_t *obj, struct chained_cache *cc)
+{
+  json_object_set_new_nocheck(obj, "fw_event", json_integer((json_int_t)cc->pnat->fw_event));
+}
+
 void compose_json_mpls_label_top(json_t *obj, struct chained_cache *cc)
 {
   json_object_set_new_nocheck(obj, "mpls_label_top", json_integer((json_int_t)cc->pmpls->mpls_label_top));
@@ -983,11 +1066,6 @@ void compose_json_mpls_label_bottom(json_t *obj, struct chained_cache *cc)
   json_object_set_new_nocheck(obj, "mpls_label_bottom", json_integer((json_int_t)cc->pmpls->mpls_label_bottom));
 }
 
-void compose_json_mpls_stack_depth(json_t *obj, struct chained_cache *cc)
-{
-  json_object_set_new_nocheck(obj, "mpls_stack_depth", json_integer((json_int_t)cc->pmpls->mpls_stack_depth));
-}
-
 void compose_json_tunnel_src_mac(json_t *obj, struct chained_cache *cc)
 {
   char mac[18];
@@ -1042,6 +1120,27 @@ void compose_json_tunnel_dst_port(json_t *obj, struct chained_cache *cc)
   json_object_set_new_nocheck(obj, "tunnel_port_dst", json_integer((json_int_t)cc->ptun->tunnel_dst_port));
 }
 
+void compose_json_tunnel_tcp_flags(json_t *obj, struct chained_cache *cc)
+{
+  char misc_str[VERYSHORTBUFLEN];
+
+  sprintf(misc_str, "%u", cc->tunnel_tcp_flags);
+  json_object_set_new_nocheck(obj, "tunnel_tcp_flags", json_string(misc_str));
+}
+
+void compose_json_array_tunnel_tcp_flags(json_t *obj, struct chained_cache *cc)
+{
+  /* linked-list creation */
+  cdada_list_t *ll = tcpflags_to_linked_list(cc->tunnel_tcp_flags);
+  size_t ll_size = cdada_list_size(ll);
+
+  json_t *root_l1 = compose_tcpflags_json_data(ll, ll_size);
+
+  json_object_set_new_nocheck(obj, "tunnel_tcp_flags", root_l1);
+
+  cdada_list_destroy(ll);
+}
+
 void compose_json_vxlan(json_t *obj, struct chained_cache *cc)
 {
   json_object_set_new_nocheck(obj, "vxlan", json_integer((json_int_t)cc->ptun->tunnel_id));
@@ -1224,3 +1323,174 @@ void *compose_purge_close_json(char *writer_name, pid_t writer_pid, int purged_e
   return NULL;
 }
 #endif
+
+void compose_json_map_label(json_t *obj, struct chained_cache *cc)
+{
+  char empty_string[] = "", *str_ptr;
+
+  vlen_prims_get(cc->pvlen, COUNT_INT_LABEL, &str_ptr);
+  if (!str_ptr) str_ptr = empty_string;
+
+  /* labels normalization */
+  cdada_str_t *lbls_cdada = cdada_str_create(str_ptr);
+  cdada_str_replace_all(lbls_cdada, PRETAG_LABEL_KV_SEP, DEFAULT_SEP);
+  const char *lbls_norm = cdada_str(lbls_cdada);
+
+  /* linked-list creation */
+  cdada_list_t *ptm_ll = ptm_labels_to_linked_list(lbls_norm);
+  size_t ll_size = cdada_list_size(ptm_ll);
+
+  json_t *root_l1 = compose_label_json_data(ptm_ll, ll_size);
+
+  json_object_set_new_nocheck(obj, "label", root_l1);
+  
+  cdada_str_destroy(lbls_cdada);
+  cdada_list_destroy(ptm_ll);
+}
+
+void compose_json_array_tcpflags(json_t *obj, struct chained_cache *cc)
+{
+  /* linked-list creation */
+  cdada_list_t *ll = tcpflags_to_linked_list(cc->tcp_flags);
+  size_t ll_size = cdada_list_size(ll);
+
+  json_t *root_l1 = compose_tcpflags_json_data(ll, ll_size);
+
+  json_object_set_new_nocheck(obj, "tcp_flags", root_l1);
+
+  cdada_list_destroy(ll);
+}
+
+void compose_json_string_fwd_status(json_t *obj, struct chained_cache *cc)
+{
+  /* linked-list creation */
+  cdada_list_t *fwd_status_ll = fwd_status_to_linked_list();
+  size_t ll_size = cdada_list_size(fwd_status_ll);
+
+  json_t *root_l1 = compose_fwd_status_json_data(cc->pnat->fwd_status, fwd_status_ll, ll_size);
+
+  json_object_set_new_nocheck(obj, "fwd_status", root_l1);
+
+  cdada_list_destroy(fwd_status_ll);
+}
+
+void compose_json_array_mpls_label_stack(json_t *obj, struct chained_cache *cc)
+{
+  char *label_stack_ptr = NULL;
+  int label_stack_len = 0;
+
+  label_stack_len = vlen_prims_get(cc->pvlen, COUNT_INT_MPLS_LABEL_STACK, &label_stack_ptr);
+  json_t *root_l1 = compose_mpls_label_stack_json_data((u_int32_t *)label_stack_ptr, label_stack_len);
+
+  json_object_set_new_nocheck(obj, "mpls_label_stack", root_l1);
+}
+
+json_t *compose_label_json_data(cdada_list_t *ll, int ll_size)
+{
+  ptm_label lbl;
+
+  json_t *root = json_object();
+  json_t *j_str_tmp = NULL;
+
+  size_t idx_0;
+  for (idx_0 = 0; idx_0 < ll_size; idx_0++) {
+    memset(&lbl, 0, sizeof(lbl));
+    cdada_list_get(ll, idx_0, &lbl);
+    j_str_tmp = json_string(lbl.value);
+    json_object_set_new_nocheck(root, lbl.key, j_str_tmp);
+  }
+
+  return root;
+}
+
+json_t *compose_tcpflags_json_data(cdada_list_t *ll, int ll_size)
+{
+  tcpflag tcpstate;
+
+  json_t *root = json_array();
+  json_t *j_str_tmp = NULL;
+
+  size_t idx_0;
+  for (idx_0 = 0; idx_0 < ll_size; idx_0++) {
+    memset(&tcpstate, 0, sizeof(tcpstate));
+    cdada_list_get(ll, idx_0, &tcpstate);
+    if (strncmp(tcpstate.flag, "NULL", (TCP_FLAG_LEN - 1)) != 0) {
+      j_str_tmp = json_string(tcpstate.flag);
+      json_array_append(root, j_str_tmp);
+    }
+  }
+
+  return root;
+}
+
+json_t *compose_fwd_status_json_data(size_t fwdstatus_decimal, cdada_list_t *ll, int ll_size)
+{
+  fwd_status fwdstate;
+  json_t *root = NULL;
+
+  /* default fwdstatus */
+  if ((fwdstatus_decimal >= 0) && (fwdstatus_decimal <= 63)) {
+    root = json_string("UNKNOWN Unclassified");
+  }
+  else if ((fwdstatus_decimal >= 64) && (fwdstatus_decimal <= 127)) {
+    root = json_string("FORWARDED Unclassified");
+  }
+  else if ((fwdstatus_decimal >= 128) && (fwdstatus_decimal <= 191)) {
+    root = json_string("DROPPED Unclassified");
+  }
+  else if ((fwdstatus_decimal >= 192) && (fwdstatus_decimal <= 255)) {
+    root = json_string("CONSUMED Unclassified");
+  }
+  else {
+    root = json_string("RFC-7270 Misinterpreted");
+  }
+
+  size_t idx_0;
+  for (idx_0 = 0; idx_0 < ll_size; idx_0++) {
+    memset(&fwdstate, 0, sizeof(fwdstate));
+    cdada_list_get(ll, idx_0, &fwdstate);
+    if (fwdstate.decimal == fwdstatus_decimal) {
+      json_string_set(root, fwdstate.description);
+    }
+  }
+
+  return root;
+}
+
+json_t *compose_mpls_label_stack_json_data(u_int32_t *label_stack, int ls_len)
+{
+  const int MAX_IDX_LEN = 4;
+  const int MAX_MPLS_LABEL_IDX_LEN = (MAX_IDX_LEN + MAX_MPLS_LABEL_LEN);
+  int max_mpls_label_idx_len_dec = 0;
+  char label_buf[MAX_MPLS_LABEL_LEN];
+  char label_idx_buf[MAX_MPLS_LABEL_IDX_LEN];
+  char idx_buf[MAX_IDX_LEN];
+  u_int8_t ls_depth = 0;
+
+  if (!(ls_len % 4)) {
+    ls_depth = (ls_len / 4);
+  }
+  else {
+    return NULL;
+  }
+
+  json_t *root = json_array();
+  json_t *j_str_tmp = NULL;
+
+  size_t idx_0;
+  for (idx_0 = 0; idx_0 < ls_depth; idx_0++) {
+    memset(&label_buf, 0, sizeof(label_buf));
+    snprintf(label_buf, MAX_MPLS_LABEL_LEN, "%u", *(label_stack + idx_0));
+    memset(&idx_buf, 0, sizeof(idx_buf));
+    memset(&label_idx_buf, 0, sizeof(label_idx_buf));
+    snprintf(idx_buf, MAX_IDX_LEN, "%zu", idx_0);
+    strncat(label_idx_buf, idx_buf, (MAX_MPLS_LABEL_IDX_LEN - max_mpls_label_idx_len_dec));
+    strncat(label_idx_buf, "-", (MAX_MPLS_LABEL_IDX_LEN - max_mpls_label_idx_len_dec));
+    strncat(label_idx_buf, label_buf, (MAX_MPLS_LABEL_IDX_LEN - max_mpls_label_idx_len_dec));
+    max_mpls_label_idx_len_dec = (strlen(idx_buf) + strlen("-") + strlen(label_buf) + 3);
+    j_str_tmp = json_string(label_idx_buf);
+    json_array_append(root, j_str_tmp); 
+  }
+
+  return root;
+}
