@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2019 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2021 by Paolo Lucente
 */
 
 /*
@@ -74,7 +74,7 @@ void set_preprocess_funcs(char *string, struct preprocess *prep, int dictionary)
       for (dindex = 0; strcmp(print_prep_dict[dindex].key, ""); dindex++) {
         if (!strcmp(print_prep_dict[dindex].key, key)) {
           err = FALSE;
-          break;      
+          break;
         }           
         else err = E_NOTFOUND; /* key not found */
       }
@@ -241,6 +241,12 @@ void set_preprocess_funcs(char *string, struct preprocess *prep, int dictionary)
       sql_idx++;
       prep->checkno++;
     }
+    else if (dictionary == PREP_DICT_PRINT) {
+      P_preprocess_funcs[p_idx] = P_check_maxp;
+      prep->num++;
+      p_idx++;
+      prep->checkno++;
+    }
   }
 
   if (prep->maxf) {
@@ -250,6 +256,12 @@ void set_preprocess_funcs(char *string, struct preprocess *prep, int dictionary)
       sql_idx++;
       prep->checkno++;
     }
+    else if (dictionary == PREP_DICT_PRINT) {
+      P_preprocess_funcs[p_idx] = P_check_maxf;
+      prep->num++;
+      p_idx++;
+      prep->checkno++;
+    }
   }
 
   if (prep->maxb) {
@@ -259,6 +271,12 @@ void set_preprocess_funcs(char *string, struct preprocess *prep, int dictionary)
       sql_idx++;
       prep->checkno++;
     }
+    else if (dictionary == PREP_DICT_PRINT) {
+      P_preprocess_funcs[p_idx] = P_check_maxb;
+      prep->num++;
+      p_idx++;
+      prep->checkno++;
+    }
   }
 
   if (prep->maxbpp) {
@@ -786,6 +804,51 @@ int P_check_minf(struct chained_cache *queue[], int *num, int seq)
   return FALSE;
 }
 
+int P_check_maxp(struct chained_cache *queue[], int *num, int seq)
+{
+  int x;
+
+  for (x = 0; x < *num; x++) {
+    if (queue[x]->valid == PRINT_CACHE_INVALID || queue[x]->valid == PRINT_CACHE_COMMITTED) {
+      if (queue[x]->packet_counter < prep.maxp) queue[x]->prep_valid++;
+
+      P_check_validity(queue[x], seq);
+    }
+  }
+
+  return FALSE;
+}
+
+int P_check_maxb(struct chained_cache *queue[], int *num, int seq)
+{
+  int x;
+
+  for (x = 0; x < *num; x++) {
+    if (queue[x]->valid == PRINT_CACHE_INVALID || queue[x]->valid == PRINT_CACHE_COMMITTED) {
+      if (queue[x]->bytes_counter < prep.maxb) queue[x]->prep_valid++;
+
+      P_check_validity(queue[x], seq);
+    }
+  }
+
+  return FALSE;
+}
+
+int P_check_maxf(struct chained_cache *queue[], int *num, int seq)
+{
+  int x;
+
+  for (x = 0; x < *num; x++) {
+    if (queue[x]->valid == PRINT_CACHE_INVALID || queue[x]->valid == PRINT_CACHE_COMMITTED) {
+      if (queue[x]->flow_counter < prep.maxf) queue[x]->prep_valid++;
+
+      P_check_validity(queue[x], seq);
+    }
+  }
+
+  return FALSE;
+}
+
 int P_check_minbpp(struct chained_cache *queue[], int *num, int seq)
 {
   int x;
