@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2019 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2021 by Paolo Lucente
 */
 
 /*
@@ -88,6 +88,9 @@ extern int action_adjb(struct db_cache *[], int *, int);
 extern int P_check_minp(struct chained_cache *[], int *, int);
 extern int P_check_minb(struct chained_cache *[], int *, int);
 extern int P_check_minf(struct chained_cache *[], int *, int);
+extern int P_check_maxp(struct chained_cache *[], int *, int);
+extern int P_check_maxb(struct chained_cache *[], int *, int);
+extern int P_check_maxf(struct chained_cache *[], int *, int);
 extern int P_check_minbpp(struct chained_cache *[], int *, int);
 extern int P_check_minppf(struct chained_cache *[], int *, int);
 
