@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2021 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2022 by Paolo Lucente
 */
 
 /*
@@ -21,7 +21,6 @@
 
 /* includes */
 #include "pmacct.h"
-#include "addr.h"
 #include "nfacctd.h"
 #include "pretag_handlers.h"
 #include "pretag-data.h"
@@ -58,20 +57,23 @@ int (*find_id_func)(struct id_table *, struct packet_ptrs *, pm_id_t *, pm_id_t
 void load_id_file(int acct_type, char *filename, struct id_table *t, struct plugin_requests *req, int *map_allocated)
 {
   struct id_table tmp;
-  struct id_entry *ptr, *ptr2;
+  struct id_entry *ptr;
   FILE *file;
   char *buf = NULL;
-  int v4_num = 0, x, tot_lines = 0, err, index, label_solved, sz;
-  int ignoring, map_entries, map_row_len;
+  int v4_num = 0, x, tot_lines = 0, err, errs = 0, index, label_solved;
+  int ignoring, report = TRUE, map_entries, map_row_len;
   struct stat st;
   int v6_num = 0;
+  u_int64_t sz;
 
   if (!filename || !map_allocated) return;
 
   if (acct_type == ACCT_NF || acct_type == ACCT_SF || acct_type == ACCT_PM ||
       acct_type == MAP_BGP_PEER_AS_SRC || acct_type == MAP_BGP_TO_XFLOW_AGENT ||
       acct_type == MAP_BGP_SRC_LOCAL_PREF || acct_type == MAP_BGP_SRC_MED ||
-      acct_type == MAP_FLOW_TO_RD || acct_type == MAP_SAMPLING) {
+      acct_type == MAP_FLOW_TO_RD || acct_type == MAP_SAMPLING ||
+      acct_type == ACCT_PMBGP || acct_type == ACCT_PMBMP ||
+      acct_type == ACCT_PMTELE) {
     req->key_value_table = (void *) &tmp;
   }
 
@@ -108,7 +110,7 @@ void load_id_file(int acct_type, char *filename, struct id_table *t, struct plug
       goto handle_error;
     }
 
-    sz = sizeof(struct id_entry)*map_entries;
+    sz = sizeof(struct id_entry) * map_entries;
 
     if (t) {
       if (*map_allocated == 0) {
@@ -127,12 +129,17 @@ void load_id_file(int acct_type, char *filename, struct id_table *t, struct plug
         if (config.maps_index && pretag_index_have_one(t)) {
 	  pretag_index_destroy(t);
 	}
+
 	for (index = 0; index < t->num; index++) {
 	  pcap_freecode(&t->e[index].key.filter);
 	  pretag_free_label(&t->e[index].label);
 	  if (t->e[index].jeq.label) free(t->e[index].jeq.label); 
+	  if (t->e[index].entry_label) free(t->e[index].entry_label);
 	}
 
+	cdada_map_destroy(t->label_map_v4);
+	cdada_map_destroy(t->label_map_v6);
+
         memset(t, 0, sizeof(struct id_table));
         t->e = ptr ;
       }
@@ -361,7 +368,24 @@ void load_id_file(int acct_type, char *filename, struct id_table *t, struct plug
                   }
                   key = NULL; value = NULL;
                 }
+		else if (acct_type == ACCT_PMBGP || acct_type == ACCT_PMBMP || acct_type == ACCT_PMTELE) {
+                  for (dindex = 0; strcmp(tag_map_nonflow_dictionary[dindex].key, ""); dindex++) {
+                    if (!strcmp(tag_map_nonflow_dictionary[dindex].key, key)) {
+                      err = (*tag_map_nonflow_dictionary[dindex].func)(filename, &tmp.e[tmp.num], value, req, acct_type);
+                      break;
+                    }
+                    else err = E_NOTFOUND; /* key not found */
+		  }
+                  if (err) {
+                    if (err == E_NOTFOUND) Log(LOG_WARNING, "WARN ( %s/%s ): [%s:%u] unknown key '%s'. Ignored.\n", 
+						config.name, config.type, filename, tot_lines, key);
+                    else Log(LOG_WARNING, "WARN ( %s/%s ): [%s:%u] Line ignored.\n", config.name, config.type, filename, tot_lines);
+                    break;
+                  }
+                  key = NULL; value = NULL;
+		}
 		else {
+		  /* ACCT_NF, ACCT_SF, ACCT_PM, etc. */
 		  if (tee_plugins) {
                     for (dindex = 0; strcmp(tag_map_tee_dictionary[dindex].key, ""); dindex++) {
                       if (!strcmp(tag_map_tee_dictionary[dindex].key, key)) {
@@ -392,7 +416,8 @@ void load_id_file(int acct_type, char *filename, struct id_table *t, struct plug
             }
 	    if (!ignoring) {
               /* verifying errors and required fields */
-	      if (acct_type == ACCT_NF || acct_type == ACCT_SF) {
+	      if (acct_type == ACCT_NF || acct_type == ACCT_SF || acct_type == ACCT_PMBGP ||
+		  acct_type == ACCT_PMBMP || acct_type == ACCT_PMTELE) {
 	        if (tmp.e[tmp.num].id && tmp.e[tmp.num].id2 && tmp.e[tmp.num].label.len) 
 		   Log(LOG_WARNING, "WARN ( %s/%s ): [%s:%u] set_tag (id), set_tag2 (id2) and set_label are mutual exclusive. Line ignored.\n", 
 			config.name, config.type, filename, tot_lines);
@@ -411,7 +436,15 @@ void load_id_file(int acct_type, char *filename, struct id_table *t, struct plug
 		      tmp.e[tmp.num].key.agent_mask.family = AF_INET;
 
 		      memcpy(&recirc_e, &tmp.e[tmp.num], sizeof(struct id_entry));
-		      recirculate = TRUE;
+
+		      /*
+			If indexing is enabled and no address family is specified for 'ip'
+			(ie. 'ip' is likely not specified), we will not include it as part
+			of the index hash serializer anyway; we can skip recirculation.
+		      */
+		      if (!config.maps_index) {
+			recirculate = TRUE;
+		      }
 		    }
 		    else {
 		      memcpy(&tmp.e[tmp.num], &recirc_e, sizeof(struct id_entry));
@@ -552,10 +585,36 @@ void load_id_file(int acct_type, char *filename, struct id_table *t, struct plug
       t->ipv4_num = v4_num; 
       t->ipv4_base = &t->e[x];
       t->flags = tmp.flags;
+      {
+	pm_id_t pm_cdada_map_container;
+
+        t->label_map_v4 = cdada_map_create(pm_cdada_map_container);
+        t->label_map_v6 = cdada_map_create(pm_cdada_map_container);
+	if (!t->label_map_v4 || !t->label_map_v6) {
+	  Log(LOG_ERR, "ERROR ( %s/%s ): [%s] unable to allocate Label Map. Exiting.\n", config.name, config.type, t->filename);
+	  exit_gracefully(1);
+	}
+      }
+
       for (index = 0; index < tmp.num; index++) {
-        if (tmp.e[index].key.agent_ip.a.family == AF_INET) { 
+        if (tmp.e[index].key.agent_ip.a.family == AF_INET) {
           memcpy(&t->e[x], &tmp.e[index], sizeof(struct id_entry));
 	  t->e[x].pos = x;
+
+	  if (t->e[x].entry_label) {
+	    int ret;
+	    pm_id_t *idx_value;
+
+	    ret = cdada_map_find(t->label_map_v4, t->e[x].entry_label, (void **) &idx_value);
+	    if (ret == CDADA_E_NOT_FOUND) {
+	      ret = cdada_map_insert(t->label_map_v4, t->e[x].entry_label, &t->e[x].pos);
+	      if (ret != CDADA_SUCCESS) {
+		Log(LOG_ERR, "ERROR ( %s/%s ): [%s] unable to insert in Label Map v4. Exiting.\n", config.name, config.type, t->filename);
+		exit_gracefully(1);
+	      }
+	    }
+	  }
+
 	  x++;
         }
       }
@@ -566,6 +625,21 @@ void load_id_file(int acct_type, char *filename, struct id_table *t, struct plug
         if (tmp.e[index].key.agent_ip.a.family == AF_INET6) {
           memcpy(&t->e[x], &tmp.e[index], sizeof(struct id_entry));
 	  t->e[x].pos = x;
+
+	  if (t->e[x].entry_label) {
+	    int ret;
+	    pm_id_t *idx_value;
+
+	    ret = cdada_map_find(t->label_map_v6, t->e[x].entry_label, (void **) &idx_value);
+	    if (ret == CDADA_E_NOT_FOUND) {
+	      ret = cdada_map_insert(t->label_map_v6, t->e[x].entry_label, &t->e[x].pos);
+	      if (ret != CDADA_SUCCESS) {
+		Log(LOG_ERR, "ERROR ( %s/%s ): [%s] unable to insert in Label Map v6. Exiting.\n", config.name, config.type, t->filename);
+		exit_gracefully(1);
+	      }
+	    }
+	  }
+
           x++;
         }
       }
@@ -583,11 +657,14 @@ void load_id_file(int acct_type, char *filename, struct id_table *t, struct plug
 	    label_solved = TRUE;
 	  }
 	  else {
-	    for (ptr2 = ptr+1, index = x+1; index < t->ipv4_num; ptr2++, index++) {
-	      if (!strcmp(ptr->jeq.label, ptr2->entry_label)) {
-	        ptr->jeq.ptr = ptr2;
-	        label_solved = TRUE;
-		break;
+            int ret;
+            pm_id_t *idx_value;
+
+            ret = cdada_map_find(t->label_map_v4, ptr->jeq.label, (void **) &idx_value);
+	    if (ret == CDADA_SUCCESS) {
+	      if ((*idx_value) > ptr->pos) {
+		ptr->jeq.ptr = &t->e[(*idx_value)];
+		label_solved = TRUE;
 	      }
 	    }
 	  }
@@ -601,19 +678,24 @@ void load_id_file(int acct_type, char *filename, struct id_table *t, struct plug
 
       for (ptr = t->ipv6_base, x = 0; x < t->ipv6_num; ptr++, x++) {
         if (ptr->jeq.label) {
+          int ret;
+          pm_id_t *idx_value;
+
           label_solved = FALSE;
-          for (ptr2 = ptr+1, index = x+1; index < t->ipv6_num; ptr2++, index++) {
-            if (!strcmp(ptr->jeq.label, ptr2->entry_label)) {
-              ptr->jeq.ptr = ptr2;
-              label_solved = TRUE;
-	      break;
-            }
-          }
-          if (!label_solved) {
-            ptr->jeq.ptr = NULL;
-            Log(LOG_WARNING, "WARN ( %s/%s ): [%s] Unresolved label '%s'. Ignoring it.\n",
+
+          ret = cdada_map_find(t->label_map_v6, ptr->jeq.label, (void **) &idx_value);
+	  if (ret == CDADA_SUCCESS) {
+	    if ((*idx_value) > ptr->pos) {
+	      ptr->jeq.ptr = &t->e[(*idx_value)];
+	      label_solved = TRUE;
+	    }
+	  }
+
+	  if (!label_solved) {
+	    ptr->jeq.ptr = NULL;
+	    Log(LOG_WARNING, "WARN ( %s/%s ): [%s] Unresolved label '%s'. Ignoring it.\n",
 			config.name, config.type, filename, ptr->jeq.label);
-          }
+	  }
         }
       }
 
@@ -622,7 +704,8 @@ void load_id_file(int acct_type, char *filename, struct id_table *t, struct plug
       /* pre_tag_map indexing here */
       if (config.maps_index &&
 	  (acct_type == ACCT_NF || acct_type == ACCT_SF || acct_type == ACCT_PM ||
-	   acct_type == MAP_BGP_PEER_AS_SRC || acct_type == MAP_FLOW_TO_RD)) {
+	   acct_type == MAP_BGP_PEER_AS_SRC || acct_type == MAP_FLOW_TO_RD ||
+	   acct_type == ACCT_PMBGP || acct_type == ACCT_PMBMP || acct_type == ACCT_PMTELE)) {
 	pt_bitmap_t idx_bmap;
 	
 	t->index_num = MAX_ID_TABLE_INDEXES;
@@ -649,16 +732,48 @@ void load_id_file(int acct_type, char *filename, struct id_table *t, struct plug
           idx_bmap = pretag_index_build_bitmap(ptr, acct_type);
 
 	  /* fill indexes */
-	  pretag_index_fill(t, idx_bmap, ptr);
+	  err = pretag_index_fill(t, idx_bmap, ptr, x);
+	  if (err == PRETAG_IDX_ERR_FATAL) {
+	    Log(LOG_WARNING, "WARN ( %s/%s ): [%s] Fatal error. Indexing disabled.\n", config.name, config.type, filename);
+	    pretag_index_destroy(t);
+
+	    errs++;
+	    report = FALSE;
+
+	    break;
+	  }
+	  else if (err) {
+	    errs++;
+	  }
 	}
       }
 
+      if (errs && t->flags & PRETAG_FLAG_JEQ) {
+	/* in case of fatal error, index was already destroyed */
+	if (err != PRETAG_IDX_ERR_FATAL) {
+          Log(LOG_WARNING, "WARN ( %s/%s ): [%s] 'jeq' not supported if error count is non-zero when loading. Indexing disabled.\n",
+	      config.name, config.type, filename);
+          pretag_index_destroy(t);
+	}
+
+        report = FALSE;
+      }
+
       if (t->flags & PRETAG_FLAG_NEG) {
         Log(LOG_WARNING, "WARN ( %s/%s ): [%s] Negations not supported. Indexing disabled.\n",
-                config.name, config.type, filename);
+	    config.name, config.type, filename);
         pretag_index_destroy(t);
+        report = FALSE;
+      }
+
+      if (pretag_index_validate_dedup_netmask_lists(t) == ERR) {
+        pretag_index_destroy(t);
+        report = FALSE;
+      }
+
+      if (report) {
+	pretag_index_report(t);
       }
-      else pretag_index_report(t);
     }
   }
 
@@ -688,13 +803,22 @@ u_int8_t pt_check_neg(char **value, u_int32_t *flags)
   if (**value == '-') {
     (*value)++;
 
-    if (flags) *flags |= PRETAG_FLAG_NEG;
+    if (flags) {
+      *flags |= PRETAG_FLAG_NEG;
+    }
 
     return TRUE;
   }
   else return FALSE;
 }
 
+void pt_set_jeq(u_int32_t *flags)
+{
+  if (flags) {
+    *flags |= PRETAG_FLAG_JEQ;
+  }
+}
+
 char *pt_check_range(char *str)
 {
   char *ptr;
@@ -757,16 +881,24 @@ int pretag_malloc_label(pt_label_t *label, int len)
   return SUCCESS;
 }
 
-int pretag_realloc_label(pt_label_t *label, int len)
+int pretag_realloc_label(pt_label_t *label, int old_len, int add_len)
 {
+  char *local_val = NULL;
+
   if (!label) return ERR;
 
-  label->val = realloc(label->val, len);
-  if (!label->val) {
+  local_val = malloc((old_len + add_len));
+  if (!local_val) {
     Log(LOG_ERR, "ERROR ( %s/%s ): malloc() failed (pretag_realloc_label).\n", config.name, config.type);
     return ERR;
   }
-  label->len = len;
+
+  memset(local_val, 0, (old_len + add_len));
+  strcpy(local_val, label->val);
+
+  free(label->val);
+  label->val = local_val;
+  label->len = (old_len + add_len);
 
   return SUCCESS;
 }
@@ -818,8 +950,6 @@ int pretag_move_label(pt_label_t *dst, pt_label_t *src)
 
 int pretag_append_label(pt_label_t *dst, pt_label_t *src)
 {
-  char default_sep[] = ",";
-
   if (!src || !dst) return ERR;
 
   if (!dst->val) {
@@ -828,15 +958,15 @@ int pretag_append_label(pt_label_t *dst, pt_label_t *src)
   }
   else {
     if (src->len) {
-      pretag_realloc_label(dst, (dst->len + src->len + 1 /* sep */ + 1 /* null */));
+      pretag_realloc_label(dst, dst->len, (src->len + 1 /* sep */ + 1 /* null */));
       if (!dst->val) {
         Log(LOG_ERR, "ERROR ( %s/%s ): malloc() failed (pretag_append_label).\n", config.name, config.type);
         return ERR;
       }
 
-      strncat(dst->val, default_sep, 1);
+      strcat(dst->val, DEFAULT_SEP);
       strncat(dst->val, src->val, src->len);
-      dst->val[dst->len] = '\0';
+      dst->val[dst->len - 1] = '\0';
     }
   }
 
@@ -941,6 +1071,9 @@ pt_bitmap_t pretag_index_build_bitmap(struct id_entry *ptr, int acct_type)
   if (idx_bmap & PRETAG_SET_TAG2) idx_bmap ^= PRETAG_SET_TAG2;
   if (idx_bmap & PRETAG_SET_LABEL) idx_bmap ^= PRETAG_SET_LABEL;
 
+  /* 3) handle the case of catch-all rule */
+  if (!idx_bmap) idx_bmap = PRETAG_NULL;
+
   return idx_bmap;
 }
 
@@ -967,7 +1100,7 @@ int pretag_index_set_handlers(struct id_table *t)
   u_int32_t index = 0, iterator = 0, handler_index = 0;
 
   if (!t) return TRUE;
-  
+
   for (iterator = 0; iterator < t->index_num; iterator++) {
     residual_idx_bmap = t->index[iterator].bitmap;
 
@@ -1038,9 +1171,10 @@ int pretag_index_allocate(struct id_table *t)
 	}
       }
 
-      {
+      if (idx_entry_size) {
 	char pm_cdada_map_container[idx_entry_size];
 
+	hash_init_serial(&t->index[iterator].hash_serializer, idx_entry_size);
         t->index[iterator].idx_map = cdada_map_create(pm_cdada_map_container);
         if (!t->index[iterator].idx_map) {
 	  Log(LOG_WARNING, "WARN ( %s/%s ): [%s] maps_index: unable to allocate index %llx. Destroying.\n", config.name,
@@ -1049,6 +1183,12 @@ int pretag_index_allocate(struct id_table *t)
 	  break;
 	}
       }
+      else {
+	Log(LOG_WARNING, "WARN ( %s/%s ): [%s] maps_index: null key for index %llx. Destroying.\n", config.name,
+	    config.type, t->filename, (unsigned long long)t->index[iterator].bitmap);
+	destroy = TRUE;
+	break;
+      }
     }
   }
 
@@ -1060,44 +1200,75 @@ int pretag_index_allocate(struct id_table *t)
   return SUCCESS;
 }
 
-int pretag_index_fill(struct id_table *t, pt_bitmap_t idx_bmap, struct id_entry *ptr)
+int pretag_index_fill(struct id_table *t, pt_bitmap_t idx_bmap, struct id_entry *ptr, int lineno)
 {
   u_int32_t iterator = 0, handler_index = 0;
-  int ret;
+  int ret = SUCCESS;
 
-  if (!t) return ERR;
+  if (!t) return PRETAG_IDX_ERR_FATAL;
 
   for (iterator = 0; iterator < t->index_num; iterator++) {
     if (t->index[iterator].entries && t->index[iterator].bitmap == idx_bmap) {
-      struct id_entry e;
       pm_hash_serial_t *hash_serializer;
       pm_hash_key_t *hash_key;
 
       /* fill serializer in */
-      memset(&e, 0, sizeof(struct id_entry));
       hash_serializer = &t->index[iterator].hash_serializer;
       hash_serial_set_off(hash_serializer, 0);
       hash_key = hash_serial_get_key(hash_serializer);
 
-      if (!hash_key) return ERR;
+      if (!hash_key) return PRETAG_IDX_ERR_FATAL;
 
       for (handler_index = 0; t->index[iterator].idt_handler[handler_index]; handler_index++) {
-	(*t->index[iterator].idt_handler[handler_index])(&e, hash_serializer, ptr);
+	ret = (*t->index[iterator].idt_handler[handler_index])(&t->index[iterator], handler_index, hash_serializer, ptr);
+        if (ret) {
+	  Log(LOG_WARNING, "WARN ( %s/%s ): [%s] pretag_index_fill(): index=%llx line=%d (err!)\n",
+	      config.name, config.type, t->filename, (unsigned long long)idx_bmap, (lineno + 1));
+	  break;
+        }
       }
 
-      ret = cdada_map_insert(t->index[iterator].idx_map, hash_key_get_val(hash_key), ptr);
-      if (ret == CDADA_E_EXISTS) {
-	u_char key_hexdump[hash_key_get_len(hash_key) * 3];
-	serialize_hex(hash_key_get_val(hash_key), key_hexdump, hash_key_get_len(hash_key));
+      if (!ret) {
+        ret = cdada_map_insert(t->index[iterator].idx_map, hash_key_get_val(hash_key), ptr);
+        if (ret == CDADA_E_EXISTS) {
+	  u_char key_hexdump[hash_key_get_len(hash_key) * 3];
+	  serialize_hex(hash_key_get_val(hash_key), key_hexdump, hash_key_get_len(hash_key));
 
-	Log(LOG_DEBUG, "DEBUG ( %s/%s ): [%s] pretag_index_fill(): index=%llx key=%s (dup!)\n",
-	    config.name, config.type, t->filename, (unsigned long long)idx_bmap, key_hexdump);
+	  Log(LOG_DEBUG, "DEBUG ( %s/%s ): [%s] pretag_index_fill(): index=%llx line=%d key=%s (dup!)\n",
+	      config.name, config.type, t->filename, (unsigned long long)idx_bmap, (lineno + 1), key_hexdump);
+
+	  ret = PRETAG_IDX_ERR_NONE; /* alias SUCCESS */
+	}
       }
 
       break;
     }
   }
 
+  return ret;
+}
+
+int pretag_index_validate_dedup_netmask_lists(struct id_table *t)
+{
+  u_int32_t iterator = 0;
+
+  if (!t) return ERR;
+
+  /* dedup */
+  for (iterator = 0; iterator < t->index_num; iterator++) {
+    if (t->index[iterator].netmask.v4.list) {
+      cdada_list_sort(t->index[iterator].netmask.v4.list);
+      cdada_list_unique(t->index[iterator].netmask.v4.list);
+      t->index[iterator].netmask.v4.size = cdada_list_size(t->index[iterator].netmask.v4.list);
+    }
+
+    if (t->index[iterator].netmask.v6.list) {
+      cdada_list_sort(t->index[iterator].netmask.v6.list);
+      cdada_list_unique(t->index[iterator].netmask.v6.list);
+      t->index[iterator].netmask.v6.size = cdada_list_size(t->index[iterator].netmask.v6.list);
+    }
+  }
+
   return SUCCESS;
 }
 
@@ -1133,6 +1304,18 @@ void pretag_index_report(struct id_table *t)
 	  config.name, config.type, t->filename, (unsigned long long)t->index[iterator].bitmap,
 	  cdada_map_size(t->index[iterator].idx_map));
 
+      if (t->index[iterator].netmask.v4.list) {
+        Log(LOG_INFO, "INFO ( %s/%s ): [%s] pretag_index_report(): index=%llx handler=%u netmask.v4.list_entries=%u\n",
+	    config.name, config.type, t->filename, (unsigned long long)t->index[iterator].bitmap,
+	    t->index[iterator].netmask.hdlr_no, cdada_list_size(t->index[iterator].netmask.v4.list));
+      }
+
+      if (t->index[iterator].netmask.v6.list) {
+        Log(LOG_INFO, "INFO ( %s/%s ): [%s] pretag_index_report(): index=%llx handler=%u netmask.v6.list_entries=%u\n",
+	    config.name, config.type, t->filename, (unsigned long long)t->index[iterator].bitmap,
+	    t->index[iterator].netmask.hdlr_no, cdada_list_size(t->index[iterator].netmask.v6.list));
+      }
+
       if (config.debug) {
 	cdada_map_traverse(t->index[iterator].idx_map, &pretag_index_print_key, &t->index[iterator]);
       }
@@ -1171,6 +1354,15 @@ void pretag_index_destroy(struct id_table *t)
     /* destroy the serializer */
     hash_destroy_key(hash_key);
 
+    /* destroy the lists */
+    if (t->index[iterator].netmask.v4.list) {
+      cdada_list_destroy(t->index[iterator].netmask.v4.list);
+    }
+
+    if (t->index[iterator].netmask.v6.list) {
+      cdada_list_destroy(t->index[iterator].netmask.v6.list);
+    }
+
     /* cleanup */
     memset(&t->index[iterator], 0, sizeof(struct id_table_index));
   }
@@ -1184,34 +1376,58 @@ u_int32_t pretag_index_lookup(struct id_table *t, struct packet_ptrs *pptrs, str
   pm_hash_serial_t *hash_serializer;
   pm_hash_key_t *hash_key;
   u_int32_t iterator, index_hdlr;
-  int iterator_ir;
-  void *idx_value;
+  int iterator_ir, netmask_idx, netmask_max, ret;
+  void *idx_value = NULL;
 
   if (!t || !pptrs || !index_results) return 0;
 
   memset(&res_fdata, 0, sizeof(res_fdata));
   memset(index_results, 0, (sizeof(struct id_entry *) * ir_entries));
   iterator_ir = 0;
+  netmask_idx = netmask_max = ERR;
 
   for (iterator = 0; iterator < t->index_num; iterator++) {
     if (t->index[iterator].entries) {
+      if (pptrs->l3_proto == ETHERTYPE_IP && t->index[iterator].netmask.v4.size) {
+	netmask_idx = 0;
+	netmask_max = t->index[iterator].netmask.v4.size;
+      }
+      else if (pptrs->l3_proto == ETHERTYPE_IPV6 && t->index[iterator].netmask.v6.size) {
+	netmask_idx = 0;
+	netmask_max = t->index[iterator].netmask.v6.size;
+      }
+
+      netmask_recirc:
+
       hash_serializer = &t->index[iterator].hash_serializer;
       hash_serial_set_off(hash_serializer, 0);
       hash_key = hash_serial_get_key(hash_serializer);
 
       for (index_hdlr = 0; (*t->index[iterator].fdata_handler[index_hdlr]); index_hdlr++) {
-        (*t->index[iterator].fdata_handler[index_hdlr])(&res_fdata, &t->index[iterator].hash_serializer, pptrs);
+        (*t->index[iterator].fdata_handler[index_hdlr])(&t->index[iterator], index_hdlr, netmask_idx,
+							&res_fdata, &t->index[iterator].hash_serializer, pptrs);
       }
 
-      cdada_map_find(t->index[iterator].idx_map, hash_key_get_val(hash_key), &idx_value);
-      index_results[iterator_ir] = idx_value;
-      if (iterator_ir < ir_entries) iterator_ir++;
-      else {
-	Log(LOG_WARNING, "WARN ( %s/%s ): [%s] maps_index: out of index results space. Indexing disabled.\n",
-	    config.name, config.type, t->filename);
-	pretag_index_destroy(t);
-	memset(index_results, 0, (sizeof(struct id_entry *) * ir_entries));
-	return 0;
+      if (netmask_idx >= 0) {
+	netmask_idx++;
+      }
+
+      idx_value = NULL;
+      ret = cdada_map_find(t->index[iterator].idx_map, hash_key_get_val(hash_key), &idx_value);
+      if (ret == CDADA_SUCCESS) {
+	index_results[iterator_ir] = idx_value;
+	if (iterator_ir < ir_entries) iterator_ir++;
+	else {
+	  Log(LOG_WARNING, "WARN ( %s/%s ): [%s] maps_index: out of index results space. Indexing disabled.\n",
+	      config.name, config.type, t->filename);
+	  pretag_index_destroy(t);
+	  memset(index_results, 0, (sizeof(struct id_entry *) * ir_entries));
+	  return 0;
+	}
+      }
+
+      if (netmask_idx < netmask_max) {
+        goto netmask_recirc;
       }
     }
     else break;
