@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2021 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2022 by Paolo Lucente
 */
 
 /*
@@ -36,9 +36,11 @@ extern int PT_map_engine_id_handler(char *, struct id_entry *, char *, struct pl
 extern int PT_map_filter_handler(char *, struct id_entry *, char *, struct plugin_requests *, int);
 extern int PT_map_agent_id_handler(char *, struct id_entry *, char *, struct plugin_requests *, int);
 extern int PT_map_flowset_id_handler(char *, struct id_entry *, char *, struct plugin_requests *, int);
-extern int PT_map_fwdstatus_handler(char *, struct id_entry *, char *, struct plugin_requests *, int);
+extern int PT_map_fwd_status_handler(char *, struct id_entry *, char *, struct plugin_requests *, int);
 extern int PT_map_sample_type_handler(char *, struct id_entry *, char *, struct plugin_requests *, int);
 extern int PT_map_is_bi_flow_handler(char *, struct id_entry *, char *, struct plugin_requests *, int);
+extern int PT_map_is_nsel_handler(char *, struct id_entry *, char *, struct plugin_requests *, int);
+extern int PT_map_is_nel_handler(char *, struct id_entry *, char *, struct plugin_requests *, int);
 extern int PT_map_direction_handler(char *, struct id_entry *, char *, struct plugin_requests *, int);
 extern int PT_map_nat_event_handler(char *, struct id_entry *, char *, struct plugin_requests *, int);
 extern int PT_map_src_as_handler(char *, struct id_entry *, char *, struct plugin_requests *, int);
@@ -68,46 +70,48 @@ extern int PT_map_stack_handler(char *, struct id_entry *, char *, struct plugin
 extern pm_id_t PT_stack_sum(pm_id_t, pm_id_t);
 extern pm_id_t PT_stack_logical_or(pm_id_t, pm_id_t);
 
-extern int PT_map_index_entries_ip_handler(struct id_entry *, pm_hash_serial_t *, void *); 
-extern int PT_map_index_entries_input_handler(struct id_entry *, pm_hash_serial_t *, void *); 
-extern int PT_map_index_entries_output_handler(struct id_entry *, pm_hash_serial_t *, void *); 
-extern int PT_map_index_entries_bgp_nexthop_handler(struct id_entry *, pm_hash_serial_t *, void *); 
-extern int PT_map_index_entries_src_as_handler(struct id_entry *, pm_hash_serial_t *, void *); 
-extern int PT_map_index_entries_dst_as_handler(struct id_entry *, pm_hash_serial_t *, void *); 
-extern int PT_map_index_entries_peer_src_as_handler(struct id_entry *, pm_hash_serial_t *, void *); 
-extern int PT_map_index_entries_peer_dst_as_handler(struct id_entry *, pm_hash_serial_t *, void *); 
-extern int PT_map_index_entries_mpls_vpn_id_handler(struct id_entry *, pm_hash_serial_t *, void *); 
-extern int PT_map_index_entries_mpls_vpn_rd_handler(struct id_entry *, pm_hash_serial_t *, void *); 
-extern int PT_map_index_entries_mpls_pw_id_handler(struct id_entry *, pm_hash_serial_t *, void *); 
-extern int PT_map_index_entries_mpls_label_bottom_handler(struct id_entry *, pm_hash_serial_t *, void *); 
-extern int PT_map_index_entries_src_mac_handler(struct id_entry *, pm_hash_serial_t *, void *); 
-extern int PT_map_index_entries_dst_mac_handler(struct id_entry *, pm_hash_serial_t *, void *); 
-extern int PT_map_index_entries_vlan_id_handler(struct id_entry *, pm_hash_serial_t *, void *); 
-extern int PT_map_index_entries_cvlan_id_handler(struct id_entry *, pm_hash_serial_t *, void *); 
-extern int PT_map_index_entries_src_net_handler(struct id_entry *, pm_hash_serial_t *, void *);
-extern int PT_map_index_entries_dst_net_handler(struct id_entry *, pm_hash_serial_t *, void *);
-extern int PT_map_index_entries_is_multicast_handler(struct id_entry *, pm_hash_serial_t *, void *);
-extern int PT_map_index_entries_fwdstatus_handler(struct id_entry *, pm_hash_serial_t *, void *); 
-extern int PT_map_index_fdata_ip_handler(struct id_entry *, pm_hash_serial_t *, void *); 
-extern int PT_map_index_fdata_input_handler(struct id_entry *, pm_hash_serial_t *, void *); 
-extern int PT_map_index_fdata_output_handler(struct id_entry *, pm_hash_serial_t *, void *); 
-extern int PT_map_index_fdata_bgp_nexthop_handler(struct id_entry *, pm_hash_serial_t *, void *); 
-extern int PT_map_index_fdata_src_as_handler(struct id_entry *, pm_hash_serial_t *, void *); 
-extern int PT_map_index_fdata_dst_as_handler(struct id_entry *, pm_hash_serial_t *, void *); 
-extern int PT_map_index_fdata_peer_src_as_handler(struct id_entry *, pm_hash_serial_t *, void *); 
-extern int PT_map_index_fdata_peer_dst_as_handler(struct id_entry *, pm_hash_serial_t *, void *); 
-extern int PT_map_index_fdata_mpls_vpn_id_handler(struct id_entry *, pm_hash_serial_t *, void *); 
-extern int PT_map_index_fdata_mpls_vpn_rd_handler(struct id_entry *, pm_hash_serial_t *, void *); 
-extern int PT_map_index_fdata_mpls_pw_id_handler(struct id_entry *, pm_hash_serial_t *, void *); 
-extern int PT_map_index_fdata_mpls_label_bottom_handler(struct id_entry *, pm_hash_serial_t *, void *); 
-extern int PT_map_index_fdata_src_mac_handler(struct id_entry *, pm_hash_serial_t *, void *); 
-extern int PT_map_index_fdata_dst_mac_handler(struct id_entry *, pm_hash_serial_t *, void *); 
-extern int PT_map_index_fdata_vlan_id_handler(struct id_entry *, pm_hash_serial_t *, void *);
-extern int PT_map_index_fdata_cvlan_id_handler(struct id_entry *, pm_hash_serial_t *, void *);
-extern int PT_map_index_fdata_src_net_handler(struct id_entry *, pm_hash_serial_t *, void *);
-extern int PT_map_index_fdata_dst_net_handler(struct id_entry *, pm_hash_serial_t *, void *);
-extern int PT_map_index_fdata_is_multicast_handler(struct id_entry *, pm_hash_serial_t *, void *);
-extern int PT_map_index_fdata_fwdstatus_handler(struct id_entry *, pm_hash_serial_t *, void *);
+extern int PT_map_index_entries_ip_handler(struct id_table_index *, int, pm_hash_serial_t *, void *);
+extern int PT_map_index_entries_input_handler(struct id_table_index *, int, pm_hash_serial_t *, void *);
+extern int PT_map_index_entries_output_handler(struct id_table_index *, int, pm_hash_serial_t *, void *);
+extern int PT_map_index_entries_bgp_nexthop_handler(struct id_table_index *, int, pm_hash_serial_t *, void *);
+extern int PT_map_index_entries_src_as_handler(struct id_table_index *, int, pm_hash_serial_t *, void *);
+extern int PT_map_index_entries_dst_as_handler(struct id_table_index *, int, pm_hash_serial_t *, void *);
+extern int PT_map_index_entries_peer_src_as_handler(struct id_table_index *, int, pm_hash_serial_t *, void *);
+extern int PT_map_index_entries_peer_dst_as_handler(struct id_table_index *, int, pm_hash_serial_t *, void *);
+extern int PT_map_index_entries_mpls_vpn_id_handler(struct id_table_index *, int, pm_hash_serial_t *, void *);
+extern int PT_map_index_entries_mpls_vpn_rd_handler(struct id_table_index *, int, pm_hash_serial_t *, void *);
+extern int PT_map_index_entries_mpls_pw_id_handler(struct id_table_index *, int, pm_hash_serial_t *, void *);
+extern int PT_map_index_entries_mpls_label_bottom_handler(struct id_table_index *, int, pm_hash_serial_t *, void *);
+extern int PT_map_index_entries_src_mac_handler(struct id_table_index *, int, pm_hash_serial_t *, void *);
+extern int PT_map_index_entries_dst_mac_handler(struct id_table_index *, int, pm_hash_serial_t *, void *);
+extern int PT_map_index_entries_vlan_id_handler(struct id_table_index *, int, pm_hash_serial_t *, void *);
+extern int PT_map_index_entries_cvlan_id_handler(struct id_table_index *, int, pm_hash_serial_t *, void *);
+extern int PT_map_index_entries_src_net_handler(struct id_table_index *, int, pm_hash_serial_t *, void *);
+extern int PT_map_index_entries_dst_net_handler(struct id_table_index *, int, pm_hash_serial_t *, void *);
+extern int PT_map_index_entries_is_multicast_handler(struct id_table_index *, int, pm_hash_serial_t *, void *);
+extern int PT_map_index_entries_fwd_status_handler(struct id_table_index *, int, pm_hash_serial_t *, void *);
+extern int PT_map_index_entries_null_handler(struct id_table_index *, int, pm_hash_serial_t *, void *);
+extern int PT_map_index_fdata_ip_handler(struct id_table_index *, int, int, struct id_entry *, pm_hash_serial_t *, void *); 
+extern int PT_map_index_fdata_input_handler(struct id_table_index *, int, int, struct id_entry *, pm_hash_serial_t *, void *); 
+extern int PT_map_index_fdata_output_handler(struct id_table_index *, int, int, struct id_entry *, pm_hash_serial_t *, void *); 
+extern int PT_map_index_fdata_bgp_nexthop_handler(struct id_table_index *, int, int, struct id_entry *, pm_hash_serial_t *, void *); 
+extern int PT_map_index_fdata_src_as_handler(struct id_table_index *, int, int, struct id_entry *, pm_hash_serial_t *, void *); 
+extern int PT_map_index_fdata_dst_as_handler(struct id_table_index *, int, int, struct id_entry *, pm_hash_serial_t *, void *); 
+extern int PT_map_index_fdata_peer_src_as_handler(struct id_table_index *, int, int, struct id_entry *, pm_hash_serial_t *, void *); 
+extern int PT_map_index_fdata_peer_dst_as_handler(struct id_table_index *, int, int, struct id_entry *, pm_hash_serial_t *, void *); 
+extern int PT_map_index_fdata_mpls_vpn_id_handler(struct id_table_index *, int, int, struct id_entry *, pm_hash_serial_t *, void *); 
+extern int PT_map_index_fdata_mpls_vpn_rd_handler(struct id_table_index *, int, int, struct id_entry *, pm_hash_serial_t *, void *); 
+extern int PT_map_index_fdata_mpls_pw_id_handler(struct id_table_index *, int, int, struct id_entry *, pm_hash_serial_t *, void *); 
+extern int PT_map_index_fdata_mpls_label_bottom_handler(struct id_table_index *, int, int, struct id_entry *, pm_hash_serial_t *, void *); 
+extern int PT_map_index_fdata_src_mac_handler(struct id_table_index *, int, int, struct id_entry *, pm_hash_serial_t *, void *); 
+extern int PT_map_index_fdata_dst_mac_handler(struct id_table_index *, int, int, struct id_entry *, pm_hash_serial_t *, void *); 
+extern int PT_map_index_fdata_vlan_id_handler(struct id_table_index *, int, int, struct id_entry *, pm_hash_serial_t *, void *);
+extern int PT_map_index_fdata_cvlan_id_handler(struct id_table_index *, int, int, struct id_entry *, pm_hash_serial_t *, void *);
+extern int PT_map_index_fdata_src_net_handler(struct id_table_index *, int, int, struct id_entry *, pm_hash_serial_t *, void *);
+extern int PT_map_index_fdata_dst_net_handler(struct id_table_index *, int, int, struct id_entry *, pm_hash_serial_t *, void *);
+extern int PT_map_index_fdata_is_multicast_handler(struct id_table_index *, int, int, struct id_entry *, pm_hash_serial_t *, void *);
+extern int PT_map_index_fdata_fwd_status_handler(struct id_table_index *, int, int, struct id_entry *, pm_hash_serial_t *, void *);
+extern int PT_map_index_fdata_null_handler(struct id_table_index *, int, int, struct id_entry *, pm_hash_serial_t *, void *);
 
 /* BPAS_*: bgp_peer_as_src map specific handlers */
 extern int BPAS_map_bgp_nexthop_handler(char *, struct id_entry *, char *, struct plugin_requests *, int);
@@ -156,6 +160,8 @@ extern int pretag_src_comms_handler(struct packet_ptrs *, void *, void *);
 extern int pretag_comms_handler(struct packet_ptrs *, void *, void *);
 extern int pretag_sample_type_handler(struct packet_ptrs *, void *, void *);
 extern int pretag_is_bi_flow_handler(struct packet_ptrs *, void *, void *);
+extern int pretag_is_nsel_handler(struct packet_ptrs *, void *, void *);
+extern int pretag_is_nel_handler(struct packet_ptrs *, void *, void *);
 extern int pretag_direction_handler(struct packet_ptrs *, void *, void *);
 extern int pretag_nat_event_handler(struct packet_ptrs *, void *, void *);
 extern int pretag_mpls_vpn_rd_handler(struct packet_ptrs *, void *, void *);
@@ -166,7 +172,7 @@ extern int pretag_vlan_id_handler(struct packet_ptrs *, void *, void *);
 extern int pretag_src_net_handler(struct packet_ptrs *, void *, void *);
 extern int pretag_dst_net_handler(struct packet_ptrs *, void *, void *);
 extern int pretag_is_multicast_handler(struct packet_ptrs *, void *, void *);
-extern int pretag_forwarding_status_handler(struct packet_ptrs *, void *, void *);
+extern int pretag_fwd_status_handler(struct packet_ptrs *, void *, void *);
 extern int pretag_cvlan_id_handler(struct packet_ptrs *, void *, void *);
 extern int pretag_set_tos_handler(struct packet_ptrs *, void *, void *);
 
@@ -213,6 +219,9 @@ extern void pm_pcap_interfaces_map_destroy(struct pm_pcap_interfaces *);
 extern void pm_pcap_interfaces_map_copy(struct pm_pcap_interfaces *, struct pm_pcap_interfaces *);
 extern u_int32_t pm_pcap_interfaces_map_lookup_ifname(struct pm_pcap_interfaces *, char *);
 extern struct pm_pcap_interface *pm_pcap_interfaces_map_getentry_by_ifname(struct pm_pcap_interfaces *, char *);
+extern struct pm_pcap_interface *pm_pcap_interfaces_map_getentry_by_idx(struct pm_pcap_interfaces *, int);
 extern char *pm_pcap_interfaces_map_getnext_ifname(struct pm_pcap_interfaces *, int *);
+extern char *pm_pcap_interfaces_map_get_ifname(struct pm_pcap_interfaces *, int);
+extern int pm_pcap_interfaces_map_get_direction(struct pm_pcap_interfaces *, int);
 
 #endif //PRETAG_HANDLERS_H
