@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2019 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2022 by Paolo Lucente
 */
 
 /*
@@ -22,7 +22,6 @@
 /* includes */
 #include "pmacct.h"
 #include "plugin_common.h"
-#include "addr.h"
 #include "pmacct-data.h"
 #include "plugin_hooks.h"
 #include "plugin_common.h"
@@ -39,7 +38,6 @@
 #include "ndpi/ndpi.h"
 #endif
 #include "net_aggr.h"
-#include "ports_aggr.h"
 #include "preprocess-internal.h"
 
 /* Global variables */
@@ -50,6 +48,7 @@ void print_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
 {
   struct pkt_data *data;
   struct ports_table pt;
+  struct protos_table prt, tost;
   unsigned char *pipebuf;
   struct pollfd pfd;
   struct insert_data idata;
@@ -60,7 +59,7 @@ void print_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   u_int32_t bufsz = ((struct channels_list_entry *)ptr)->bufsize;
   pid_t core_pid = ((struct channels_list_entry *)ptr)->core_pid;
   struct networks_file_data nfd;
-  char default_sep[] = ",", spacing_sep[2];
+  char spacing_sep[2];
 
   unsigned char *rgptr;
   int pollagain = TRUE;
@@ -76,6 +75,10 @@ void print_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   void *zmq_host = NULL;
 #endif
 
+#ifdef WITH_REDIS
+  struct p_redis_host redis_host;
+#endif
+
   memcpy(&config, cfgptr, sizeof(struct configuration));
   memcpy(&extras, &((struct channels_list_entry *)ptr)->extras, sizeof(struct extra_primitives));
   recollect_pipe_memory(ptr);
@@ -91,14 +94,26 @@ void print_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
 
   refresh_timeout = config.sql_refresh_time*1000;
 
+  if (config.print_output != PRINT_OUTPUT_JSON &&
+      config.print_output != PRINT_OUTPUT_AVRO_BIN &&
+      config.print_output != PRINT_OUTPUT_AVRO_JSON) {
+    if (config.tcpflags_encode_as_array ||
+	config.mpls_label_stack_encode_as_array ||
+	config.pretag_label_encode_as_map) {
+      Log(LOG_WARNING, "WARN ( %s/%s ): Complex data types (ie. pre_tag_label_encode_as_map) not supported by the selected print_output. Ignored.\n", config.name, config.type);
+    }
+  }
+
   if (config.print_output & PRINT_OUTPUT_JSON) {
+#ifdef WITH_JANSSON
     compose_json(config.what_to_count, config.what_to_count_2);
+#endif
   }
   else if ((config.print_output & PRINT_OUTPUT_AVRO_BIN) ||
 	   (config.print_output & PRINT_OUTPUT_AVRO_JSON)) {
 #ifdef WITH_AVRO
-    avro_acct_schema = avro_schema_build_acct_data(config.what_to_count, config.what_to_count_2);
-    if (config.avro_schema_file) write_avro_schema_to_file(config.avro_schema_file, avro_acct_schema);
+    p_avro_acct_schema = p_avro_schema_build_acct_data(config.what_to_count, config.what_to_count_2);
+    if (config.avro_schema_file) write_avro_schema_to_file(config.avro_schema_file, p_avro_acct_schema);
 #endif
   }
   else if (config.print_output & PRINT_OUTPUT_CUSTOM) {
@@ -121,11 +136,15 @@ void print_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   memset(&nt, 0, sizeof(nt));
   memset(&nc, 0, sizeof(nc));
   memset(&pt, 0, sizeof(pt));
+  memset(&prt, 0, sizeof(prt));
+  memset(&tost, 0, sizeof(tost));
 
   load_networks(config.networks_file, &nt, &nc);
   set_net_funcs(&nt);
 
   if (config.ports_file) load_ports(config.ports_file, &pt);
+  if (config.protos_file) load_protos(config.protos_file, &prt);
+  if (config.tos_file) load_tos(config.tos_file, &tost);
 
   memset(&idata, 0, sizeof(idata));
   memset(&prim_ptrs, 0, sizeof(prim_ptrs));
@@ -151,7 +170,7 @@ void print_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   /* setting number of entries in _protocols structure */
   while (_protocols[protocols_number].number != -1) protocols_number++;
 
-  if (!config.print_output_separator) config.print_output_separator = default_sep;
+  if (!config.print_output_separator) config.print_output_separator = DEFAULT_SEP;
   else {
     if (!strcmp(config.print_output_separator, "\\s")) {
       spacing_sep[0] = ' ';
@@ -166,15 +185,23 @@ void print_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
     }
   }
 
-  if (extras.off_pkt_vlen_hdr_primitives && config.print_output & PRINT_OUTPUT_FORMATTED) {
+  if ((extras.off_pkt_vlen_hdr_primitives || config.what_to_count_2 & COUNT_MPLS_LABEL_STACK) &&
+      config.print_output & PRINT_OUTPUT_FORMATTED) {
     Log(LOG_ERR, "ERROR ( %s/%s ): variable-length primitives, ie. label as_path std_comm etc., are not supported in print plugin with formatted output.\n", config.name, config.type);
     Log(LOG_ERR, "ERROR ( %s/%s ): Please switch to one of the other supported output formats (ie. csv, json, avro). Exiting ..\n", config.name, config.type);
     exit_gracefully(1);
   }
 
   print_output_stdout_header = TRUE;
-  if (!config.sql_table && !config.print_output_lock_file)
+
+  if (!config.sql_table && config.daemon) {
+    Log(LOG_ERR, "ERROR ( %s/%s ): no print_output_file defined and 'daemonize: true'. Output would be lost. Exiting ..\n", config.name, config.type);
+    exit_gracefully(1);
+  }
+
+  if (!config.sql_table && !config.print_output_lock_file) {
     Log(LOG_WARNING, "WARN ( %s/%s ): no print_output_file and no print_output_lock_file defined.\n", config.name, config.type);
+  }
 
   if (config.sql_table) {
     if (strchr(config.sql_table, '%') || strchr(config.sql_table, '$')) {
@@ -194,6 +221,15 @@ void print_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
     }
   }
 
+#ifdef WITH_REDIS
+  if (config.redis_host) {
+    char log_id[SHORTBUFLEN];
+
+    snprintf(log_id, sizeof(log_id), "%s/%s", config.name, config.type);
+    p_redis_init(&redis_host, log_id, p_redis_thread_produce_common_plugin_handler);
+  }
+#endif
+
   /* plugin main loop */
   for(;;) {
     poll_again:
@@ -222,7 +258,7 @@ void print_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
       int saved_qq_ptr;
 
       saved_qq_ptr = qq_ptr;
-      P_cache_handle_flush_event(&pt);
+      P_cache_handle_flush_event(&pt, &prt, &tost);
       if (saved_qq_ptr) print_output_stdout_header = FALSE;
     }
 
@@ -308,10 +344,18 @@ void print_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
 	  (*net_funcs[num])(&nt, &nc, &data->primitives, prim_ptrs.pbgp, &nfd);
 
 	if (config.ports_file) {
-          if (!pt.table[data->primitives.src_port]) data->primitives.src_port = 0;
-          if (!pt.table[data->primitives.dst_port]) data->primitives.dst_port = 0;
+          if (!pt.table[data->primitives.src_port]) data->primitives.src_port = PM_L4_PORT_OTHERS;
+          if (!pt.table[data->primitives.dst_port]) data->primitives.dst_port = PM_L4_PORT_OTHERS;
         }
 
+	if (config.protos_file) {
+          if (!prt.table[data->primitives.proto]) data->primitives.proto = PM_IP_PROTO_OTHERS;
+        }
+
+	if (config.tos_file) {
+	  if (!tost.table[data->primitives.tos]) data->primitives.tos = PM_IP_TOS_OTHERS;
+	}
+
         prim_ptrs.data = data;
         (*insert_func)(&prim_ptrs, &idata);
 
@@ -359,7 +403,7 @@ void P_cache_purge(struct chained_cache *queue[], int index, int safe_action)
   struct pkt_data dummy_data, elem_dummy_data;
   pid_t writer_pid = getpid();
 #ifdef WITH_AVRO
-  avro_file_writer_t avro_writer;
+  avro_file_writer_t p_avro_writer;
 #endif
 
   if (!index && !config.print_write_empty_file) {
@@ -431,10 +475,10 @@ void P_cache_purge(struct chained_cache *queue[], int index, int safe_action)
       close_output_file(f);
 
       if (config.print_output_file_append && !file_is_empty) {
-        ret = avro_file_writer_open(current_table, &avro_writer);
+        ret = avro_file_writer_open(current_table, &p_avro_writer);
       }
       else {
-        ret = avro_file_writer_create(current_table, avro_acct_schema, &avro_writer);
+        ret = avro_file_writer_create(current_table, p_avro_acct_schema, &p_avro_writer);
       }
 
       if (ret) {
@@ -474,10 +518,12 @@ void P_cache_purge(struct chained_cache *queue[], int index, int safe_action)
 	if ((config.print_output & PRINT_OUTPUT_CSV) || (config.print_output & PRINT_OUTPUT_FORMATTED))
 	  fprintf(f, "--START (%u)--\n", writer_pid);
 	else if (config.print_output & PRINT_OUTPUT_JSON) {
+#ifdef WITH_JANSSON
           void *json_obj;
 
 	  json_obj = compose_purge_init_json(config.name, writer_pid);
           if (json_obj) write_and_free_json(f, json_obj);
+#endif
 	}
       }
 
@@ -502,10 +548,12 @@ void P_cache_purge(struct chained_cache *queue[], int index, int safe_action)
       if ((config.print_output & PRINT_OUTPUT_CSV) || (config.print_output & PRINT_OUTPUT_FORMATTED))
         fprintf(stdout, "--START (%u)--\n", writer_pid);
       else if (config.print_output & PRINT_OUTPUT_JSON) {
+#ifdef WITH_JANSSON
         void *json_obj;
 
         json_obj = compose_purge_init_json(config.name, writer_pid);
         if (json_obj) write_and_free_json(stdout, json_obj);
+#endif
       }
     }
 
@@ -597,6 +645,7 @@ void P_cache_purge(struct chained_cache *queue[], int index, int safe_action)
             fprintf(f, "%-17s  ", empty_macaddress);
         }
         if (config.what_to_count & COUNT_VLAN) fprintf(f, "%-5u  ", data->vlan_id); 
+        if (config.what_to_count_2 & COUNT_OUT_VLAN) fprintf(f, "%-8u  ", data->out_vlan_id);
         if (config.what_to_count & COUNT_COS) fprintf(f, "%-2u  ", data->cos); 
         if (config.what_to_count & COUNT_ETHERTYPE) fprintf(f, "%-5x  ", data->etype); 
   #endif
@@ -621,7 +670,7 @@ void P_cache_purge(struct chained_cache *queue[], int index, int safe_action)
   	  else fprintf(f, "%-45s  ", empty_ip6);
         }
         if (config.what_to_count & COUNT_PEER_DST_IP) {
-          addr_to_str(ip_address, &pbgp->peer_dst_ip);
+          addr_to_str2(ip_address, &pbgp->peer_dst_ip, ft2af(queue[j]->flow_type));
 
           if (strlen(ip_address)) fprintf(f, "%-45s  ", ip_address);
           else fprintf(f, "%-45s  ", empty_ip6);
@@ -703,7 +752,7 @@ void P_cache_purge(struct chained_cache *queue[], int index, int safe_action)
   #endif
   
         if (config.what_to_count_2 & COUNT_SAMPLING_RATE) fprintf(f, "%-7u       ", data->sampling_rate);
-        if (config.what_to_count_2 & COUNT_SAMPLING_DIRECTION) fprintf(f, "%-1s                   ", data->sampling_direction);
+        if (config.what_to_count_2 & COUNT_SAMPLING_DIRECTION) fprintf(f, "%-1s                   ", sampling_direction_print(data->sampling_direction));
   
         if (config.what_to_count_2 & COUNT_POST_NAT_SRC_HOST) {
           addr_to_str(ip_address, &pnat->post_nat_src_ip);
@@ -722,6 +771,8 @@ void P_cache_purge(struct chained_cache *queue[], int index, int safe_action)
         if (config.what_to_count_2 & COUNT_POST_NAT_SRC_PORT) fprintf(f, "%-5u              ", pnat->post_nat_src_port);
         if (config.what_to_count_2 & COUNT_POST_NAT_DST_PORT) fprintf(f, "%-5u              ", pnat->post_nat_dst_port);
         if (config.what_to_count_2 & COUNT_NAT_EVENT) fprintf(f, "%-3u       ", pnat->nat_event);
+        if (config.what_to_count_2 & COUNT_FW_EVENT) fprintf(f, "%-3u      ", pnat->fw_event);
+        if (config.what_to_count_2 & COUNT_FWD_STATUS) fprintf(f, "%-3u        ", pnat->fwd_status);
   
         if (config.what_to_count_2 & COUNT_MPLS_LABEL_TOP) {
   	fprintf(f, "%-7u         ", pmpls->mpls_label_top);
@@ -729,9 +780,6 @@ void P_cache_purge(struct chained_cache *queue[], int index, int safe_action)
         if (config.what_to_count_2 & COUNT_MPLS_LABEL_BOTTOM) {
   	fprintf(f, "%-7u            ", pmpls->mpls_label_bottom);
         }
-        if (config.what_to_count_2 & COUNT_MPLS_STACK_DEPTH) {
-  	fprintf(f, "%-2u                ", pmpls->mpls_stack_depth);
-        }
 
         if (config.what_to_count_2 & COUNT_TUNNEL_SRC_MAC) {
           etheraddr_string(ptun->tunnel_eth_shost, src_mac);
@@ -772,6 +820,7 @@ void P_cache_purge(struct chained_cache *queue[], int index, int safe_action)
 	if (config.what_to_count_2 & COUNT_TUNNEL_IP_TOS) fprintf(f, "%-3u         ", ptun->tunnel_tos);
         if (config.what_to_count_2 & COUNT_TUNNEL_SRC_PORT) fprintf(f, "%-5u            ", ptun->tunnel_src_port);
         if (config.what_to_count_2 & COUNT_TUNNEL_DST_PORT) fprintf(f, "%-5u            ", ptun->tunnel_dst_port);
+	if (config.what_to_count_2 & COUNT_TUNNEL_TCPFLAGS) fprintf(f, "%-3u               ", queue[j]->tunnel_tcp_flags);
 
 	if (config.what_to_count_2 & COUNT_VXLAN) fprintf(f, "%-8u  ", ptun->tunnel_id);
   
@@ -805,6 +854,16 @@ void P_cache_purge(struct chained_cache *queue[], int index, int safe_action)
           fprintf(f, "%-30s ", tstamp_str);
         }
 
+        if (config.what_to_count_2 & COUNT_EXPORT_PROTO_TIME) {
+	  char tstamp_str[VERYSHORTBUFLEN];
+
+	  compose_timestamp(tstamp_str, VERYSHORTBUFLEN, &pnat->timestamp_export, TRUE,
+			    config.timestamps_since_epoch, config.timestamps_rfc3339,
+			    config.timestamps_utc);
+
+          fprintf(f, "%-30s ", tstamp_str);
+        }
+
         if (config.nfacctd_stitching && queue[j]->stitch) {
 	  char tstamp_str[VERYSHORTBUFLEN];
 
@@ -848,15 +907,9 @@ void P_cache_purge(struct chained_cache *queue[], int index, int safe_action)
         }
 
         if (!is_event) {
-  #if defined HAVE_64BIT_COUNTERS
           fprintf(f, "%-20" PRIu64 "  ", queue[j]->packet_counter);
           if (config.what_to_count & COUNT_FLOWS) fprintf(f, "%-20" PRIu64 "  ", queue[j]->flow_counter);
           fprintf(f, "%" PRIu64 "\n", queue[j]->bytes_counter);
-  #else
-          fprintf(f, "%-10lu  ", queue[j]->packet_counter);
-          if (config.what_to_count & COUNT_FLOWS) fprintf(f, "%-10lu  ", queue[j]->flow_counter);
-          fprintf(f, "%lu\n", queue[j]->bytes_counter);
-  #endif
         }
         else fprintf(f, "\n");
       }
@@ -883,6 +936,7 @@ void P_cache_purge(struct chained_cache *queue[], int index, int safe_action)
           fprintf(f, "%s%s", write_sep(sep, &count), dst_mac);
         }
         if (config.what_to_count & COUNT_VLAN) fprintf(f, "%s%u", write_sep(sep, &count), data->vlan_id); 
+        if (config.what_to_count_2 & COUNT_OUT_VLAN) fprintf(f, "%s%u", write_sep(sep, &count), data->out_vlan_id);
         if (config.what_to_count & COUNT_COS) fprintf(f, "%s%u", write_sep(sep, &count), data->cos); 
         if (config.what_to_count & COUNT_ETHERTYPE) fprintf(f, "%s%x", write_sep(sep, &count), data->etype); 
   #endif
@@ -1030,7 +1084,7 @@ void P_cache_purge(struct chained_cache *queue[], int index, int safe_action)
           fprintf(f, "%s%s", write_sep(sep, &count), ip_address);
         }
         if (config.what_to_count & COUNT_PEER_DST_IP) {
-          addr_to_str(ip_address, &pbgp->peer_dst_ip);
+          addr_to_str2(ip_address, &pbgp->peer_dst_ip, ft2af(queue[j]->flow_type));
           fprintf(f, "%s%s", write_sep(sep, &count), ip_address);
         }
   
@@ -1097,7 +1151,8 @@ void P_cache_purge(struct chained_cache *queue[], int index, int safe_action)
   #endif
   
         if (config.what_to_count_2 & COUNT_SAMPLING_RATE) fprintf(f, "%s%u", write_sep(sep, &count), data->sampling_rate);
-        if (config.what_to_count_2 & COUNT_SAMPLING_DIRECTION) fprintf(f, "%s%s", write_sep(sep, &count), data->sampling_direction);
+        if (config.what_to_count_2 & COUNT_SAMPLING_DIRECTION) fprintf(f, "%s%s", write_sep(sep, &count),
+								       sampling_direction_print(data->sampling_direction));
   
         if (config.what_to_count_2 & COUNT_POST_NAT_SRC_HOST) {
           addr_to_str(src_host, &pnat->post_nat_src_ip);
@@ -1110,10 +1165,25 @@ void P_cache_purge(struct chained_cache *queue[], int index, int safe_action)
         if (config.what_to_count_2 & COUNT_POST_NAT_SRC_PORT) fprintf(f, "%s%u", write_sep(sep, &count), pnat->post_nat_src_port);
         if (config.what_to_count_2 & COUNT_POST_NAT_DST_PORT) fprintf(f, "%s%u", write_sep(sep, &count), pnat->post_nat_dst_port);
         if (config.what_to_count_2 & COUNT_NAT_EVENT) fprintf(f, "%s%u", write_sep(sep, &count), pnat->nat_event);
+        if (config.what_to_count_2 & COUNT_FW_EVENT) fprintf(f, "%s%u", write_sep(sep, &count), pnat->fw_event);
+        if (config.what_to_count_2 & COUNT_FWD_STATUS) fprintf(f, "%s%u", write_sep(sep, &count), pnat->fwd_status);
   
         if (config.what_to_count_2 & COUNT_MPLS_LABEL_TOP) fprintf(f, "%s%u", write_sep(sep, &count), pmpls->mpls_label_top);
         if (config.what_to_count_2 & COUNT_MPLS_LABEL_BOTTOM) fprintf(f, "%s%u", write_sep(sep, &count), pmpls->mpls_label_bottom);
-        if (config.what_to_count_2 & COUNT_MPLS_STACK_DEPTH) fprintf(f, "%s%u", write_sep(sep, &count), pmpls->mpls_stack_depth);
+        if (config.what_to_count_2 & COUNT_MPLS_LABEL_STACK) {
+	  char label_stack[MAX_MPLS_LABEL_STACK];
+          char *label_stack_ptr = NULL;
+	  int label_stack_len = 0; 
+
+	  memset(label_stack, 0, MAX_MPLS_LABEL_STACK);
+
+          label_stack_len = vlen_prims_get(pvlen, COUNT_INT_MPLS_LABEL_STACK, &label_stack_ptr);
+          if (label_stack_ptr) {
+            mpls_label_stack_to_str(label_stack, sizeof(label_stack), (u_int32_t *)label_stack_ptr, label_stack_len);
+          }
+
+	  fprintf(f, "%s%s", write_sep(sep, &count), label_stack);
+	}
 
         if (config.what_to_count_2 & COUNT_TUNNEL_SRC_MAC) {
           etheraddr_string(ptun->tunnel_eth_shost, src_mac);
@@ -1143,6 +1213,7 @@ void P_cache_purge(struct chained_cache *queue[], int index, int safe_action)
 	if (config.what_to_count_2 & COUNT_TUNNEL_IP_TOS) fprintf(f, "%s%u", write_sep(sep, &count), ptun->tunnel_tos);
 	if (config.what_to_count_2 & COUNT_TUNNEL_SRC_PORT) fprintf(f, "%s%u", write_sep(sep, &count), ptun->tunnel_src_port);
         if (config.what_to_count_2 & COUNT_TUNNEL_DST_PORT) fprintf(f, "%s%u", write_sep(sep, &count), ptun->tunnel_dst_port);
+        if (config.what_to_count_2 & COUNT_TUNNEL_TCPFLAGS) fprintf(f, "%s%u", write_sep(sep, &count), queue[j]->tunnel_tcp_flags);
 
 	if (config.what_to_count_2 & COUNT_VXLAN) fprintf(f, "%s%u", write_sep(sep, &count), ptun->tunnel_id);
   
@@ -1176,6 +1247,16 @@ void P_cache_purge(struct chained_cache *queue[], int index, int safe_action)
           fprintf(f, "%s%s", write_sep(sep, &count), tstamp_str);
         }
 
+        if (config.what_to_count_2 & COUNT_EXPORT_PROTO_TIME) {
+	  char tstamp_str[VERYSHORTBUFLEN];
+
+	  compose_timestamp(tstamp_str, VERYSHORTBUFLEN, &pnat->timestamp_export, TRUE,
+			    config.timestamps_since_epoch, config.timestamps_rfc3339,
+			    config.timestamps_utc);
+
+          fprintf(f, "%s%s", write_sep(sep, &count), tstamp_str);
+        }
+
         if (config.nfacctd_stitching && queue[j]->stitch) {
 	  char tstamp_str[VERYSHORTBUFLEN];
 
@@ -1218,15 +1299,9 @@ void P_cache_purge(struct chained_cache *queue[], int index, int safe_action)
         }
   
         if (!is_event) {
-  #if defined HAVE_64BIT_COUNTERS
           fprintf(f, "%s%" PRIu64 "", write_sep(sep, &count), queue[j]->packet_counter);
           if (config.what_to_count & COUNT_FLOWS) fprintf(f, "%s%" PRIu64 "", write_sep(sep, &count), queue[j]->flow_counter);
           fprintf(f, "%s%" PRIu64 "\n", write_sep(sep, &count), queue[j]->bytes_counter);
-  #else
-          fprintf(f, "%s%lu", write_sep(sep, &count), queue[j]->packet_counter);
-          if (config.what_to_count & COUNT_FLOWS) fprintf(f, "%s%lu", write_sep(sep, &count), queue[j]->flow_counter);
-          fprintf(f, "%s%lu\n", write_sep(sep, &count), queue[j]->bytes_counter);
-  #endif
         }
         else fprintf(f, "\n");
       }
@@ -1243,30 +1318,30 @@ void P_cache_purge(struct chained_cache *queue[], int index, int safe_action)
 	       ((config.print_output & PRINT_OUTPUT_AVRO_BIN) ||
 	       (config.print_output & PRINT_OUTPUT_AVRO_JSON))) {
 #ifdef WITH_AVRO
-        avro_value_iface_t *avro_iface = avro_generic_class_from_schema(avro_acct_schema);
+        avro_value_iface_t *p_avro_iface = avro_generic_class_from_schema(p_avro_acct_schema);
 
-        avro_value_t avro_value = compose_avro_acct_data(config.what_to_count, config.what_to_count_2,
+        avro_value_t p_avro_value = compose_avro_acct_data(config.what_to_count, config.what_to_count_2,
 			 queue[j]->flow_type, &queue[j]->primitives, pbgp, pnat, pmpls, ptun, pcust,
 			 pvlen, queue[j]->bytes_counter, queue[j]->packet_counter, queue[j]->flow_counter,
-			 queue[j]->tcp_flags, NULL, queue[j]->stitch, avro_iface);
+			 queue[j]->tcp_flags, queue[j]->tunnel_tcp_flags, NULL, queue[j]->stitch, p_avro_iface);
 
         if (config.sql_table) {
 	  if (config.print_output & PRINT_OUTPUT_AVRO_BIN) {
-	    if (avro_file_writer_append_value(avro_writer, &avro_value)) {
+	    if (avro_file_writer_append_value(p_avro_writer, &p_avro_value)) {
 	      Log(LOG_ERR, "ERROR ( %s/%s ): P_cache_purge(): avro_file_writer_append_value() failed: %s\n", config.name, config.type, avro_strerror());
 	      exit_gracefully(1);
 	    }
           }
 	  else if (config.print_output & PRINT_OUTPUT_AVRO_JSON) {
-	    write_avro_json_record_to_file(f, avro_value);
+	    write_avro_json_record_to_file(f, p_avro_value);
 	  }
         }
         else {
-	  write_avro_json_record_to_file(f, avro_value);
+	  write_avro_json_record_to_file(f, p_avro_value);
         }
 
-        avro_value_iface_decref(avro_iface);
-        avro_value_decref(&avro_value);
+        avro_value_iface_decref(p_avro_iface);
+        avro_value_decref(&p_avro_value);
 #else
         if (config.debug) Log(LOG_DEBUG, "DEBUG ( %s/%s ): compose_avro_acct_data(): AVRO object not created due to missing --enable-avro\n", config.name, config.type);
 #endif
@@ -1275,8 +1350,8 @@ void P_cache_purge(struct chained_cache *queue[], int index, int safe_action)
       if (config.print_output & PRINT_OUTPUT_CUSTOM) {
 	custom_print_plugin.print(config.what_to_count, config.what_to_count_2, queue[j]->flow_type,
 				  &queue[j]->primitives, pbgp, pnat, pmpls, ptun, pcust, pvlen, queue[j]->bytes_counter,
-				  queue[j]->packet_counter, queue[j]->flow_counter, queue[j]->tcp_flags, NULL,
-				  queue[j]->stitch);
+				  queue[j]->packet_counter, queue[j]->flow_counter, queue[j]->tcp_flags,
+				  queue[j]->tunnel_tcp_flags, NULL, queue[j]->stitch);
       }
     }
   }
@@ -1287,17 +1362,19 @@ void P_cache_purge(struct chained_cache *queue[], int index, int safe_action)
     if ((config.print_output & PRINT_OUTPUT_CSV) || (config.print_output & PRINT_OUTPUT_FORMATTED))
       fprintf(f, "--END (%u)--\n", writer_pid);
     else if (config.print_output & PRINT_OUTPUT_JSON) {
+#ifdef WITH_JANSSON
       void *json_obj;
 
       json_obj = compose_purge_close_json(config.name, writer_pid, qn, saved_index, duration);
       if (json_obj) write_and_free_json(f, json_obj);
+#endif
     }
   }
     
   if (config.sql_table) {
 #ifdef WITH_AVRO
     if (config.print_output & PRINT_OUTPUT_AVRO_BIN) {
-      avro_file_writer_flush(avro_writer);
+      avro_file_writer_flush(p_avro_writer);
     }
 #endif
 
@@ -1326,7 +1403,7 @@ void P_cache_purge(struct chained_cache *queue[], int index, int safe_action)
 
 #ifdef WITH_AVRO
     if (config.print_output & PRINT_OUTPUT_AVRO_BIN) {
-      avro_file_writer_close(avro_writer);
+      avro_file_writer_close(p_avro_writer);
     }
 #endif
     else {
@@ -1361,7 +1438,15 @@ void P_write_stats_header_formatted(FILE *f, int is_event)
 #if defined (HAVE_L2)
   if (config.what_to_count & (COUNT_SRC_MAC|COUNT_SUM_MAC)) fprintf(f, "SRC_MAC            ");
   if (config.what_to_count & COUNT_DST_MAC) fprintf(f, "DST_MAC            ");
-  if (config.what_to_count & COUNT_VLAN) fprintf(f, "VLAN   ");
+  if (config.what_to_count & COUNT_VLAN) {
+    if (config.tmp_vlan_legacy) {
+      fprintf(f, "VLAN   ");
+    }
+    else {
+      fprintf(f, "IN_VLAN  ");
+    }
+  }
+  if (config.what_to_count_2 & COUNT_OUT_VLAN) fprintf(f, "OUT_VLAN  ");
   if (config.what_to_count & COUNT_COS) fprintf(f, "COS ");
   if (config.what_to_count & COUNT_ETHERTYPE) fprintf(f, "ETYPE  ");
 #endif
@@ -1415,9 +1500,10 @@ void P_write_stats_header_formatted(FILE *f, int is_event)
   if (config.what_to_count_2 & COUNT_POST_NAT_SRC_PORT) fprintf(f, "POST_NAT_SRC_PORT  ");
   if (config.what_to_count_2 & COUNT_POST_NAT_DST_PORT) fprintf(f, "POST_NAT_DST_PORT  ");
   if (config.what_to_count_2 & COUNT_NAT_EVENT) fprintf(f, "NAT_EVENT ");
+  if (config.what_to_count_2 & COUNT_FW_EVENT) fprintf(f, "FW_EVENT ");
+  if (config.what_to_count_2 & COUNT_FWD_STATUS) fprintf(f, "FWD_STATUS ");
   if (config.what_to_count_2 & COUNT_MPLS_LABEL_TOP) fprintf(f, "MPLS_LABEL_TOP  ");
   if (config.what_to_count_2 & COUNT_MPLS_LABEL_BOTTOM) fprintf(f, "MPLS_LABEL_BOTTOM  ");
-  if (config.what_to_count_2 & COUNT_MPLS_STACK_DEPTH) fprintf(f, "MPLS_STACK_DEPTH  ");
   if (config.what_to_count_2 & COUNT_TUNNEL_SRC_MAC) fprintf(f, "TUNNEL_SRC_MAC     ");
   if (config.what_to_count_2 & COUNT_TUNNEL_DST_MAC) fprintf(f, "TUNNEL_DST_MAC     "); 
   if (config.what_to_count_2 & COUNT_TUNNEL_SRC_HOST) fprintf(f, "TUNNEL_SRC_IP                                  ");
@@ -1426,10 +1512,12 @@ void P_write_stats_header_formatted(FILE *f, int is_event)
   if (config.what_to_count_2 & COUNT_TUNNEL_IP_TOS) fprintf(f, "TUNNEL_TOS  ");
   if (config.what_to_count_2 & COUNT_TUNNEL_SRC_PORT) fprintf(f, "TUNNEL_SRC_PORT  "); 
   if (config.what_to_count_2 & COUNT_TUNNEL_DST_PORT) fprintf(f, "TUNNEL_DST_PORT  "); 
+  if (config.what_to_count_2 & COUNT_TUNNEL_TCPFLAGS) fprintf(f, "TUNNEL_TCP_FLAGS  "); 
   if (config.what_to_count_2 & COUNT_VXLAN) fprintf(f, "VXLAN     ");
   if (config.what_to_count_2 & COUNT_TIMESTAMP_START) fprintf(f, "TIMESTAMP_START                ");
   if (config.what_to_count_2 & COUNT_TIMESTAMP_END) fprintf(f, "TIMESTAMP_END                  "); 
   if (config.what_to_count_2 & COUNT_TIMESTAMP_ARRIVAL) fprintf(f, "TIMESTAMP_ARRIVAL              ");
+  if (config.what_to_count_2 & COUNT_EXPORT_PROTO_TIME) fprintf(f, "TIMESTAMP_EXPORT               ");
   if (config.nfacctd_stitching) {
     fprintf(f, "TIMESTAMP_MIN                  ");
     fprintf(f, "TIMESTAMP_MAX                  "); 
@@ -1450,15 +1538,9 @@ void P_write_stats_header_formatted(FILE *f, int is_event)
   }
 
   if (!is_event) {
-#if defined HAVE_64BIT_COUNTERS
     fprintf(f, "PACKETS               ");
     if (config.what_to_count & COUNT_FLOWS) fprintf(f, "FLOWS                 ");
     fprintf(f, "BYTES\n");
-#else
-    fprintf(f, "PACKETS     ");
-    if (config.what_to_count & COUNT_FLOWS) fprintf(f, "FLOWS       ");
-    fprintf(f, "BYTES\n");
-#endif
   }
   else fprintf(f, "\n");
 }
@@ -1478,7 +1560,15 @@ void P_write_stats_header_csv(FILE *f, int is_event)
 #if defined HAVE_L2
   if (config.what_to_count & (COUNT_SRC_MAC|COUNT_SUM_MAC)) fprintf(f, "%sSRC_MAC", write_sep(sep, &count));
   if (config.what_to_count & COUNT_DST_MAC) fprintf(f, "%sDST_MAC", write_sep(sep, &count));
-  if (config.what_to_count & COUNT_VLAN) fprintf(f, "%sVLAN", write_sep(sep, &count));
+  if (config.what_to_count & COUNT_VLAN) {
+    if (config.tmp_vlan_legacy) {
+      fprintf(f, "%sVLAN", write_sep(sep, &count));
+    }
+    else {
+      fprintf(f, "%sIN_VLAN", write_sep(sep, &count));
+    }
+  }
+  if (config.what_to_count_2 & COUNT_OUT_VLAN) fprintf(f, "%sOUT_VLAN", write_sep(sep, &count));
   if (config.what_to_count & COUNT_COS) fprintf(f, "%sCOS", write_sep(sep, &count));
   if (config.what_to_count & COUNT_ETHERTYPE) fprintf(f, "%sETYPE", write_sep(sep, &count));
 #endif
@@ -1540,9 +1630,11 @@ void P_write_stats_header_csv(FILE *f, int is_event)
   if (config.what_to_count_2 & COUNT_POST_NAT_SRC_PORT) fprintf(f, "%sPOST_NAT_SRC_PORT", write_sep(sep, &count));
   if (config.what_to_count_2 & COUNT_POST_NAT_DST_PORT) fprintf(f, "%sPOST_NAT_DST_PORT", write_sep(sep, &count));
   if (config.what_to_count_2 & COUNT_NAT_EVENT) fprintf(f, "%sNAT_EVENT", write_sep(sep, &count));
+  if (config.what_to_count_2 & COUNT_FW_EVENT) fprintf(f, "%sFW_EVENT", write_sep(sep, &count));
+  if (config.what_to_count_2 & COUNT_FWD_STATUS) fprintf(f, "%sFWD_STATUS", write_sep(sep, &count));
   if (config.what_to_count_2 & COUNT_MPLS_LABEL_TOP) fprintf(f, "%sMPLS_LABEL_TOP", write_sep(sep, &count));
   if (config.what_to_count_2 & COUNT_MPLS_LABEL_BOTTOM) fprintf(f, "%sMPLS_LABEL_BOTTOM", write_sep(sep, &count));
-  if (config.what_to_count_2 & COUNT_MPLS_STACK_DEPTH) fprintf(f, "%sMPLS_STACK_DEPTH", write_sep(sep, &count));
+  if (config.what_to_count_2 & COUNT_MPLS_LABEL_STACK) fprintf(f, "%sMPLS_LABEL_STACK", write_sep(sep, &count));
   if (config.what_to_count_2 & COUNT_TUNNEL_SRC_MAC) fprintf(f, "%sTUNNEL_SRC_MAC", write_sep(sep, &count));
   if (config.what_to_count_2 & COUNT_TUNNEL_DST_MAC) fprintf(f, "%sTUNNEL_DST_MAC", write_sep(sep, &count));
   if (config.what_to_count_2 & COUNT_TUNNEL_SRC_HOST) fprintf(f, "%sTUNNEL_SRC_IP", write_sep(sep, &count));
@@ -1551,10 +1643,12 @@ void P_write_stats_header_csv(FILE *f, int is_event)
   if (config.what_to_count_2 & COUNT_TUNNEL_IP_TOS) fprintf(f, "%sTUNNEL_TOS", write_sep(sep, &count));
   if (config.what_to_count_2 & COUNT_TUNNEL_SRC_PORT) fprintf(f, "%sTUNNEL_SRC_PORT", write_sep(sep, &count));
   if (config.what_to_count_2 & COUNT_TUNNEL_DST_PORT) fprintf(f, "%sTUNNEL_DST_PORT", write_sep(sep, &count));
+  if (config.what_to_count_2 & COUNT_TUNNEL_TCPFLAGS) fprintf(f, "%sTUNNEL_TCP_FLAGS", write_sep(sep, &count));
   if (config.what_to_count_2 & COUNT_VXLAN) fprintf(f, "%sVXLAN", write_sep(sep, &count));
   if (config.what_to_count_2 & COUNT_TIMESTAMP_START) fprintf(f, "%sTIMESTAMP_START", write_sep(sep, &count));
   if (config.what_to_count_2 & COUNT_TIMESTAMP_END) fprintf(f, "%sTIMESTAMP_END", write_sep(sep, &count));
   if (config.what_to_count_2 & COUNT_TIMESTAMP_ARRIVAL) fprintf(f, "%sTIMESTAMP_ARRIVAL", write_sep(sep, &count));
+  if (config.what_to_count_2 & COUNT_EXPORT_PROTO_TIME) fprintf(f, "%sTIMESTAMP_EXPORT", write_sep(sep, &count));
   if (config.nfacctd_stitching) {
     fprintf(f, "%sTIMESTAMP_MIN", write_sep(sep, &count));
     fprintf(f, "%sTIMESTAMP_MAX", write_sep(sep, &count));
