@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2019 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2022 by Paolo Lucente
 */
 
 /*
@@ -20,6 +20,7 @@
 */
 
 #include "pmacct.h"
+#include "plugin_common.h"
 #include "sql_common.h"
 
 void AddToLRUTail(struct db_cache *Cursor)
@@ -142,14 +143,14 @@ void SwapChainedElems(struct db_cache *Cursor, struct db_cache *staleElem)
 
 void SQL_SetENV()
 {
-  u_char *ptrs[16];
+  char *ptrs[16];
   int count = 0, i;
 
   INIT_BUF(envbuf);
   memset(ptrs, 0, sizeof(ptrs));
 
   if (config.sql_db) {
-    strncat(envbuf.ptr, "SQL_DB=", envbuf.end-envbuf.ptr);
+    strncat(envbuf.ptr, "SQL_DB=", (envbuf.end-envbuf.ptr - 1));
     strncat(envbuf.ptr, config.sql_db, envbuf.end-envbuf.ptr);
     ptrs[count] = envbuf.ptr;
     envbuf.ptr += strlen(envbuf.ptr)+1;
@@ -181,7 +182,7 @@ void SQL_SetENV()
   }
 
   {
-    u_char *tmpptr;
+    char *tmpptr;
 
     strncat(envbuf.ptr, "SQL_REFRESH_TIME=", envbuf.end-envbuf.ptr);
     tmpptr = envbuf.ptr + strlen(envbuf.ptr);
@@ -192,7 +193,7 @@ void SQL_SetENV()
   }
 
   if (config.sampling_rate >= 1 || config.ext_sampling_rate >= 1) {
-    u_char *tmpptr;
+    char *tmpptr;
 
     strncat(envbuf.ptr, "SAMPLING_RATE=", envbuf.end-envbuf.ptr);
     tmpptr = envbuf.ptr + strlen(envbuf.ptr);
@@ -211,7 +212,7 @@ void SQL_SetENV()
   }
 
   {
-    u_char *tmpptr;
+    char *tmpptr;
 
     strncat(envbuf.ptr, "SQL_MAX_WRITERS=", envbuf.end-envbuf.ptr);
     tmpptr = envbuf.ptr + strlen(envbuf.ptr);
@@ -227,13 +228,13 @@ void SQL_SetENV()
 
 void SQL_SetENV_child(const struct insert_data *idata)
 {
-  u_char *ptrs[N_FUNCS];
+  char *ptrs[N_FUNCS];
   int count = 0, i;
 
   memset(ptrs, 0, sizeof(ptrs));
 
   {
-    u_char *tmpptr;
+    char *tmpptr;
 
     strncat(envbuf.ptr, "INSERT_QUERIES_NUMBER=", envbuf.end-envbuf.ptr);
     tmpptr = envbuf.ptr + strlen(envbuf.ptr);
@@ -244,7 +245,7 @@ void SQL_SetENV_child(const struct insert_data *idata)
   }
 
   {
-    u_char *tmpptr;
+    char *tmpptr;
 
     strncat(envbuf.ptr, "UPDATE_QUERIES_NUMBER=", envbuf.end-envbuf.ptr);
     tmpptr = envbuf.ptr + strlen(envbuf.ptr);
@@ -255,7 +256,7 @@ void SQL_SetENV_child(const struct insert_data *idata)
   }
 
   {
-    u_char *tmpptr;
+    char *tmpptr;
 
     strncat(envbuf.ptr, "ELAPSED_TIME=", envbuf.end-envbuf.ptr);
     tmpptr = envbuf.ptr + strlen(envbuf.ptr);
@@ -266,7 +267,7 @@ void SQL_SetENV_child(const struct insert_data *idata)
   }
 
   {
-    u_char *tmpptr;
+    char *tmpptr;
 
     strncat(envbuf.ptr, "TOTAL_ELEM_NUMBER=", envbuf.end-envbuf.ptr);
     tmpptr = envbuf.ptr + strlen(envbuf.ptr);
@@ -277,7 +278,7 @@ void SQL_SetENV_child(const struct insert_data *idata)
   }
 
   {
-    u_char *tmpptr;
+    char *tmpptr;
 
     strncat(envbuf.ptr, "EFFECTIVE_ELEM_NUMBER=", envbuf.end-envbuf.ptr);
     tmpptr = envbuf.ptr + strlen(envbuf.ptr);
@@ -288,7 +289,7 @@ void SQL_SetENV_child(const struct insert_data *idata)
   }
 
   if (idata->basetime) {
-    u_char *tmpptr;
+    char *tmpptr;
 
     strncat(envbuf.ptr, "SQL_HISTORY_BASETIME=", envbuf.end-envbuf.ptr);
     tmpptr = envbuf.ptr + strlen(envbuf.ptr);
@@ -299,7 +300,7 @@ void SQL_SetENV_child(const struct insert_data *idata)
   }
 
   if (idata->timeslot) {
-    u_char *tmpptr;
+    char *tmpptr;
 
     strncat(envbuf.ptr, "SQL_HISTORY_TIMESLOT=", envbuf.end-envbuf.ptr);
     tmpptr = envbuf.ptr + strlen(envbuf.ptr);
@@ -310,7 +311,7 @@ void SQL_SetENV_child(const struct insert_data *idata)
   }
 
   if (idata->dyn_table) {
-    u_char *tmpptr;
+    char *tmpptr;
 
     strncat(envbuf.ptr, "EFFECTIVE_SQL_TABLE=", envbuf.end-envbuf.ptr);
     tmpptr = envbuf.ptr + strlen(envbuf.ptr);
@@ -321,7 +322,7 @@ void SQL_SetENV_child(const struct insert_data *idata)
   }
 
   {
-    u_char *tmpptr;
+    char *tmpptr;
 
     strncat(envbuf.ptr, "SQL_ACTIVE_WRITERS=", envbuf.end-envbuf.ptr);
     tmpptr = envbuf.ptr + strlen(envbuf.ptr);
