@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2021 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2022 by Paolo Lucente
 */
 
 /*
@@ -20,7 +20,6 @@
 */
 
 #include "pmacct.h"
-#include "addr.h"
 #ifdef WITH_KAFKA
 #include "kafka_common.h"
 #endif
@@ -86,7 +85,7 @@ void tee_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   signal(SIGPIPE, SIG_IGN);
   signal(SIGCHLD, SIG_IGN);
 
-  if (config.tee_transparent && getuid() != 0) {
+  if (config.tee_transparent && getuid() != 0 && !config.pmacctd_nonroot) {
     Log(LOG_ERR, "ERROR ( %s/%s ): Transparent mode requires super-user permissions. Exiting ...\n", config.name, config.type);
     exit_gracefully(1);
   }
@@ -180,7 +179,7 @@ void tee_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
     }
 
     if (reload_log) {
-      reload_logs();
+      reload_logs(NULL);
       reload_log = FALSE;
     }
 
@@ -700,6 +699,9 @@ void Tee_init_zmq_host(struct p_zmq_host *zmq_host, char *zmq_address, u_int32_t
 int Tee_prepare_sock(struct sockaddr *addr, socklen_t len, char *src_ip, u_int16_t src_port, int transparent, int pipe_size)
 {
   int s, ret = 0;
+#if defined BSD
+  int hincl = TRUE;
+#endif
 
   if (!transparent) {
     struct host_addr source_ip;
@@ -734,7 +736,6 @@ int Tee_prepare_sock(struct sockaddr *addr, socklen_t len, char *src_ip, u_int16
       exit_gracefully(1);
     }
 
-
 #if defined BSD
     setsockopt(s, IPPROTO_IP, IP_HDRINCL, &hincl, (socklen_t) sizeof(hincl));
 #endif
@@ -924,9 +925,8 @@ void Tee_select_templates(unsigned char *pkt, int pkt_len, int nfv, unsigned cha
   while (term1 > term2) {
     int fset_id = ntohs(hdr_flowset->flow_id);
     int fset_len = ntohs(hdr_flowset->flow_len);
-    int fset_hdr_len = sizeof(struct data_hdr_v9);
 
-    if (!fset_len || (fset_hdr_len + fset_len) > pkt_len) break;
+    if (!fset_len || fset_len > pkt_len) break;
 
     /* if template, copy over */
     if (((nfv == 9) && (fset_id == 0 || fset_id == 1)) ||
