@@ -1,6 +1,6 @@
 /*  
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2019 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2021 by Paolo Lucente
 */
 
 /*
@@ -44,4 +44,12 @@ extern void telemetry_log_global_stats(struct telemetry_data *);
 extern void telemetry_init_zmq_host(void *, int *);
 #endif
 
+#ifdef WITH_KAFKA
+extern void telemetry_init_kafka_host(void *);
+#endif
+
+#ifdef WITH_UNYTE_UDP_NOTIF
+extern int create_socket_unyte_udp_notif(struct telemetry_data *, char *, char *);
+#endif
+
 #endif //TELEMETRY_UTIL_H
